/*
 * AlienInterfaceSurface
 *
 * Front end user interface for communicating with aliens.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "Ship.h"
#include "View.h"
#include "Coords.h"
#include "Artifact.h"
#include "Star.h"
#include "SoundSystem.h"
#include "ScreenSurface.h"
#include "AlienInterface.h"
#include "AlienInterfaceSurface.h"

using namespace std;

bool AlienInterfaceSurface::_speaking = false;

/*
 * Constructor.  Must be run before blit().
 */
AlienInterfaceSurface::AlienInterfaceSurface() {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);

	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY, 0x00000000);

	_translucency_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);
	SDL_SetAlpha(_translucency_surface,SDL_SRCALPHA,127);
	boxColor(_translucency_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x00007FFF);

	_rect.x=0;
	_rect.y=0;
	_rect.w=SCREEN_CENTER*2;
	_rect.h=SCREEN_HEIGHT;

	_frame_count=0;
}

/*
 * Destructor.  Frees surfaces.
 */
AlienInterfaceSurface::~AlienInterfaceSurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_translucency_surface);
}

/*
 * Blit the Alien Interface to the screen surface.
 */
void AlienInterfaceSurface::blit() {
	if (AlienInterface::isNewPage()) {
		_frame_count=0;
		_message = AlienInterface::getMessage();
		_options = AlienInterface::getOptions();
	}
	SDL_BlitSurface(_translucency_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	boxColor(_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x000000FF);

	Ship *my_ship = Ship::getMyShip();
	int alien_type = (*my_ship).getShipStar().getAlienType();
	SoundSystem::pauseMusic();
	SoundSystem::playSound(alien_type);

	blitMessage();
	blitOptions();
	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	_frame_count++;
}

/*
 * Blit the message to the Alien Interface Surface.
 */
void AlienInterfaceSurface::blitMessage() {
	int height = AlienInterface::getMessageVPos();
	int count=_frame_count*2;
	int w,h;

	Ship *my_ship = Ship::getMyShip();
	int alien_type = (*my_ship).getShipStar().getAlienType();

	TTF_Font *font=NULL;
	SDL_Color textColor = {255,255,255};

	switch (alien_type) {
		case 0:
			font = Font::getTTFFont("Burtinomatic.ttf");
			break;
		case 1:
			font = Font::getTTFFont("Incognitype.ttf");
			break;
		case 2:
			font = Font::getTTFFont("HookedUp.ttf");
			break;
	}

	if (font==NULL) {
		//Alien Type not determined yet.
		return;
	}

	if (height==-1) {
		TTF_SizeText( font, "X", &w,&h );
		height=SCREEN_CENTER-(h+2)*(_message.size())/2;
	}
	int char_count=0;

	SDL_Surface *message;

	_speaking=false;

	for (int i=0;i<(int)_message.size();i++) {
		string this_line = _message[i];
		int mes_len = this_line.length();
		TTF_SizeText( font, this_line.c_str(), &w,&h );

		SDL_Rect message_rect;
		message_rect.x = (SCREEN_CENTER*2-w) / 2;
		message_rect.y = height+i*(h+2);

		if (count<char_count+mes_len) {
			this_line = this_line.substr(0,count-char_count);
			message = TTF_RenderText_Blended( font, this_line.c_str(), textColor );
			SDL_BlitSurface(message,NULL,_surface,&message_rect);
			SDL_FreeSurface(message);
			_speaking=true;
			break;
		} else {
			message = TTF_RenderText_Blended( font, this_line.c_str(), textColor );
			SDL_BlitSurface(message,NULL,_surface,&message_rect);
			SDL_FreeSurface(message);
			char_count+=mes_len;
		}
	}

}

/*
 * Blit the options to the Alien Interface Surface.
 */
void AlienInterfaceSurface::blitOptions() {
	if (_options.size()==0)
		return;

	int selected = AlienInterface::getSelectedOption();
	int height = AlienInterface::getOptionsVPos();
	Font::setFont("9x18B.fnt");

	int offset=0;
	for (int i=0;i<(int)_options.size();i++) {
		string this_line = _options[i];

		Sint32 line_color=0xFFFFFFFF;

		if ((this_line!="")&&(this_line.substr(0,1)=="@")) {
			//We're rendering a color;
			string alien_type = this_line.substr(1,1);
			if (alien_type=="0") {
				line_color=0xBF007FFF;
			} else if (alien_type=="1") {
				line_color=0x007FBFFF;
			} else if (alien_type=="2") {
				line_color=0x7FBF00FF;
			}

			this_line = this_line.substr(2,this_line.length()-2);
		}

		if(i==selected)
			this_line="> "+this_line;
		else
			this_line="  "+this_line;

		size_t found=this_line.substr(0,46).rfind(" ");
		if ((this_line.length()>46)&&(found!=string::npos)) {
			string remainder = "   "+this_line.substr(found,this_line.length());
			this_line = this_line.substr(0,found);
			stringColor(_surface,20,height+20*(i+offset),this_line.c_str(),line_color);
			offset++;
			stringColor(_surface,20,height+20*(i+offset),remainder.c_str(),line_color);
		} else {
			stringColor(_surface,20,height+20*(i+offset),this_line.c_str(),line_color);
		}
	}

	Font::setFont("default");

}

/*
 * Checks to see if Alien is speaking.  Used by OverviewSurface's alien animation.
 */
bool AlienInterfaceSurface::isSpeaking() {
	return _speaking;
}
