/*
 * Artifact
 *
 * Class representing alien artifacts, with helper functions.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>

#include "Artifact.h"
#include "Coords.h"
#include "Star.h"
#include "Messages.h"
#include "StringParser.h"
#include "Ship.h"
#include "View.h"
#include "FrameTimer.h"
#include "Information.h"
#include "SoundSystem.h"
#include "MenuInterface.h"
#include "AlienInterface.h"
#include "OverviewSurface.h"
#include "ArtifactNames.h"

using namespace std;

std::vector<Artifact> Artifact::_found_artifacts;
bool Artifact::_updated = true;

Star Artifact::_prize_star;
int Artifact::_prize_location[7];
Artifact Artifact::_prize_artifact;

string Artifact::_artifact_message;

bool Artifact::_between_games=true;

string Artifact::_alien_types[3] = {"Ancient","Industrialist","Futurist"};

/*
 * Default Constructor
 */
Artifact::Artifact() {
	_star_number=-1;
	_updated=true;
}

/*
 * Copy Constructor
 */
Artifact::Artifact(const Artifact &a) {
	_star_number=a._star_number;
}

/*
 * Create an artifact with star_number.
 */
Artifact::Artifact(int star_number) {
	_star_number=star_number;
}

/*
 * Eqality operator.
 */
bool Artifact::operator==(const Artifact &a) {
	return _star_number==a._star_number;
}

/*
 * Inequality operator.
 */
bool Artifact::operator!=(const Artifact &a) {
	return _star_number!=a._star_number;
}

/*
 * A null artifact, returned in some instances and used with the equality comparitor.
 */
Artifact Artifact::nullArtifact() {
	return Artifact(-1);
}

/*
 * Returns a unique artifact name based on the artifact.
 */
string Artifact::getArtifactName() {
	stringstream name;
	int artifact_number = getArtifactNumber();

	int word_part[3];

	word_part[0]=(artifact_number/20449)%143;
	word_part[1]=(artifact_number/143)%143;
	word_part[2]=artifact_number%143;


	switch(getArtifactType()) {
		case 0:
			//Ancient
			name << "The " << artifact_name_part_1a[word_part[0]] << " " << artifact_name_part_2a[word_part[1]] << " of " << artifact_name_part_3a[word_part[2]];
			break;
		case 1:
			//Industrialist
			name << "The " << artifact_name_part_1i[word_part[0]] << " " << artifact_name_part_2i[word_part[1]] << " of " << artifact_name_part_3i[word_part[2]];
			break;
		case 2:
			//Futurist
			name << "The " << artifact_name_part_1f[word_part[0]] << " " << artifact_name_part_2f[word_part[1]] << " of " << artifact_name_part_3f[word_part[2]];
			break;
	}

	return name.str();
}

/*
 * Get an artifact number based on the star_number.  These numbers are not unique..
 * There is Artifact #1 for each of Ancient, Industrialist, and Futurist.
 */
int Artifact::getArtifactNumber() {
	int artifact_number = _star_number/27;

	long long rand;

	//Scramble the number with modulo arthmetic so that neighbouring artifacts aren't similar.
	rand = (((long long)(artifact_number))*1687489)%2893519;

	return (int)rand;

}

/*
 * Return the artifact cash value, based on the alien type.
 * low value     ($) = 10
 * medium value ($$) = 40
 * high value  ($$$) = 70
 * Times three if artifact type and alien type match.
 */
int Artifact::getArtifactCashValue(int alien_type) {
	int cash_value=10+30*getArtifactValue();

	if (alien_type==getArtifactType())
		cash_value*=3;

	return cash_value;
}

/*
 * Return the artifact type:
 * 0 = Ancient
 * 1 = Industrial
 * 2 = Futurist
 */
int Artifact::getArtifactType() {
	return (_star_number%27)/9;
}

/*
 * Returns the artifact value:
 * 70% chance of low ($) value
 * 25% chance of medium ($$) value
 * 5% chance of high ($$$) value
 */
int Artifact::getArtifactValue() {
	int artifact_value, artifact_number;
	artifact_number = getArtifactNumber();
	if (artifact_number%20==19) {
		artifact_value=2;
	} else if (artifact_number%10>6) {
		artifact_value=1;
	} else {
		artifact_value=0;
	}
	return artifact_value;
}

/*
 * Returns a vector of artifacts found by the player (and presently in the cargo hold).
 */
vector<Artifact> Artifact::getFoundArtifacts() {
	return _found_artifacts;
}

/*
 * Adds an artifact to the list of found artifacts (presently in the cargo hold).
 */
void Artifact::addFoundArtifact(Artifact a) {
	_found_artifacts.push_back(a);
	_updated=true;
}

/*
 * Checks to see if there are new artifacts found.
 */
bool Artifact::isUpdated() {
	if (_updated) {
		_updated=false;
		return true;
	}
	return false;
}

/*
 * Set the location of the Prize Artifact.
 */
void Artifact::initPrizeLocation() {
	_between_games=false;
	_prize_star = Star::getRandomStar(_prize_location,true);
	_prize_artifact = _prize_star.getArtifact();
}

/*
 * Returns the Prize Artifact.
 */
Artifact Artifact::getPrizeArtifact() {
	return _prize_artifact;
}

/*
 * Returns the star where the prize artifact is located.
 */
Star Artifact::getPrizeStar() {
	return _prize_star;
}

/*
 * Returns the sector location where the prize is located (as an int[7], e.g. {5,5,5,1,2,3,4})
 */
int *Artifact::getPrizeLocation() {
	return _prize_location;
}

/*
 * Returns true if this artifact is the prize artifact.
 */
bool Artifact::isPrizeArtifact() {
	if (_star_number<0)
		return false;
	return (_star_number==_prize_artifact._star_number);
}

/*
 * Returns the most interesting artifact to an alien.
 * Will be the highest value artifact of the same type.
 * Returns null artifact if no artifacts are of interest.
 */
Artifact Artifact::getInterestingArtifact(const int &alien_type) {
	Artifact interesting_artifact = nullArtifact();
	int max_value=-1;

	for (int i=0;i<(int)_found_artifacts.size();i++) {
		int value =_found_artifacts[i].getArtifactValue();
		int type = _found_artifacts[i].getArtifactType();
		if ((value>max_value)&&(type==alien_type)) {
			max_value=value;
			interesting_artifact = _found_artifacts[i];
		}
 	}

	return interesting_artifact;
}

/*
 * Remove this artifact from the list of found artifacts (presently in the cargo hold).
 */
void Artifact::removeFromFoundArtifacts() {
	for (int i=0;i<(int)_found_artifacts.size();i++) {
		if (_star_number==_found_artifacts[i]._star_number) {
			std::vector<Artifact>::iterator pos = _found_artifacts.begin()+i;
			_found_artifacts.erase(pos,pos+1);
			break;
		}
	}
}

/*
 * Removes all items from the cargo hold, in preparation for a game reset.
 */
void Artifact::reset() {
	_found_artifacts.erase(_found_artifacts.begin(),_found_artifacts.end());
}

/*
 * Used by the Gameserver to initialize multiplayer artifact location.
 * Creates an ARTI message to be broadcast to all players.
 */
void Artifact::createArtifactMessage() {
	Star::removeStars(1);
	Artifact::initPrizeLocation();
	stringstream ss;
	ss << Artifact::getPrizeStar().getStarNumber();
	Coords artifact_position = Artifact::getPrizeStar().getPosition();
	ss << " " << artifact_position.getX();
	ss << " " << artifact_position.getY();
	int *location = Artifact::getPrizeLocation();
	for (int i=0;i<7;i++)
		ss << " "<<location[i];

	_artifact_message = ss.str();
}

/*
 * Returns the artifact message. Used by GameServer to tell clients where the multiplayer artifact is.
 */
string Artifact::getArtifactMessage() {
	return _artifact_message;
}

/*
 * Used by the Game client in a multiplayer game to handle the server ARTI message and set the
 * multiplayer game artifact location.  Acts as a trigger to reset all other game interfaces and
 * backend values.
 */
void Artifact::handleArtiMessage(string message) {
	//Reset ALL (ARTI signifies start of new game)
	_between_games=false;

	View::reset();

	Ship *my_ship = Ship::getMyShip();
	(*my_ship).reset();
	Artifact::reset();
	Star::reset();


	View::setUpdateOverride();
	Information::reset();
	MenuInterface::reset();
	AlienInterface::reset();
	//Messages::reset();
	OverviewSurface::reset();

	Ship::initShipLocation();

	(*my_ship).queueShipDataMessage();

	FrameTimer::resetTimer();

	//In case players visit alients between games, shut off their sound and restart
	//the music
	SoundSystem::stopSound(-1);
	SoundSystem::resumeMusic();

	_prize_artifact._star_number = StringParser::skimInt(message);
	float x = StringParser::skimFloat(message);
	float y = StringParser::skimFloat(message);
	Coords artifact_position = Coords(x,y);
	_prize_star.setPosition(artifact_position);
	for (int i=0;i<7;i++) {
		_prize_location[i] = StringParser::skimInt(message);
	}
	_prize_star.setStarNumber(_prize_artifact._star_number);

	Messages::addMessageToQueue("You are searching for '"+_prize_artifact.getArtifactName()+"', a prized "+_alien_types[_prize_artifact.getArtifactType()]+" artifact!");

}

/*
 * For multiplayer games, tests if we are between a game victory and a new game
 * (i.e. "Server will restart in X seconds")
 */
bool Artifact::isBetweenGames() {
	return _between_games;
}

/*
 * Sets a flag to indicate if the multiplayer client is between games
 * Player can still play (perhaps to chat, or zoom around the screen and gloat)
 * but can't win.
 */
void Artifact::setBetweenGames(bool between_games) {
	_between_games = between_games;
}

/*
 * Returns a string vector of the three alien types.  Used by other parts of the program
 * to determine alien race names.
 */
string *Artifact::getAlienTypes() {
	return _alien_types;
}
