/*
 * Coords
 *
 * Class to represent an (x,y) vector, with helper functions.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <sstream>
#include "SDL/SDL.h"

#include "Coords.h"
#include "Definitions.h"

using namespace std;

string Coords::_coarse_angle_names[4] = {"east","south","west","north"};
string Coords::_fine_angle_names[8] = {"east","southeast","south","southwest",
                                       "west","northwest","north","northeast"};
float Coords::_coarse_angle_values[4] = {PI/4.0,3.0*PI/4.0,5.0*PI/4.0,7.0*PI/4.0};
float Coords::_fine_angle_values[8] = {PI/8.0,3.0*PI/8.0,5.0*PI/8.0,7.0*PI/8.0,
                                       9.0*PI/8.0,11.0*PI/8.0,13.0*PI/8.0,15.0*PI/8.0};

/*
 * Constructor.  Initializes Coords to (0.0,0.0).
 */
Coords::Coords() {
	_x=_y=0.0;
}

/*
 * Copy Constructor
 */
Coords::Coords(const Coords &c) {
	_x=c._x;
	_y=c._y;
}

/*
 * Constructor.  Initializes Coords to (x,y).
 */
Coords::Coords(const float x, const float y) {
	_x=x;
	_y=y;
}

/*
 * Constructor.  Initializes Coords to polar coordinates (r,theta) if polar is true.
 */
Coords::Coords(const float r, const float theta, const bool polar) {
	if (polar) {
		_x=r*cos(theta);
		_y=r*sin(theta);
	} else {
		_x=r;
		_y=theta;
	}
}

/*
 * Equality operator.
 */
bool Coords::operator== (const Coords &b) {
	return ((_x==b._x)&&(_y==b._y));
}

/*
 * Inequality operator.
 */
bool Coords::operator!= (const Coords &b) {
	return (!((_x==b._x)&&(_y==b._y)));
}

/*
 * Addition operator. Performs vector addition.
 */
Coords Coords::operator+ (const Coords &b) {
	return Coords(_x+b._x,_y+b._y);
}

/*
 * Substraction operator.  Performs vector subtraction.
 */
Coords Coords::operator- (const Coords &b) {
	return Coords(_x-b._x,_y-b._y);
}

/*
 * Multiplication operator. Performs scalar multiplication of vector
 * e.g. (1,2)*3 = (3,6)
 * order cannot be reversed.. scalar must always go last.
 */
Coords Coords::operator* (const float &d) {
	return Coords(_x*d,_y*d);
}

/*
 * Division operator. Performs scalar division of vector
 * e.g. (3,6)/3 = (1,2)
 * order cannot be reversed.. scalar must always go last.
 */
Coords Coords::operator/(const float &d) {
	return Coords(_x/d,_y/d);
}

/*
 * Addition equals operator.
 */
void Coords::operator+= (const Coords &b) {
	_x+=b._x;
	_y+=b._y;
}

/*
 * Subtraction equals operator.
 */
void Coords::operator-= (const Coords &b) {
	_x-=b._x;
	_y-=b._y;
}

/*
 * Returns the x co-ordinate of Coords
 */
float Coords::getX() {
	return _x;
}

/*
 * Returns the y co-ordinate of Coords
 */
float Coords::getY() {
	return _y;
}

/*
 * Returns the angle theta the (x,y) vector takes in radians.
 * Note that radians starts at 0 at (1,0) (right), and increases
 * CLOCKWISE, because y increases in the down direction on computer screens
 * Returns a value between 0 and 2*PI radians.
 */
float Coords::getAngle() {
	float angle=0;

	if (_x==0.0) {
		if (_y>0) {
			angle=PI/2.0;
		} else {
			angle=3.0*PI/2.0;
		}
	} else {
		angle = atan(_y/_x);
		if (_x>0) {
			if(_y<0)
				angle+=2.0*PI;
		} else { //delta.x<0
			angle+=PI;
		}
	}

	return angle;
}

/*
 * Returns the length of the Coords vector
 */
float Coords::getLength() {
	return sqrt(pow(_x,2)+pow(_y,2));
}

/*
 * Assuming the vector is centered at the middle of the screen, and given a scale
 * and a shrink value, return integer x_pos,y_pos values suitable for displaying on the screen.
 * shrink = 1 for large galaxy surface (480*480)
 * shrink = 3 for minimap (160*160)
 */
void Coords::getIntValues(int *x_pos, int *y_pos, float scale, int shrink) {
	*x_pos = SCREEN_CENTER/shrink+(int)((_x)*2.0*SCREEN_CENTER/scale)/shrink;
	*y_pos = SCREEN_CENTER+(int)((_y)*2.0*SCREEN_CENTER/scale)/shrink;
}

/*
 * Assuming the vector is centered at the middle of the screen, and given a scale
 * return integer x_pos,y_pos values suitable for displaying on the Galaxy Surface screen.
 */
void Coords::getIntValues(int *x_pos, int *y_pos, float scale) {
	*x_pos = SCREEN_CENTER+(int)((_x)*2.0*SCREEN_CENTER/scale);
	*y_pos = SCREEN_CENTER+(int)((_y)*2.0*SCREEN_CENTER/scale);
}

/*
 * Assuming the vector is centered at the middle of the screen, and given a scale
 * and a shrink value, return integer x_pos,y_pos values suitable for displaying on the screen.
 * shrink = 1 for large galaxy surface (480*480)
 * shrink = 3 for minimap (160*160)
 * (uses Sint16 instead of int)
 */
void Coords::getIntValues(Sint16 *x_pos, Sint16 *y_pos, float scale, int shrink) {
	*x_pos = SCREEN_CENTER/shrink+(Sint16)((_x)*2.0*SCREEN_CENTER/scale)/shrink;
	*y_pos = SCREEN_CENTER/shrink+(Sint16)((_y)*2.0*SCREEN_CENTER/scale)/shrink;
}

/*
 * Assuming the vector is centered at the middle of the screen, and given a scale
 * return integer x_pos,y_pos values suitable for displaying on the Galaxy Surface screen.
 * (uses Sint16 instead of int)
 */
void Coords::getIntValues(Sint16 *x_pos, Sint16 *y_pos, float scale) {
	*x_pos = SCREEN_CENTER+(Sint16)((_x)*2.0*SCREEN_CENTER/scale);
	*y_pos = SCREEN_CENTER+(Sint16)((_y)*2.0*SCREEN_CENTER/scale);
}

/*
 * Return the value of the Coarse Angle name (e.g. "North, West") based on the angle given.
 */
string Coords::getCoarseAngleName(float angle) {
	while (angle>2.0*PI)
		angle-=2.0*PI;
	for (int i=0;i<4;i++) {
		if (angle<_coarse_angle_values[i]) {
			return _coarse_angle_names[i];
		}
	}

	return _coarse_angle_names[0];
}

/*
 * Return the value of the Fine Angle name (e.g. "North-east") based on the angle given.
 */
string Coords::getFineAngleName(float angle) {
	while (angle>2.0*PI)
		angle-=2.0*PI;
	for (int i=0;i<8;i++) {
		if (angle<_fine_angle_values[i]) {
			return _fine_angle_names[i];
		}
	}

	return _fine_angle_names[0];
}


/*
 * Round up an angle to the next coarse value
 */
float Coords::roundAngleUpCoarse(float angle) {
	while (angle>2.0*PI)
		angle-=2.0*PI;

	for (int i=0;i<4;i++) {
		if (angle<_coarse_angle_values[i]) {
			return _coarse_angle_values[i];
		}
	}

	return _coarse_angle_values[0];
}

/*
 * Round down an angle to the next coarse value
 */
float Coords::roundAngleDownCoarse(float angle) {
	while (angle>2.0*PI)
		angle-=2.0*PI;

	for (int i=3;i>=0;i--) {
		if (angle>_coarse_angle_values[i]) {
			return _coarse_angle_values[i];
		}
	}

	return _coarse_angle_values[3];
}

/*
 * Round up an angle to the next fine value
 */
float Coords::roundAngleUpFine(float angle) {
	while (angle>2.0*PI)
		angle-=2.0*PI;

	for (int i=0;i<8;i++) {
		if (angle<_fine_angle_values[i]) {
			return _fine_angle_values[i];
		}
	}

	return _fine_angle_values[0];
}

/*
 * Round down an angle to the next fine value
 */
float Coords::roundAngleDownFine(float angle) {
	while (angle>2.0*PI)
		angle-=2.0*PI;

	for (int i=7;i>=0;i--) {
		if (angle>_fine_angle_values[i]) {
			return _fine_angle_values[i];
		}
	}

	return _fine_angle_values[7];
}

/*
 * Returns the Coords as a string, useful for debugging.
 */
string Coords::toString() {
	stringstream ss;
	ss<< "x=" << _x << " y=" << _y;
	return ss.str();
}

/*
 * Finds the average of angle1 and angle2, and returns the angle opposite to it (+PI)
 * Useful for mathematics used later for Alien Information.
 */
float Coords::getBackAngle(float angle1, float angle2) {
	float back_angle;

	if (angle2<angle1)
		angle2+=2*PI;
	back_angle = (angle1+angle2)/2.0;

	back_angle += PI;

	while (back_angle>2.0*PI)
		back_angle-=2.0*PI;

	return back_angle;
}

/*
 * Dermines if angle1 is between angle2 and angle3.
 * Radians overflow at 2*PI, so it's assumed angle2<angle3.
 */
bool Coords::angleBetween(float angle1, float angle2, float angle3) {
	while (angle1<0)
		angle1+=2.0*PI;
	while (angle2<0)
		angle2+=2.0*PI;
	while (angle3<0)
		angle3+=2.0*PI;
	while (angle1>2.0*PI)
		angle1-=2.0*PI;
	while (angle3<angle2)
		angle3+=2.0*PI;
	while (angle1<angle2)
		angle1+=2.0*PI;
	if ((angle1>=angle2)&&(angle1<=angle3))
		return true;
	return false;
}

