/*
 * FrameTimer
 *
 * Class to time frame length, and extend it if necessary.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <unistd.h>
#include <time.h>

#include "SDL/SDL.h"

#include "FrameTimer.h"

using namespace std;

int FrameTimer::_frame_length=30;
int FrameTimer::_last_tick=0;
int FrameTimer::_interim_tick=0;
int FrameTimer::_time_elapsed=0;
int FrameTimer::_last_start=0;

/*
 * Constructor.  Starts the timer.
 */
FrameTimer::FrameTimer() {
	startTimer();
}

/*
 * Sets the frame length (useful for paused games).
 */
void FrameTimer::setFrameLength(int frame_length) {
	_frame_length=frame_length;
}

/*
 * Returns the frame length.
 */
int FrameTimer::getFrameLength() {
	return _frame_length;
}

/*
 * Gets the Interim Frame Length.  Useful for debugging.  Measures the time from the last
 * call to getInterimFrameLength().
 */
int FrameTimer::getInterimFrameLength() {
	int frame_length = SDL_GetTicks() - _interim_tick;
	_interim_tick = SDL_GetTicks();
	return frame_length ;
}

/*
 * Returns the game FPS, based on Frame Length.
 */
int FrameTimer::getFPS() {
	return 1000/_frame_length;
}

/*
 * Delay the frame so that at least 30ms pass between frames.
 *
 * ***NOTE!!!*** Because of the 10ms accuracy level on Linux, I've had to busy wait here.
 * I was hoping to get nanosleep to work, but it's restricted by the same limitation.
 * I'd love an alternative solution here, but I can't find one.
 * _sleep works just fine for Windows users (though it is depricated)
 */
void FrameTimer::delayFrame() {

	_frame_length = (SDL_GetTicks() - _last_tick);
	if (_frame_length<30) {
#if defined(HAVE_NANOSLEEP)
		//Linux
		/*
		timespec tmReq;
		tmReq.tv_sec = 0;
		tmReq.tv_nsec = (30-_frame_length)*1000*1000;
		(void)nanosleep(&tmReq, (timespec *)NULL);
		*/

		while (_frame_length<10) {
			//(void)nanosleep(&tmReq, (timespec *)NULL);
			//This delays 10ms anyways...
			SDL_Delay(1);
			_frame_length = (SDL_GetTicks() - _last_tick);
		}
		while (_frame_length<30) {
			//FINE!  I'll busy loop...
			_frame_length = (SDL_GetTicks() - _last_tick);
		}

#else
		//MingW
		_sleep(30-_frame_length);
#endif
	}

	_frame_length = (SDL_GetTicks() - _last_tick);
	_last_tick=SDL_GetTicks();
	_interim_tick=_last_tick;
}

/*
 * Get time elapsed since game start.
 */
int FrameTimer::getTimeElapsed() {
	return (SDL_GetTicks()-_last_start+_time_elapsed)/1000;
}

/*
 * Start the game timer.
 */
void FrameTimer::startTimer() {
	_last_start = SDL_GetTicks();
	_last_tick = SDL_GetTicks();
}

/*
 * Pause the game timer.
 */
void FrameTimer::pauseTimer() {
	_time_elapsed += SDL_GetTicks()-_last_start;
}

/*
 * Reset the game timer, for a game restart.
 */
void FrameTimer::resetTimer() {
	_last_start = SDL_GetTicks();
	_time_elapsed = 0;
}
