/*
 * GameServer
 *
 * Server backend that handles network events from clients, creating NetworkConnections and
 * exchanges messages between players.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <stdio.h>
#include <time.h>

#include <iostream>
#include <string>

#include "SDL/SDL.h"
#include "SDL/SDL_net.h"

#include "Definitions.h"
#include "CaptainNames.h"
#include "Artifact.h"
#include "NetworkConnection.h"
#include "GameServer.h"

#define MAX_CLIENTS 32

using namespace std;

/*
 * Constructor. Initalizes network game server.
 */
GameServer::GameServer() {
	_port = SERVER_PORT;
	_wait = false;

	SDLNet_Init();

	CaptainNames();
}

/*
 * Deconstructor.  Cuts network connection.
 */
GameServer::~GameServer() {
	SDLNet_Quit();
}

/*
 * Initialize the host by setting up the server socket.
 */
bool GameServer::initHost() {
	SDLNet_ResolveHost(&_ip_address, NULL, (Uint16) _port);
	_server_socket=SDLNet_TCP_Open(&_ip_address);
	return (_server_socket!=NULL);
}

/*
 * Accepts new connections from clients.
 */
NetworkConnection *GameServer::acceptNewClientConnection() {
	TCPsocket client_socket=SDLNet_TCP_Accept(_server_socket);
	if (client_socket) {
		NetworkConnection client = NetworkConnection(client_socket);
		if (_clients.size()+1>MAX_CLIENTS) {
			stringstream ss;
			ss << MAX_CLIENTS;
			string max_clients = ss.str();
			client.queueMessage("MESG","The server has reached the maximum number of connections ("+max_clients+").");
			client.queueMessage("KICK","Server Full");
			client.send();
			client.destroy();
			return NULL;
		}
		client.setActive(true);
		_clients.push_back(client);
		log(&_clients[_clients.size()-1],true,"CONN","");
		return &_clients[_clients.size()-1];
	}
	return NULL;
}

/*
 * Queues broadcast command for all players
 */
void GameServer::queueBroadcastMessage(string command) {
	queueBroadcastMessage(command,"");
}

/*
 * Queues broadcast message for all players
 */
void GameServer::queueBroadcastMessage(string command, string message) {
	log(NULL,false,command,message);
	for (int i=0;i<(int)_clients.size();i++) {
		_clients[i].queueMessage(command, message);
	}
}

/*
 * Queues broadcast commands to all players except one.
 */
void GameServer::queueBroadcastMessageExcept(int except, string command) {
	queueBroadcastMessageExcept(except,command,"");
}

/*
 * Queues broadcast messages to all players but one.
 */
void GameServer::queueBroadcastMessageExcept(int except, string command, string message) {
	log(NULL,false,command,message);
	for (int i=0;i<(int)_clients.size();i++) {
		if (i==except)
			continue;
		_clients[i].queueMessage(command, message);
	}
}

/*
 * Push queued messages to all clients.
 */
void GameServer::pushMessages() {
   	for (int i=_clients.size()-1;i>=0;i--) {
   		if (!_clients[i].send()) {
			log(&_clients[i],true,"DEAD","");
			string captain_name = _clients[i].getIdent();
			if (captain_name!="") {
				CaptainNames::returnName(captain_name);
				stringstream id;
				id << _clients[i].getID();
				queueBroadcastMessage("PART",id.str());
			} //otherwise likely a version mismatch kick

   			_clients[i].destroy();
			_clients.erase(_clients.begin()+i,_clients.begin()+i+1);
   		}
   	}
}

/*
 * Handle messages sent from game clients.
 */
void GameServer::handleMessages() {
	for (int i=(int)_clients.size()-1;i>=0;i--) {
		vector<string> messages = _clients[i].receive();
		for (int j=0;j<(int)messages.size();j++) {
			_clients[i].setLastRecvTime();
			string command="";
			string message="";

			if (messages[j].length()>=4) {
				command = messages[j].substr(0,4);
				if (messages[j].length()>=6) {
					message = messages[j].substr(5,messages[j].length());
				}
			}

			log(&_clients[i],true,command,message);

			if (command=="PING") {
				//Client sends PING, return PONG.
				_clients[i].queueMessage("PONG");
				log(&_clients[i],false,"PONG","");
			} else if (command=="PONG") {
				//Do nothing.
			} else if (command=="JOIN") {
				//Client attempting to join game.
				stringstream version;
				version << GAME_VERSION;

				if (version.str()!=message) {
					//version mismatch... kick player
					_clients[i].queueMessage("KICK","Version Mismatch.  Please download a new version at http://games.randomlynx.net/");
					log(&_clients[i],false,"KICK","Version Mismatch.  Please download a new version at http://games.randomlynx.net/");
				} else {
					string captain_name = CaptainNames::getRandomName();
					_clients[i].setIdent(captain_name);
					stringstream id;
					id << _clients[i].getID() << " " <<captain_name;
					_clients[i].queueMessage("IDEN",id.str());
					log(&_clients[i],false,"IDEN",id.str());

					id.str(string());
					id << _clients[i].getID() << " " << captain_name;
					queueBroadcastMessageExcept(i,"JOIN",id.str());

					if (Artifact::getArtifactMessage()!="")
						_clients[i].queueMessage("ARTI",Artifact::getArtifactMessage());
					log(&_clients[i],false,"ARTI",Artifact::getArtifactMessage());
					for (int j=0;j<(((int)_plundered_stars.size()+9)/10);j++) {
						int remain=10;
						if ((j+1)*10>(int)_plundered_stars.size()) {
							remain=_plundered_stars.size()-j*10;
						}
						stringstream ss;
						for (int k=0;k<remain;k++) {
							int m = j*10+k;
							ss<<_plundered_stars[m];
							if (k<remain-1)
								ss<<" ";
						}
						_clients[i].queueMessage("PLUN",ss.str());

						//cout << "ID#" <<_clients[i].getID()<<" << PLUN " << ss.str()<<endl;
						log(&_clients[i],false,"PLUN",ss.str());
					}

					for (int j=0;j<(int)_clients.size();j++) {
						if (i==j)
							continue;
						id.str(string());
						id << _clients[j].getID() << " " << _clients[j].getIdent();

						_clients[i].queueMessage("PLAY",id.str());
						log(&_clients[i],false,"PLAY",id.str());
					}
				}
			} else if (command=="SHIP") {
				//Client sending ship location.
				stringstream ss;
				ss << _clients[i].getID() << " " << message;
				queueBroadcastMessageExcept(i,"SHIP",ss.str());
				_clients[i].setActive(true);
			} else if (command=="PART") {
				//Client leaving the game
				CaptainNames::returnName(_clients[i].getIdent());
				stringstream id;
				id << _clients[i].getID();
				queueBroadcastMessage("PART",id.str());
				_clients[i].destroy();
				_clients.erase(_clients.begin()+i,_clients.begin()+i+1);
				continue;
			} else if (command=="PLUN") {
				//Client plundering an artifact.
				queueBroadcastMessageExcept(i,"PLUN",message);
				_plundered_stars.push_back(message);
			} else if (command=="TALK") {
				//Client sending a chat message.
				stringstream ss;
				ss << _clients[i].getID() << " " << message;
				queueBroadcastMessage("TALK",ss.str());
			} else if (command=="VICT") {
				//Client claims Victory!
				stringstream ss;
				ss << _clients[i].getID();
				queueBroadcastMessageExcept(i,"VICT",ss.str());
				_wait=true;
			} else {
				//Do nothing.  All non PING/PONG are logged anyways.
			}
		}
	}
}

/*
 * Returns the value of _wait, which defines if we're waiting between games.
 */
bool GameServer::isWait() {
	if (_wait) {
		_wait=false;
		return true;
	}
	return false;
}

/*
 * Start a new game.
 */
void GameServer::gameStart() {
	kickInactive();
	_plundered_stars.erase(_plundered_stars.begin(),_plundered_stars.end());
	Artifact::createArtifactMessage();
	queueBroadcastMessage("ARTI",Artifact::getArtifactMessage());
}

/*
 * Get a string time, suitable for logging.
 */
string GameServer::getTime() {
	time_t rawtime;
	struct tm * timeinfo;
	char buffer[80];

	time ( &rawtime );
	timeinfo = localtime(&rawtime);

	strftime (buffer,80,"%Y/%m/%d %H:%M:%S",timeinfo);
	return string(buffer);
}

/*
 * Pad a string with a character.
 */
string GameServer::pad(string s, int length, char with) {
	while ((int)s.length()<length)
		s=with+s;
	return s;
}

/*
 * Log network events to stdout.  Record time, IP address, command and message for everything
 * except PING and PONG.
 */
void GameServer::log(NetworkConnection *client, bool inbound, string command, string message) {
	if ((command=="PING")||(command=="PONG"))
		return;
	cout << getTime();
	if (client!=NULL) {
		stringstream id_ss ;
		id_ss << (*client).getID();
		string id = pad(id_ss.str(),3,' ');
		cout << " [ID#" << id;
		cout << "] "<<pad("("+(*client).getIPAddress()+")",17,' ')<<" ";
	} else
		cout << "                            ";
	if (inbound)
		cout << " >> ";
	else
		cout << " << ";
	cout << command << " " << message << endl;
}

/*
 * Kick inactive players from the game.
 * Inactive players are those who have not moved since the game has started.
 * Newly joined players are given a one game exemption (in case they joined and the game
 * is about to restart.
 */
void GameServer::kickInactive() {
	for (int i=_clients.size()-1;i>=0;i--) {
		if (!_clients[i].isActive()) {
			_clients[i].queueMessage("KICK","Inactivity");
			log(&_clients[i],false,"KICK","Inactivity");
			_clients[i].send();
			CaptainNames::returnName(_clients[i].getIdent());
			queueBroadcastMessage("BROA",_clients[i].getIdent()+" has been kicked for inactivity.");
			stringstream id;
			id << _clients[i].getID();
			queueBroadcastMessage("PART",id.str());
			_clients[i].destroy();
			_clients.erase(_clients.begin()+i,_clients.begin()+i+1);
		} else {
			_clients[i].setActive(false);
		}
	}
}

/*
 * Check the time of the last message sent from client.  If the client hasn't reported in within
 * the last 10 seconds, declare them DEAD.
 */
void GameServer::checkIfDead() {
	for (int i=_clients.size()-1;i>=0;i--) {
		if (_clients[i].getTimeSinceLastRecv()>10000) {
			log(&_clients[i],true,"DEAD","");
			string captain_name = _clients[i].getIdent();
			if (captain_name!="") {
				CaptainNames::returnName(captain_name);
				stringstream id;
				id << _clients[i].getID();
				queueBroadcastMessage("PART",id.str());
			} //otherwise likely a version mismatch kick

   			_clients[i].destroy();
			_clients.erase(_clients.begin()+i,_clients.begin()+i+1);
		}
	}
}

/*
 * Send a PING message to all clients (which should reply with PONG)
 */
void GameServer::pingClients() {
	for (int i=_clients.size()-1;i>=0;i--) {
		_clients[i].sendPing();
	}
}
