/*
 * Information
 *
 * Object describing a single piece of alien information.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <vector>

#include "Definitions.h"
#include "Coords.h"
#include "Ship.h"
#include "View.h"
#include "Information.h"

#include "SoundSystem.h"

using namespace std;

vector<Information> Information::_information;
Information Information::_last;

/*
 * Constructor.
 */
Information::Information() {
}

/*
 * Copy Constructor.
 */
Information::Information(const Information &b) {
	_position=b._position;
	_type=b._type;
	_value1=b._value1;
	_value2=b._value2;
}

/*
 * Create new information.
 * Type 0 is a circle
 * Type 1 is an inverted circle
 * Type 3 is an inverted wedge
 * Type 4 is a wedge.
 * Type 6 is a sector
 * Type 7 is an inverted sector.
 *
 * For circles, value1 is the radius.  value2 is ignored.
 * For wedges, value1 and value2 are the angle of the wedge sides.
 * For sectors, value1 is the sector number in full stored as a float (e.g. 1234567.0).
 *   value2 is ignored.
 */
Information::Information(int type, float value1, float value2) {
	Ship *my_ship = Ship::getMyShip();
	_position=(*my_ship).getShipCoords();
	_type=type;
	_value1=value1;
	_value2=value2;
}

/*
 * Create new information (ignoring value2).
 */
Information::Information(int type, float value1) {
	Ship *my_ship = Ship::getMyShip();
	_position=(*my_ship).getShipCoords();
	_type=type;
	_value1=value1;
	_value2=0.0;
}

/*
 * Add information to the collection of information.
 */
void Information::addInformation(Information info) {
	for (int i=0;i<(int)_information.size();i++) {
		if (info.isExcludedBy(_information[i])) {
			//Throw away new info, already excluded.
			return;
		}
	}

	for (int i=_information.size()-1;i>=0;i--) {
		if (_information[i].isExcludedBy(info)) {
			//Throw out old info excluded by new info
			_information.erase(_information.begin()+i,_information.begin()+i+1);
		}
	}

	_information.push_back(info);
}

/*
 * Remove information from collection. Used for debugging
 */
void Information::removeInformation() {
	if (_information.size()==0)
		return;
	_last = _information[_information.size()-1];
	_information.erase(_information.end()-1);
}

/*
 * Readd removed information from collection.  Used for debugging.
 */
void Information::readdInformation() {
	cout << "ADDING INFORMATION\n";
	cout << "TYPE     = " << _last._type <<endl;
	cout << "POSITION = " << _last._position.toString()<< endl;
	cout << "VALUE1   = " << _last._value1 <<endl;
	cout << "VALUE2   = " << _last._value2 <<endl;
	cout << "INFO SIZE= " << _information.size() <<endl;

	addInformation(_last);

	_last._value1*=1.1;
}

/*
 * Promote information to the top, so that it's checked first.
 * In theory, should save time calculating and rendering information.
 */
void Information::promoteInformation(int i) {
	if (i==0)
		return;
	Information temp = _information[i];
	_information.erase(_information.begin()+i,_information.begin()+i+1);
	_information.insert(_information.begin(),temp);
}

/*
 * Returns a vector containing the collection of information.
 */
vector<Information> Information::getInformation() {
	return _information;
}

/*
 * Returns the type of the information.
 */
int Information::getType() {
	return _type;
}

/*
 * Returns value1 of the information.
 */
float Information::getValue1() {
	return _value1;
}

/*
 * Returns value2 of the information.
 */
float Information::getValue2() {
	return _value2;
}

/*
 * Gets the Coords of the information (where the information was received).
 */
Coords Information::getPosition() {
	return _position;
}

/*
 * Checks to see if we can throw away data.
 * If this information shape is inside the shape of info, return true.
 * This is probably some of the most complicated code in the game.
 */
bool Information::isExcludedBy(Information info) {
	Coords delta = _position-info._position;
	float dist = delta.getLength();
	float angle = delta.getAngle();
	int extype=_type*9+info._type;
	if (extype==9*CIRCLE+CIRCLE) {
		//Circle excluded by a circle
		//TESTED WORKING
		return (info._value1>=dist+_value1);
	} else if (extype==9*CIRCLE+INV_CIRCLE) {
		//Circle excluded by an inverse circle
		//TESTED WORKING
		return (info._value1<=dist-_value1);
	} else if (extype==9*CIRCLE+INV_WEDGE) {
		//Circle excluded by an inverse wedge
		//TESTED WORKING
		float tangent_theta1 = info._value1+PI/2.0;
		Coords intersect1 =  Coords(_value1,tangent_theta1,true)+delta;
		float intersect1_angle = intersect1.getAngle();

		float tangent_theta2 = info._value2-PI/2.0;
		Coords intersect2 =  Coords(_value1,tangent_theta2,true)+delta;
		float intersect2_angle = intersect2.getAngle();

		if ((!Coords::angleBetween(intersect2_angle,info._value1,info._value2)) && (!Coords::angleBetween(intersect1_angle,info._value1,info._value2)) && (dist>=_value1))
			return true;
		return false;

	} else if (extype==9*CIRCLE+WEDGE) {
		//Circle excluded by a wedge
		//TESTED WORKING
		float tangent_theta1 = info._value1-PI/2.0;
		Coords intersect1 =  Coords(_value1,tangent_theta1,true)+delta;
		float intersect1_angle = intersect1.getAngle();

		float tangent_theta2 = info._value2+PI/2.0;
		Coords intersect2 =  Coords(_value1,tangent_theta2,true)+delta;
		float intersect2_angle = intersect2.getAngle();

		if ((Coords::angleBetween(intersect2_angle,info._value1,info._value2)) && (Coords::angleBetween(intersect1_angle,info._value1,info._value2)) && (dist>=_value1))
			return true;
		return false;
	} else if (extype==9*CIRCLE+SECTOR) {
		//Circle excluded by a sector
		//TESTED WORKING
		//This is an imperfect solution.  Circles off screen aren't excluded.
		//However later circles should clear things up.
		Coords offset = View::getOffset((int)info._value1);
		int level = View::getLevel((int)info._value1);
		float scale = View::getScale(level);

		Coords this_delta = _position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;
		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta+_value1<=scale/2.0))
			return true;
		return false;
	} else if (extype==9*CIRCLE+INV_SECTOR) {
		//Circle excluded by an inverse sector
		//TESTED WORKING
		Coords offset = View::getOffset((int)info._value1);
		int level = View::getLevel((int)info._value1);
		float scale = View::getScale(level);

		Coords this_delta = _position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;
		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta-_value1<=scale/2.0))
			return false;
		return true;
	//} else if (extype==9*INV_CIRCLE+CIRCLE) {
		//Inverse Circle excluded by a circle
		//NOT POSSIBLE
	} else if (extype==9*INV_CIRCLE+INV_CIRCLE) {
		//Inverse Circle excluded by an inverse circle
		//TESTED WORKING
		return (_value1>=dist+info._value1);
	//} else if (extype==9*INV_CIRCLE+INV_WEDGE) {
		//Inverse Circle excluded by an inverse wedge
		//NOT POSSIBLE
	//} else if (extype==9*INV_CIRCLE+WEDGE) {
		//Inverse Circle excluded by a wedge
		//NOT POSSIBLE
	//} else if (extype==9*INV_CIRCLE+SECTOR) {
		//Inverse Circle excluded by a sector
		//NOT POSSIBLE
	} else if (extype==9*INV_CIRCLE+INV_SECTOR) {
		//Inverse Circle excluded by an inverse sector
		//TESTED WORKING
		Coords offset = View::getOffset((int)info._value1);
		int level = View::getLevel((int)info._value1);
		float scale = View::getScale(level);

		Coords points[4];
		float points_radius[4];
		for (int i=0;i<4;i++) {
			int x_sign=1;
			int y_sign=1;
			if ((i/2)==0)
				x_sign=-1;
			if ((i%2)==0)
				y_sign=-1;

			points[i]=Coords(offset.getX()+x_sign*scale/2.0,offset.getY()+y_sign*scale/2.0)-_position;
			points_radius[i]=points[i].getLength();
			//if a point in the square is outside the circle
			if (points_radius[i]>_value1)
				return false;
		}
		return true;
	//} else if (extype==9*INV_WEDGE+CIRCLE) {
		//Inverse Wedge excluded by a circle
		//NOT POSSIBLE
	//} else if (extype==9*INV_WEDGE+INV_CIRCLE) {
		//Inverse Wedge excluded by an inverse circle
		//NOT POSSIBLE
	} else if (extype==9*INV_WEDGE+INV_WEDGE) {
		//Inverse Wedge excluded by an inverse wedge
		//TESTED WORKING
		if ((Coords::angleBetween(angle+PI,_value1,_value2))&&
			(Coords::angleBetween(info._value1,_value1,_value2))&&
			(Coords::angleBetween(info._value2,_value1,_value2))) {
			return true;
		}
		return false;
	//} else if (extype==9*INV_WEDGE+WEDGE) {
		//Inverse Wedge excluded by a wedge
		//NOT POSSIBLE
	//} else if (extype==9*INV_WEDGE+SECTOR) {
		//Inverse Wedge excluded by a sector
		//NOT POSSIBLE
	} else if (extype==9*INV_WEDGE+INV_SECTOR) {
		//Inverse Wedge excluded by an inverse sector
		//TESTED WORKING
		Coords offset = View::getOffset((int)info._value1);
		int level = View::getLevel((int)info._value1);
		float scale = View::getScale(level);

		Coords this_delta = _position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;

		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta<=scale/2.0))
			return false;

		Coords points[4];
		float points_theta[4];
		for (int i=0;i<4;i++) {
			int x_sign=1;
			int y_sign=1;
			if ((i/2)==0)
				x_sign=-1;
			if ((i%2)==0)
				y_sign=-1;

			points[i]=Coords(offset.getX()+x_sign*scale/2.0,offset.getY()+y_sign*scale/2.0)-_position;
			points_theta[i]=points[i].getAngle();
			//if a point in the square is outside the wedge
			if (!Coords::angleBetween(points_theta[i],_value1,_value2))
				return false;
		}
		return true;
	//} else if (extype==9*WEDGE+CIRCLE) {
		//Wedge excluded by a circle
		//NOT POSSIBLE
	} else if (extype==9*WEDGE+INV_CIRCLE) {
		//Wedge excluded by an inverse circle
		//TESTED WORKING
		float tangent_theta1 = _value1+PI/2.0;
		Coords intersect1 =  Coords(info._value1,tangent_theta1,true)-delta;
		float intersect1_angle = intersect1.getAngle();

		float tangent_theta2 = _value2-PI/2.0;
		Coords intersect2 =  Coords(info._value1,tangent_theta2,true)-delta;
		float intersect2_angle = intersect2.getAngle();

		if ((!Coords::angleBetween(intersect2_angle,_value1,_value2)) && (!Coords::angleBetween(intersect1_angle,_value1,_value2)) && (dist>=info._value1))
			return true;

		return false;
	} else if (extype==9*WEDGE+INV_WEDGE) {
		//Wedge excluded by an inverse wedge
		//TESTED WORKING
		if (Coords::angleBetween(angle,info._value1,info._value2))
			return false;

		if (Coords::angleBetween(_value1,info._value1,info._value2-0.001))
			return false;

		if (Coords::angleBetween(_value2,info._value1+0.001,info._value2))
			return false;

		if ((Coords::angleBetween(_value1,info._value2,info._value2+PI))&&
			(Coords::angleBetween(_value2,info._value2,info._value2+PI))&&
			(Coords::angleBetween(angle,info._value2,info._value2+PI))) {
			return true;
		}

		if ((Coords::angleBetween(_value1,info._value1-PI,info._value1))&&
			(Coords::angleBetween(_value2,info._value1-PI,info._value1))&&
			(Coords::angleBetween(angle,info._value1-PI,info._value1))) {
			return true;
		}

		return false;
	} else if (extype==9*WEDGE+WEDGE) {
		//Wedge excluded by a wedge
		//TESTED WORKING
		if ((Coords::angleBetween(angle,info._value1,info._value2)) &&
			(Coords::angleBetween(_value1,info._value1,info._value2)) &&
			(Coords::angleBetween(_value2,info._value1,info._value2)))
			return true;
		return false;

	//} else if (extype==9*WEDGE+SECTOR) {
		//Wedge excluded by a sector
		//FIXME
		//So Unlikely, likely meanless to check.
	} else if (extype==9*WEDGE+INV_SECTOR) {
		//Wedge excluded by an inverse sector
		//TESTED WORKING
		Coords offset = View::getOffset((int)info._value1);
		int level = View::getLevel((int)info._value1);
		float scale = View::getScale(level);

		Coords this_delta = _position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;

		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta<=scale/2.0))
			return false;

		Coords points[4];
		float points_theta[4];
		for (int i=0;i<4;i++) {
			int x_sign=1;
			int y_sign=1;
			if ((i/2)==0)
				x_sign=-1;
			if ((i%2)==0)
				y_sign=-1;

			points[i]=Coords(offset.getX()+x_sign*scale/2.0,offset.getY()+y_sign*scale/2.0)-_position;
			points_theta[i]=points[i].getAngle();
			//if a point in the square is outside the circle
			if (Coords::angleBetween(points_theta[i],_value1,_value2))
				return false;
		}
		return true;
	} else if (extype==9*SECTOR+CIRCLE) {
		//Sector excluded by a circle
		//TESTED WORKING
		Coords offset = View::getOffset((int)_value1);
		int level = View::getLevel((int)_value1);
		float scale = View::getScale(level);

		Coords points[4];
		float points_radius[4];
		for (int i=0;i<4;i++) {
			int x_sign=1;
			int y_sign=1;
			if ((i/2)==0)
				x_sign=-1;
			if ((i%2)==0)
				y_sign=-1;

			points[i]=Coords(offset.getX()+x_sign*scale/2.0,offset.getY()+y_sign*scale/2.0)-info._position;
			points_radius[i]=points[i].getLength();
			//if a point in the square is outside the circle
			if (points_radius[i]>info._value1)
				return false;
		}
		return true;
	} else if (extype==9*SECTOR+INV_CIRCLE) {
		//Sector excluded by an inverse circle
		//TESTED WORKING
		Coords offset = View::getOffset((int)_value1);
		int level = View::getLevel((int)_value1);
		float scale = View::getScale(level);

		Coords this_delta = info._position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;
		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta-info._value1<scale/2.0))
			return false;
		return true;
	} else if (extype==9*SECTOR+INV_WEDGE) {
		//Sector excluded by an inverse wedge
		//TESTED WORKING
		Coords offset = View::getOffset((int)_value1);
		int level = View::getLevel((int)_value1);
		float scale = View::getScale(level);

		Coords this_delta = info._position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;

		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta<=scale/2.0))
			return false;

		Coords points[4];
		float points_theta[4];
		for (int i=0;i<4;i++) {
			int x_sign=1;
			int y_sign=1;
			if ((i/2)==0)
				x_sign=-1;
			if ((i%2)==0)
				y_sign=-1;

			points[i]=Coords(offset.getX()+x_sign*scale/2.0,offset.getY()+y_sign*scale/2.0)-info._position;
			points_theta[i]=points[i].getAngle();
			//if a point in the square is outside the wedge
			if (Coords::angleBetween(points_theta[i],info._value1,info._value2))
				return false;
		}
		return true;
	} else if (extype==9*SECTOR+WEDGE) {
		//Sector excluded by a wedge
		//TESTED WORKING
		Coords offset = View::getOffset((int)_value1);
		int level = View::getLevel((int)_value1);
		float scale = View::getScale(level);

		Coords this_delta = info._position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;

		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta<=scale/2.0))
			return false;

		Coords points[4];
		float points_theta[4];
		for (int i=0;i<4;i++) {
			int x_sign=1;
			int y_sign=1;
			if ((i/2)==0)
				x_sign=-1;
			if ((i%2)==0)
				y_sign=-1;

			points[i]=Coords(offset.getX()+x_sign*scale/2.0,offset.getY()+y_sign*scale/2.0)-info._position;
			points_theta[i]=points[i].getAngle();
			//if a point in the square is outside the wedge
			if (!Coords::angleBetween(points_theta[i],info._value1,info._value2))
				return false;
		}
		return true;
	} else if (extype==9*SECTOR+SECTOR) {
		//Sector excluded by a sector
		//TESTED WORKING
		int info_sector = (int)info._value1;
		int sector = (int)_value1;
		int position = 1000000;
		for (int i=6;i>=0;i--) {
			int info_digit = info_sector/position;
			int digit = sector/position;
			if (info_digit==0)
				//Info has better data
				return true;
			if (digit==0)
				//We have better data
				return false;
			if (info_digit!=digit)
				//Different Data
				return false;
			info_sector-=position*info_digit;
			sector-=position*digit;
			position /=10;
		}
		//Sectors are the same!
		return true;
	} else if (extype==9*SECTOR+INV_SECTOR) {
		//Sector excluded by an inverse sector
		//TESTED WORKING
		int info_sector = (int)info._value1;
		int sector = (int)_value1;
		int position = 1000000;
		for (int i=6;i>=0;i--) {
			int info_digit = info_sector/position;
			int digit = sector/position;
			if (info_digit==0)
				//Info has better data
				return false;
			if (digit==0)
				//We have better data
				return false;
			if (info_digit!=digit)
				//Different Data
				return true;
			info_sector-=position*info_digit;
			sector-=position*digit;
			position /=10;
		}
		//Should never happen, otherwise map filled in!
		return false;
	//} else if (extype==9*INV_SECTOR+CIRCLE) {
		//Inverse Sector excluded by a circle
		//NOT POSSIBLE
	} else if (extype==9*INV_SECTOR+INV_CIRCLE) {
		//Inverse Sector excluded by an inverse circle
		//Non optimal solution, circle has to be inside sector.
		Coords offset = View::getOffset((int)_value1);
		int level = View::getLevel((int)_value1);
		float scale = View::getScale(level);

		Coords this_delta = info._position-offset;
		float delta_x =this_delta.getX();
		float delta_y =this_delta.getY();
		float max_delta;
		if (fabs(delta_x)>fabs(delta_y))
			max_delta=fabs(delta_x);
		else
			max_delta=fabs(delta_y);
		if ((max_delta+info._value1<scale/2.0))
			return true;
		return false;
	//} else if (extype==9*INV_SECTOR+INV_WEDGE) {
		//Inverse Sector excluded by an inverse wedge
		//FIXME
		//UNLIKELY, TOO DIFFICULT TO CHECK!
	//} else if (extype==9*INV_SECTOR+WEDGE) {
		//Inverse Sector excluded by a wedge
		//NOT POSSIBLE
	//} else if (extype==9*INV_SECTOR+SECTOR) {
		//Inverse Sector excluded by a sector
		//NOT POSSIBLE
	} else if (extype==9*INV_SECTOR+INV_SECTOR) {
		//Inverse Sector excluded by an inverse sector
		//TESTED WORKING
		int info_sector = (int)info._value1;
		int sector = (int)_value1;
		int position = 1000000;
		for (int i=6;i>=0;i--) {
			int info_digit = info_sector/position;
			int digit = sector/position;
			if (digit==0)
				//We have Weaker data
				return true;
			if (info_digit==0)
				//We have better data
				return false;
			if (info_digit!=digit)
				//Should never happen, excludes whole map
				return false;
			info_sector-=position*info_digit;
			sector-=position*digit;
			position /=10;
		}
		//Sectors identical!
		return true;
	}
	//It's one of the types where exclusion is NOT POSSIBLE
	return false;
}

/*
 * Throw away all information in preparation for game reset.
 */
void Information::reset() {
	_information.erase(_information.begin(),_information.end());
}
