/*
 * InformationSurface
 *
 * Renders the collection of alien information as a red layer to be placed on top of
 * the GalaxySurface and the MinimapSurface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "Ship.h"
#include "View.h"
#include "Coords.h"
#include "Artifact.h"
#include "Star.h"
#include "FrameTimer.h"
#include "Information.h"
#include "Messages.h"
#include "ScreenSurface.h"
#include "MinimapSurface.h"
#include "GalaxySurface.h"
#include "InformationSurface.h"

SDL_Surface *InformationSurface::_surface;
SDL_Surface *InformationSurface::_border_surface;
SDL_Rect InformationSurface::_rect;
Sint32 InformationSurface::_color;
Coords InformationSurface::_offset;
float InformationSurface::_scale;
int InformationSurface::_screen_center;
int InformationSurface::_shrink;

using namespace std;

/*
 * Constructor. Initialize Information Surface.  Must be run before blit().
 */
InformationSurface::InformationSurface() {
	_shrink=1;
	_screen_center=SCREEN_CENTER/_shrink;
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_CENTER*2, SCREEN_DEPTH,0,0,0,0);
	_border_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_CENTER*2, SCREEN_DEPTH,0,0,0,0);

	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY, 0x00000000);
	SDL_SetColorKey(_border_surface, SDL_SRCCOLORKEY, 0x00000000);
	SDL_SetAlpha(_surface,SDL_SRCALPHA,63);

	_rect.x=0;
	_rect.y=0;
	_rect.w=_screen_center*2;
	_rect.h=_screen_center*2;
	_color=0xBF0000FF;

}

/*
 * Destructor.  Frees surfaces at game end.
 */
InformationSurface::~InformationSurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_border_surface);
}


/*
 * Blit the information surface to the galaxy surface.
 */
void InformationSurface::blit() {
	blit(false);
}

/*
 * Blit the information surface to the minimap if true, to the galaxy surface if false.
 */
void InformationSurface::blit(bool mini) {
	if (View::isUpdated()||View::isUpdateOverride()||mini) {
		if (!mini) {
			_offset = View::getOffset();
			_scale = View::getScale();
			_shrink=1;
		} else {
			_offset = Coords(0,0);
			_scale= 100000;
			_shrink=3;
		}
		_screen_center = SCREEN_CENTER/_shrink;
		boxColor(_surface,0,0,_screen_center*2,_screen_center*2,0x000000FF);
		boxColor(_border_surface,0,0,_screen_center*2,_screen_center*2,0x000000FF);
		vector<Information> information = Information::getInformation();

		bool all_filled=false;

		for (int i=0;i<(int)information.size();i++) {
			switch (information[i].getType()) {
				case 0:
					all_filled = blitCircle(information[i],false);
					break;
				case 1:
					all_filled = blitCircle(information[i],true);
					break;
				case 3:
					all_filled = blitWedge(information[i],true);
					break;
				case 4:
					all_filled = blitWedge(information[i],false);
					break;
				case 6:
					all_filled = blitSector(information[i],false);
					break;
				case 7:
					all_filled = blitSector(information[i],true);
					break;
			}

			if (all_filled) {
				boxColor(_surface,0,0,_screen_center*2,_screen_center*2,_color);
				boxColor(_border_surface,0,0,_screen_center*2,_screen_center*2,0x000000FF);
				Information::promoteInformation(i);
				break;
			}
		}
		if (!all_filled) {
			blitBorderSurface();
		}


		if (!mini) {
			SDL_BlitSurface(_surface,NULL,GalaxySurface::getGalaxySurface(),&_rect);
			SDL_BlitSurface(_border_surface,NULL,GalaxySurface::getGalaxySurface(),&_rect);
		} else {
			SDL_Rect mini_source_rect;
			mini_source_rect.x=mini_source_rect.y=0;
			mini_source_rect.w=mini_source_rect.h=_screen_center*2;

			boxColor(MinimapSurface::getMiniInformationSurface(),0,0,_screen_center*2,_screen_center*2,0x000000FF);
			boxColor(MinimapSurface::getMiniBorderSurface(),0,0,_screen_center*2,_screen_center*2,0x000000FF);

			SDL_BlitSurface(_surface,&mini_source_rect,MinimapSurface::getMiniInformationSurface(),NULL);
			SDL_BlitSurface(_border_surface,&mini_source_rect,MinimapSurface::getMiniBorderSurface(),NULL);
			/*
			//Prototype score code.  Doesn't work well at the moment.  Hopefully for Release D.
			int score = getInformationScore(MinimapSurface::getMiniInformationSurface());
			stringstream ss;
			ss.precision(0);
			ss << fixed << score << " points!";
			Messages::addMessageToQueue(ss.str());
			*/
		}
	}
}

/*
 * Blit information Circle (Information types 0 and 1).
 */
bool InformationSurface::blitCircle(Information info, bool inverted) {
	Coords points[4],points_delta[4];
	float points_radius[4],points_theta[4];

	Sint16 x_pos[5];
	Sint16 y_pos[5];

	Coords delta = info.getPosition()-_offset;
	float theta = delta.getAngle();
	float anti_theta = theta+PI;
	float delta_radius =delta.getLength();

	//points are the four points at the edge of the galaxy screen
	//points structure:
	//02
	//13

	for (int j=0;j<4;j++) {
		points[j]=_offset;
		if ((j/2)==0)
			points[j]+=Coords(-_scale/2.0,0.0);
		else
			points[j]+=Coords(_scale/2.0,0.0);
		if ((j%2)==0)
			points[j]+=Coords(0,-_scale/2.0);
		else
			points[j]+=Coords(0,_scale/2.0);
		points_delta[j]=points[j]-info.getPosition();
		points_theta[j]=points_delta[j].getAngle();
		points_radius[j]=points_delta[j].getLength();
	}

	delta.getIntValues(&x_pos[0],&y_pos[0],_scale,_shrink);

	float radius = info.getValue1();
	int r = (int)(radius*_screen_center*2/_scale);
	SDL_Surface* circle = SDL_CreateRGBSurface(SDL_HWSURFACE, _screen_center*2, _screen_center*2, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(circle, SDL_SRCCOLORKEY, 0x00000000);

	Sint32 color,anti_color;
	if (inverted) {
		color=0x000000FF;
		anti_color=_color;
	} else {
		color=_color;
		anti_color=0x000000FF;
	}

	int location=-1;

	// location:
	// -1 no points
	// 0 all inside
	// 1 some inside
	// 2 all outside
	for (int j=0;j<4;j++) {
		if (radius>points_radius[j]) {
			if (location==-1)
				location=0;
			else if (location==2) {
				location=1;
				break;
			}
		} else {
			if (location==-1)
				location=2;
			else if (location==0) {
				location=1;
				break;
			}
		}
	}

	if (location==0) {
		//We're inside the circle, all red.
		boxColor(circle,0,0,_screen_center*2,_screen_center*2,color);
		if (color==_color) {
			SDL_FreeSurface(circle);
			return true;
		}
	} else if ((location==2)&&(delta_radius>radius+(_scale/2.0)*ROOT_2)) {
		//We're outside the circle... do nothing
		boxColor(circle,0,0,_screen_center*2,_screen_center*2,anti_color);
		if (anti_color==_color) {
			SDL_FreeSurface(circle);
			return true;
		}
	} else if (r<10000) {
		//Circle
		boxColor(circle,0,0,_screen_center*2,_screen_center*2,anti_color);
		filledCircleColor(circle,x_pos[0],y_pos[0],r,color);
	} else {
		//Line
		//If r>11000, filledCircleColor starts to break down and eventually dissapear
		//Use a filled polygon instead
		boxColor(circle,0,0,_screen_center*2,_screen_center*2,anti_color);
		Coords tangent = Coords(radius,anti_theta,true)+delta;

		//02	  0
		//13	 1 2
		//		  3
		Coords slope[4];
		vector<Coords> dots;
		slope[1]=Coords(points[1].getX(),_offset.getY()+tangent.getY()+(_scale/2+tangent.getX())/tan(anti_theta));
		slope[2]=Coords(points[2].getX(),_offset.getY()+tangent.getY()-(_scale/2-tangent.getX())/tan(anti_theta));
		slope[0]=Coords(_offset.getX()+tangent.getX()+(_scale/2+tangent.getY())*tan(anti_theta),points[0].getY());
		slope[3]=Coords(_offset.getX()+tangent.getX()-(_scale/2-tangent.getY())*tan(anti_theta),points[3].getY());

		if (points_radius[1]<=radius) {
			dots.push_back(points[1]);
		}

		if ((slope[1].getY()>_offset.getY()-_scale/2.0)&&(slope[1].getY()<_offset.getY()+_scale/2.0)) {
			dots.push_back(slope[1]);
		}

		if (points_radius[0]<=radius) {
			dots.push_back(points[0]);
		}

		if ((slope[0].getX()>_offset.getX()-_scale/2.0)&&(slope[0].getX()<_offset.getX()+_scale/2.0)) {
			dots.push_back(slope[0]);
		}

		if (points_radius[2]<=radius) {
			dots.push_back(points[2]);
		}

		if ((slope[2].getY()>_offset.getY()-_scale/2.0)&&(slope[2].getY()<_offset.getY()+_scale/2.0)) {
			dots.push_back(slope[2]);
		}

		if (points_radius[3]<=radius) {
			dots.push_back(points[3]);
		}

		if ((slope[3].getX()>_offset.getX()-_scale/2.0)&&(slope[3].getX()<_offset.getX()+_scale/2.0)) {
			dots.push_back(slope[3]);
		}

		for (int j=0;j<(int)dots.size();j++) {
			Coords dots_delta = dots[j]-_offset;
			dots_delta.getIntValues(&x_pos[j],&y_pos[j],_scale,_shrink);
			if (x_pos[j]==1)
				x_pos[j]=0;
			if (y_pos[j]==1)
				y_pos[j]=0;
		}
		filledPolygonColor(circle,x_pos,y_pos,dots.size(),color);
	}

	SDL_BlitSurface(circle,NULL,_surface,NULL);
	SDL_FreeSurface(circle);

	return false;
}

/*
 * Blit and information Wedge (information types 3 and 4)
 */
bool InformationSurface::blitWedge(Information info, bool inverted) {
	int points_order[4] = {1,0,2,3};

	Coords points[4],points_delta[4];
	float points_radius[4],points_theta[4];
	bool points_inside[4];

	Sint16 x_pos[5];
	Sint16 y_pos[5];


	SDL_Surface* pie = SDL_CreateRGBSurface(SDL_HWSURFACE, _screen_center*2, _screen_center*2, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(pie, SDL_SRCCOLORKEY, 0x00000000);

	Sint32 color,anti_color;
	if (inverted) {
		color=0x000000FF;
		anti_color=_color;
	} else {
		color=_color;
		anti_color=0x000000FF;
	}

	Coords delta = info.getPosition()-_offset;
	float delta_radius =delta.getLength();

	bool outside_left=false;
	bool outside_right=false;
	bool inside=false;

	float back_angle = Coords::getBackAngle(info.getValue1(),info.getValue2());

	for (int j=0;j<4;j++) {
		points[j]=_offset;
		if ((j/2)==0)
			points[j]+=Coords(-_scale/2.0,0.0);
		else
			points[j]+=Coords(_scale/2.0,0.0);
		if ((j%2)==0)
			points[j]+=Coords(0,-_scale/2.0);
		else
			points[j]+=Coords(0,_scale/2.0);
		points_delta[j]=points[j]-info.getPosition();
		points_theta[j]=points_delta[j].getAngle();
		points_radius[j]=points_delta[j].getLength();
	}

	for (int j=0;j<4;j++) {
		points_inside[j]=false;
		if (Coords::angleBetween(points_theta[j],back_angle,info.getValue1()))
			outside_left=true;
		else if (Coords::angleBetween(points_theta[j],info.getValue2(),back_angle))
			outside_right=true;
		else {
			inside=true;
			points_inside[j]=true;
		}

	}

	if (!outside_left&&!outside_right&&inside) {
		//inside all red
		boxColor(pie,0,0,_screen_center*2,_screen_center*2,color);
		if (color==_color) {
			SDL_FreeSurface(pie);
			return true;
		}
	} else if ((!inside)&&(delta_radius>(_scale/2.0)*ROOT_2)) {
		//We're outside the pie... all clear
		boxColor(pie,0,0,_screen_center*2,_screen_center*2,anti_color);
		if (anti_color==_color) {
			SDL_FreeSurface(pie);
			return true;
		}

	} else if (inside&&outside_left&&!outside_right) {
		//Left Wing only
		boxColor(pie,0,0,_screen_center*2,_screen_center*2,anti_color);

		vector<Coords> dots;
		for (int j=0;j<4;j++) {
			int k=j+1;
			if (k==4)
				k=0;
			if (points_inside[points_order[j]]!=points_inside[points_order[k]]) {
				Coords dot;
				if(j%2==0) {
					//Vertical
					float dotx = points[points_order[j]].getX();
					float doty = info.getPosition().getY() + (dotx-info.getPosition().getX())*tan(info.getValue1());
					dot = Coords(dotx,doty);
				} else {
					//Horizontal
					float doty = points[points_order[j]].getY();
					float dotx = info.getPosition().getX() + (doty-info.getPosition().getY())/tan(info.getValue1());
					dot = Coords(dotx,doty);
				}
				dots.push_back(dot);
			}
			if (points_inside[points_order[k]]) {
				dots.push_back(points[points_order[k]]);
			}
		}

		for (int j=0;j<(int)dots.size();j++) {
			Coords thisPos=dots[j]-_offset;
			thisPos.getIntValues(&x_pos[j],&y_pos[j],_scale,_shrink);
			if (x_pos[j]==1)
				x_pos[j]=0;
			if (y_pos[j]==1)
				y_pos[j]=0;

		}
		filledPolygonColor(pie,x_pos,y_pos,dots.size(),color);
	} else if (inside&&!outside_left&&outside_right) {
		//Right Wing only
		boxColor(pie,0,0,_screen_center*2,_screen_center*2,anti_color);

		vector<Coords> dots;
		for (int j=0;j<4;j++) {
			int k=j+1;
			if (k==4)
				k=0;

			if (points_inside[points_order[j]]!=points_inside[points_order[k]]) {
				Coords dot;
				if(j%2==0) {
					//Vertical
					float dotx = points[points_order[j]].getX();
					float doty = info.getPosition().getY() + (dotx-info.getPosition().getX())*tan(info.getValue2());
					dot = Coords(dotx,doty);

				} else {
					//Horizontal
					float doty = points[points_order[j]].getY();
					float dotx = info.getPosition().getX() + (doty-info.getPosition().getY())/tan(info.getValue2());
					dot = Coords(dotx,doty);
				}
				dots.push_back(dot);
			}
			if (points_inside[points_order[k]]) {
				dots.push_back(points[points_order[k]]);
			}

		}

		for (int j=0;j<(int)dots.size();j++) {
			Coords thisPos=dots[j]-_offset;
			thisPos.getIntValues(&x_pos[j],&y_pos[j],_scale,_shrink);
			if (x_pos[j]==1)
				x_pos[j]=0;
			if (y_pos[j]==1)
				y_pos[j]=0;

		}
		filledPolygonColor(pie,x_pos,y_pos,dots.size(),color);


	} else {
		//Wedge
		//Draw wedge as a polygon to avoid fillPie, which can't render 22.5 degrees.
		boxColor(pie,0,0,_screen_center*2,_screen_center*2,anti_color);

		vector<Coords> dots;
		for (int j=0;j<4;j++) {
			int k=j+1;
			if (k==4)
				k=0;

			if (Coords::angleBetween(info.getValue1(),points_theta[points_order[j]],points_theta[points_order[k]])) {
				Coords dot;
				if(j%2==0) {
					//Vertical
					float dotx = points[points_order[j]].getX();
					float doty = info.getPosition().getY() + (dotx-info.getPosition().getX())*tan(info.getValue1());
					dot = Coords(dotx,doty);

				} else {
					//Horizontal
					float doty = points[points_order[j]].getY();
					float dotx = info.getPosition().getX() + (doty-info.getPosition().getY())/tan(info.getValue1());
					dot = Coords(dotx,doty);
				}
				dots.push_back(dot);
			}

			if (Coords::angleBetween(info.getValue2(),points_theta[points_order[j]],points_theta[points_order[k]])) {
				Coords dot;
				if(j%2==0) {
					//Vertical
					float dotx = points[points_order[j]].getX();
					float doty = info.getPosition().getY() + (dotx-info.getPosition().getX())*tan(info.getValue2());
					dot = Coords(dotx,doty);

				} else {
					//Horizontal
					float doty = points[points_order[j]].getY();
					float dotx = info.getPosition().getX() + (doty-info.getPosition().getY())/tan(info.getValue2());
					dot = Coords(dotx,doty);
				}
				dots.push_back(dot);
				dots.push_back(info.getPosition());
			}

			if (points_inside[points_order[k]]) {
				dots.push_back(points[points_order[k]]);
			}

		}

		for (int j=0;j<(int)dots.size();j++) {
			Coords thisPos=dots[j]-_offset;
			thisPos.getIntValues(&x_pos[j],&y_pos[j],_scale,_shrink);
			if (x_pos[j]==1)
				x_pos[j]=0;
			if (y_pos[j]==1)
				y_pos[j]=0;

		}
		filledPolygonColor(pie,x_pos,y_pos,dots.size(),color);

	}
	SDL_BlitSurface(pie,NULL,_surface,NULL);
	SDL_FreeSurface(pie);

	return false;
}

/*
 * Blit a sector (information types 6 and 7)
 */
bool InformationSurface::blitSector(Information info, bool inverted) {
	int points_order[4] = {1,0,2,3};
	Coords f_points[4];
	Sint16 x_pos[5];
	Sint16 y_pos[5];

	SDL_Surface* sector = SDL_CreateRGBSurface(SDL_HWSURFACE, _screen_center*2, _screen_center*2, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(sector, SDL_SRCCOLORKEY, 0x00000000);

	Sint32 color,anti_color;
	if (inverted) {
		color=0x000000FF;
		anti_color=_color;
	} else {
		color=_color;
		anti_color=0x000000FF;
	}

	int loc = (int)(info.getValue1());
	int location[7];
	int level=7;
	for (int j=6;j>=0;j--) {
		location[j]=loc%10;
		loc/=10;
		if (location[j]==0)
			level=j;
	}

	Coords f_offset = View::getOffset(location,level);
	float f_scale=100000/pow(3.0,level);

	Coords delta = f_offset-_offset;
	if ((fabs(_offset.getX()-f_offset.getX())<(f_scale/2.0)) && (fabs(_offset.getY()-f_offset.getY())<(f_scale/2.0)) && (_scale<=f_scale)) {
		//Red Area, Draw Red
		boxColor(sector,0,0,_screen_center*2,_screen_center*2,color);
		if (color==_color) {
			SDL_FreeSurface(sector);
			return true;
		}
	} else if ((fabs(_offset.getX()-f_offset.getX())>(_scale+f_scale)/2) || (fabs(_offset.getY()-f_offset.getY())>(_scale+f_scale)/2)) {
		//Clear Area, do nothing
		boxColor(sector,0,0,_screen_center*2,_screen_center*2,anti_color);
		if (anti_color==_color) {
			SDL_FreeSurface(sector);
			return true;
		}
	} else {
		//Draw Sector
		boxColor(sector,0,0,_screen_center*2,_screen_center*2,anti_color);
		for (int j=0;j<4;j++) {
			int x_sign=1;
			int y_sign=1;
			if ((j/2)==0)
				x_sign=-1;
			if ((j%2)==0)
				y_sign=-1;

			f_points[j]=Coords(f_offset.getX()+x_sign*f_scale/2.0,f_offset.getY()+y_sign*f_scale/2.0);

			if (f_points[j].getX()>_offset.getX()+_scale/2.0)
				f_points[j]=Coords(_offset.getX()+1.01*_scale/2.0,f_points[j].getY());
			if (f_points[j].getX()<_offset.getX()-_scale/2.0)
				f_points[j]=Coords(_offset.getX()-1.01*_scale/2.0,f_points[j].getY());
			if (f_points[j].getY()>_offset.getY()+_scale/2.0)
				f_points[j]=Coords(f_points[j].getX(),_offset.getY()+1.01*_scale/2.0);
			if (f_points[j].getY()<_offset.getY()-_scale/2.0)
				f_points[j]=Coords(f_points[j].getX(),_offset.getY()-1.01*_scale/2.0);
		}

		for (int j=0;j<4;j++) {
			x_pos[points_order[j]]=_screen_center+(int)(_screen_center*(f_points[j].getX()-_offset.getX())*2/_scale);
			y_pos[points_order[j]]=_screen_center+(int)(_screen_center*(f_points[j].getY()-_offset.getY())*2/_scale);
		}

		filledPolygonColor(sector,x_pos,y_pos,4,color);
	}
	SDL_BlitSurface(sector,NULL,_surface,NULL);
	SDL_FreeSurface(sector);

	return false;
}

/*
 * Take an existing red/black surface and create a new surface that draws a green line
 * at the border between the two.
 */
void InformationSurface::blitBorderSurface() {
	Uint16 *pixels_source, *pixels_target;
	Uint16 pixel,pixel_offset;
	int i,j;

	boxColor(_border_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x000000FF);

	SDL_LockSurface(_border_surface);

	pixels_source = (Uint16 *)_surface->pixels;
	pixels_target = (Uint16 *)_border_surface->pixels;

	for (i=0;i<_surface->h;i++) {
		for (j=0;j<_surface->w;j++) {
			pixel = pixels_source[ ( i * _surface->w ) + j ];

			if (pixel == 0xB800) {
				if (j<(_surface->w)-1) {
					//right
					pixel_offset = pixels_source[ ( i * _surface->w ) + j+1 ];
					if  (pixel_offset == 0x000000)
						pixels_target[ ( i * _surface->w ) + j ] = 0x7E0;
				}
				if (j>0) {
					//left
					pixel_offset = pixels_source[ ( i * _surface->w ) + j-1 ];
					if  (pixel_offset == 0x000000)
						pixels_target[ ( i * _surface->w ) + j ] = 0x7E0;
				}
				if (i<(_surface->h)-1) {
					//right
					pixel_offset = pixels_source[ ( (i+1) * _surface->w ) + j ];
					if  (pixel_offset == 0x000000)
						pixels_target[ ( i * _surface->w ) + j ] = 0x7E0;
				}
				if (i>0) {
					//left
					pixel_offset = pixels_source[ ( (i-1) * _surface->w ) + j ];
					if  (pixel_offset == 0x000000)
						pixels_target[ ( i * _surface->w ) + j ] = 0x7E0;
				}
			}
		}
	}

	SDL_UnlockSurface(_border_surface);
}

/*
 * Takes a surface and determines a score for the percentage filled in.
 * Not working yet, hopefully for Release D
 */
int InformationSurface::getInformationScore(SDL_Surface *surface) {
	float ratio = getInformationArea(surface);
	float score = -log(1-ratio)*100/log(9);
	return (int)score;
}

/*
 * Takes a surface, and determines what percent is filled in red.
 */
float InformationSurface::getInformationArea(SDL_Surface *surface) {
	Uint16 *pixels_source;

	SDL_LockSurface(surface);

	pixels_source = (Uint16 *) surface->pixels;

	float pixel_count=0;

	for (int i=0;i<surface->h;i++) {
		for (int j=0;j<surface->w;j++) {
			Uint16 pixel = pixels_source[ ( i * surface->w ) + j ];

			//FIXME
			//Why is this 2800 instead of B800 like the previous loop?
			//I have no idea.. Alpha channel maybe?
			if (pixel == 0x2800) {
				pixel_count++;
			}
		}
	}

	float area = (float)((surface->w)*(surface->h));
	float ratio = pixel_count/area;

	SDL_UnlockSurface(surface);

	return ratio;
}
