/*
 * KeyPressed
 *
 * Collect SDL_Events for key presses or mouse events.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include "SDL/SDL.h"

#include "Definitions.h"
#include "Coords.h"
#include "KeyPressed.h"

using namespace std;

int KeyPressed::_key_hold = 0;
bool KeyPressed::_key_down = false;
int KeyPressed::_down_key=KEY_UNPRESSED;
int KeyPressed::_mouse_x = 0;
int KeyPressed::_mouse_y = 0;

/*
 * Constructor.  Take in a keyboard or mouse SDL_Event.
 */
KeyPressed::KeyPressed() {
	SDL_Event event;
	_key_pressed=KEY_UNPRESSED;
	_mouse_pressed=0;
	_mouse_dir=5;
	_shift_pressed=false;
	_ctrl_pressed=false;

	if( SDL_PollEvent( &event ) ) {
		if ( event.type == SDL_KEYUP ) {
			_key_hold=0;
			_key_down=false;
			_down_key=KEY_UNPRESSED;
		} else if ( event.type == SDL_KEYDOWN ) {
			_key_down=true;
			switch( event.key.keysym.sym ) {
				case SDLK_BACKSPACE:
				case SDLK_SPACE:
				case SDLK_0:
				case SDLK_KP0:
					_key_pressed=KEY_0;
					break;
				case SDLK_1:
				case SDLK_KP1:
				case SDLK_z:
					_key_pressed=KEY_1;
					break;
				case SDLK_2:
				case SDLK_KP2:
				case SDLK_DOWN:
				case SDLK_x:
					_key_pressed=KEY_2;
					break;
				case SDLK_3:
				case SDLK_KP3:
				case SDLK_c:
					_key_pressed=KEY_3;
					break;
				case SDLK_4:
				case SDLK_KP4:
				case SDLK_LEFT:
				case SDLK_a:
					_key_pressed=KEY_4;
					break;
				case SDLK_5:
				case SDLK_KP5:
				case SDLK_s:
					_key_pressed=KEY_5;
					break;
				case SDLK_6:
				case SDLK_KP6:
				case SDLK_RIGHT:
				case SDLK_d:
					_key_pressed=KEY_6;
					break;
				case SDLK_7:
				case SDLK_KP7:
				case SDLK_q:
					_key_pressed=KEY_7;
					break;
				case SDLK_8:
				case SDLK_KP8:
				case SDLK_UP:
				case SDLK_w:
					_key_pressed=KEY_8;
					break;
				case SDLK_9:
				case SDLK_KP9:
				case SDLK_e:
					_key_pressed=KEY_9;
					break;
				case SDLK_TAB:
					_key_pressed=KEY_TAB;
					break;
				case SDLK_RETURN:
				case SDLK_KP_ENTER:
					_key_pressed=KEY_ENTER;
					break;
				case SDLK_HOME:
				case SDLK_KP_MULTIPLY:
					_key_pressed=KEY_HOME;
					break;
				case SDLK_ESCAPE:
					_key_pressed=KEY_ESC;
					break;
				case SDLK_PAUSE:
				case SDLK_p:
					_key_pressed=KEY_PAUSE;
					break;
				case SDLK_m:
					_key_pressed=KEY_MUTE;
					break;
				case SDLK_t:
					_key_pressed=KEY_TYPE;
					break;
				case SDLK_y:
					_key_pressed=KEY_CONFIRM;
					break;
				/* Cheats.  Commented out for live version.
				case SDLK_b:
					_key_pressed=KEY_CHEAT1;
					break;
				case SDLK_F9:
					_key_pressed=KEY_CHEAT2;
					break;
				case SDLK_F10:
					_key_pressed=KEY_CHEAT3;
					break;
				case SDLK_F11:
					_key_pressed=KEY_CHEAT4;
					break;
				case SDLK_F12:
					_key_pressed=KEY_CHEAT5;
					break;
				*/
				default:
					_key_pressed=KEY_UNKNOWN;
					break;
			}
			_down_key=_key_pressed;
		} else if ( event.type == SDL_MOUSEBUTTONDOWN ) {
			_mouse_x = event.button.x;
			_mouse_y = event.button.y;
			if (event.button.button==SDL_BUTTON_WHEELUP)
				_mouse_pressed=4;
			else if (event.button.button==SDL_BUTTON_WHEELDOWN)
				_mouse_pressed=5;
		} else if ( event.type == SDL_MOUSEBUTTONUP ) {
			int mouse_x = event.button.x;
			int mouse_y = event.button.y;
			Coords mouse_diff = Coords(mouse_x-_mouse_x,mouse_y-_mouse_y);
			float angle = mouse_diff.getAngle();
			float dist = mouse_diff.getLength();

			if (dist<10) {
				_mouse_dir = 5;
			} else if ((angle>=1*PI/8.0)&&(angle<3*PI/8)) {
				_mouse_dir=7;
			} else if ((angle>=3*PI/8.0)&&(angle<5*PI/8)) {
				_mouse_dir=8;
			} else if ((angle>=5*PI/8.0)&&(angle<7*PI/8)) {
				_mouse_dir=9;
			} else if ((angle>=7*PI/8.0)&&(angle<9*PI/8)) {
				_mouse_dir=6;
			} else if ((angle>=9*PI/8.0)&&(angle<11*PI/8)) {
				_mouse_dir=3;
			} else if ((angle>=11*PI/8.0)&&(angle<13*PI/8)) {
				_mouse_dir=2;
			} else if ((angle>=13*PI/8.0)&&(angle<15*PI/8)) {
				_mouse_dir=1;
			} else {
				_mouse_dir=4;
			}

			if (event.button.button == SDL_BUTTON_LEFT) {
				_mouse_pressed=1;
			} else if (event.button.button == SDL_BUTTON_RIGHT) {
				_mouse_pressed=2;
			}
		} else if ( event.type == SDL_QUIT ) {
			_key_pressed=KEY_QUIT;
		}
	}

	if (event.type==SDL_KEYDOWN )  {
		if (_key_down)
			_key_hold++;

		if ((event.key.keysym.mod&KMOD_SHIFT)) {
			_shift_pressed=true;
		}

		if ((event.key.keysym.mod&KMOD_CTRL)) {
			_ctrl_pressed=true;
		}
	}
}

/*
 * Returns which mouse button was pressed.
 * Mouse button 4 and 5 are mouse wheel up, down.
 */
int KeyPressed::isMousePressed() {
	return _mouse_pressed;
}

/*
 * Returns the X position of the mouse click.
 */
int KeyPressed::getMouseX() {
	return _mouse_x;
}

/*
 * Returns the Y position of the mouse click.
 */
int KeyPressed::getMouseY() {
	return _mouse_y;
}

/*
 * Returns the direction in which the mouse click was dragged.
 */
int KeyPressed::getMouseDir() {
	return _mouse_dir;
}

/*
 * Returns the key pressed.
 */
int KeyPressed::getKeyPressed() {
	return _key_pressed;
}

/*
 * Returns true if SHIFT was pressed with the key.
 */
bool KeyPressed::isShiftPressed() {
	return _shift_pressed;
}

/*
 * Returns true if CTRL was pressed with the key.
 */
bool KeyPressed::isCtrlPressed() {
	return _ctrl_pressed;
}

/*
 * Returns true if the key is being held down for multiple frames.
 */
bool KeyPressed::getKeyHold() {
	return _key_hold>2;
}
