/*
 * MenuInterfaceSurface
 *
 * Class for rendering the game's main menu.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "ScreenSurface.h"
#include "MenuInterface.h"
#include "MenuInterfaceSurface.h"

using namespace std;

/*
 * Constructor.  Must be called before blit();
 */
MenuInterfaceSurface::MenuInterfaceSurface() {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);

	_rect.x=0;
	_rect.y=0;
	_rect.w=SCREEN_WIDTH;
	_rect.h=SCREEN_HEIGHT;

	_frame_count=0;
}

/*
 * Destructor.  Frees surfaces.
 */
MenuInterfaceSurface::~MenuInterfaceSurface() {
	SDL_FreeSurface(_surface);
}

/*
 * Blits Menu to screen surface.
 */
void MenuInterfaceSurface::blit() {
	if (MenuInterface::isNewPage()) {
		_frame_count=0;
		_message = MenuInterface::getMessage();
		_options = MenuInterface::getOptions();
	}
	boxColor(_surface,0,0,SCREEN_WIDTH,SCREEN_HEIGHT,0x000000FF);
	blitMessage();
	blitOptions();
	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	_frame_count++;
	if (_frame_count>200)
		MenuInterface::longWait();
}

/*
 * Blits Menu Message to screen surface.
 */
void MenuInterfaceSurface::blitMessage() {
	int height = MenuInterface::getMessageVPos();
	int count=_frame_count*2;
	int w,h;

	TTF_Font *font=NULL;
	SDL_Color textColor = {255,255,255};

	font = Font::getTTFFont("HookedUp.ttf");

	if (font==NULL) {
		cout << "ERROR Loading Menu Interface Font\n";
		return;
	}

	if (height==-1) {
		TTF_SizeText( font, "X", &w,&h );
		height=SCREEN_CENTER-(h+2)*(_message.size())/2;
	}

	int char_count=0;
	SDL_Surface *message;
	for (int i=0;i<(int)_message.size();i++) {
		string this_line = _message[i];
		int mes_len = this_line.length();
		TTF_SizeText( font, this_line.c_str(), &w,&h );

		SDL_Rect message_rect;
		message_rect.x = (SCREEN_WIDTH-w) / 2;
		message_rect.y = height+i*(h+2);

		if (count<char_count+mes_len) {
			this_line = this_line.substr(0,count-char_count);
			message = TTF_RenderText_Blended( font, this_line.c_str(), textColor );
			SDL_BlitSurface(message,NULL,_surface,&message_rect);
			SDL_FreeSurface(message);
			break;
		} else {
			message = TTF_RenderText_Blended( font, this_line.c_str(), textColor );
			SDL_BlitSurface(message,NULL,_surface,&message_rect);
			SDL_FreeSurface(message);
			char_count+=mes_len;
		}

	}

}

/*
 * Blit Menu options to the screen surface.
 */
void MenuInterfaceSurface::blitOptions() {
	if (_options.size()==0)
		return;

	int selected = MenuInterface::getSelectedOption();
	int height = MenuInterface::getOptionsVPos();
	Font::setFont("9x18B.fnt");

	int offset=0;
	for (int i=0;i<(int)_options.size();i++) {
		string this_line = _options[i];

		if(i==selected)
			this_line="> "+this_line;
		else
			this_line="  "+this_line;

		size_t found=this_line.substr(0,46).rfind(" ");
		if ((this_line.length()>46)&&(found!=string::npos)) {
			string remainder = "   "+this_line.substr(found,this_line.length());
			this_line = this_line.substr(0,found);
			stringColor(_surface,300,height+20*(i+offset),this_line.c_str(),0xFFFFFFFF);
			offset++;
			stringColor(_surface,300,height+20*(i+offset),remainder.c_str(),0xFFFFFFFF);
		} else {
			stringColor(_surface,300,height+20*(i+offset),this_line.c_str(),0xFFFFFFFF);
		}
	}

	Font::setFont("default");
}
