/*
 * Messages
 *
 * Back end infrastructure for queueing and displaying game messages.
 * Game messages are queued and scroll upwards as they can be displayed.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <iostream>
#include <sstream>
#include <vector>

#include "SDL/SDL.h"

#include "Messages.h"

using namespace std;

vector<std::string> Messages::_messages;
vector<std::string> Messages::_message_queue;
vector<Sint32> Messages::_messages_color;
vector<Sint32> Messages::_message_queue_color;
bool Messages::_updated;

/*
 * Constructor.
 */
Messages::Messages() {
	_updated=true;
}

/*
 * Adds a new message to the message queue
 */
void Messages::addMessageToQueue(string s,Sint32 color) {
	_message_queue.push_back(s);
	_message_queue_color.push_back(color);
	_updated=true;
}

/*
 * Adds a new white message to the message queue.
 */
void Messages::addMessageToQueue(string s) {
	addMessageToQueue(s,0xFFFFFFFF);
}

/*
 * Get the number of displayed messages.
 */
int Messages::getMessagesSize() {
	return _messages.size();
}

/*
 * Get the number of messages in the message queue.
 */
int Messages::getMessageQueueSize() {
	return _message_queue.size();
}

/*
 * Take one message off the message queue and add it to the messages.
 */
void Messages::advanceMessageQueue() {
	if (_message_queue.size()==0)
		return;

	string message = _message_queue[0];

	Sint32 color = _message_queue_color[0];

	if (message.length()>34) {
		//message too long!
		size_t found=message.substr(0,34).rfind(" ");
		if ((found!=string::npos)&&(found>3)) {
			string remainder = "  "+message.substr(found+1,message.length());
			message = message.substr(0,found);
			_message_queue[0] = remainder;
		} else {
			//No spaces in string? weird...
			_message_queue[0] = "  "+message.substr(33,message.length());
			message = message.substr(0,33)+"-";
 		}
	} else {
		_message_queue.erase(_message_queue.begin());
		_message_queue_color.erase(_message_queue_color.begin());
	}

	_messages.push_back(message);
	_messages_color.push_back(color);

	if (_messages.size()>=17) {
		_messages.erase(_messages.begin());
		_messages_color.erase(_messages_color.begin());
	}

}

/*
 * Get a specific line in the current messages
 */
string Messages::getMessage(const int i) {
	return _messages[i];
}

/*
 * Get the colour of a line in the current messages.
 */
Sint32 Messages::getMessageColor(const int i) {
	if (_messages_color[i]==0xFF0000FF) {
		if (SDL_GetTicks()/1000%2==0)
			return _messages_color[i];
		else
			return _messages_color[i]&0x3F3F3FFF;
	}

	return _messages_color[i];
}

/*
 * Returns true if there are new messages to be displayed.
 */
bool Messages::isUpdated() {
	if (_updated) {
		_updated=false;
		return true;
	}
	return false;
}

/*
 * Resets messages and message queue in preparation for game start.
 *  (but not a new multiplayer game)
 */
void Messages::reset() {
	_messages.erase(_messages.begin(),_messages.end());
	_message_queue.erase(_message_queue.begin(),_message_queue.end());
	_messages_color.erase(_messages_color.begin(),_messages_color.end());
	_message_queue_color.erase(_message_queue_color.begin(),_message_queue_color.end());
	_updated=true;
}
