/*
 * MinimapSurface.
 *
 * Renders the minimap halo galaxy, level 0 stars and mini information surface to the screen surface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "SoundSystem.h"
#include "FrameTimer.h"
#include "Ship.h"
#include "View.h"
#include "Coords.h"
#include "Artifact.h"
#include "Star.h"
#include "ScreenSurface.h"
#include "InformationSurface.h"
#include "GalaxySurface.h"
#include "MinimapSurface.h"

using namespace std;

SDL_Surface *MinimapSurface::_surface;
SDL_Surface *MinimapSurface::_galaxy_surface;
SDL_Surface *MinimapSurface::_galaxy_halo;
SDL_Surface *MinimapSurface::_mini_information_surface;
SDL_Surface *MinimapSurface::_mini_border_surface;
SDL_Surface *MinimapSurface::_cursor_surface;
SDL_Surface *MinimapSurface::_artifact_cursor_surface;
SDL_Surface *MinimapSurface::_ship_surface;
SDL_Surface *MinimapSurface::_time_surface;
SDL_Surface *MinimapSurface::_fps_surface;
SDL_Rect MinimapSurface::_rect;
SDL_Rect MinimapSurface::_cursor_rect;
SDL_Rect MinimapSurface::_artifact_cursor_rect;
vector<SDL_Rect> MinimapSurface::_ship_rect;
SDL_Rect MinimapSurface::_time_rect;
SDL_Rect MinimapSurface::_fps_rect;

/*
 * Constructor.  Creates and pre-loads surfaces.  Must be instantiated before blit().
 */
MinimapSurface::MinimapSurface() {
	_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2/3, SCREEN_CENTER*2/3, SCREEN_DEPTH,0,0,0,0);
	_galaxy_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2/3, SCREEN_CENTER*2/3, SCREEN_DEPTH,0,0,0,0);

	_mini_information_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2/3, SCREEN_CENTER*2/3, SCREEN_DEPTH,0,0,0,0);

	_mini_border_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2/3, SCREEN_CENTER*2/3, SCREEN_DEPTH,0,0,0,0);

	SDL_SetColorKey(_mini_information_surface, SDL_SRCCOLORKEY, 0x00000000);
	SDL_SetColorKey(_mini_border_surface, SDL_SRCCOLORKEY, 0x00000000);
	SDL_SetAlpha(_mini_information_surface,SDL_SRCALPHA,127);

	_cursor_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, CURSOR_SIZE,CURSOR_SIZE, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_cursor_surface, SDL_SRCCOLORKEY, 0x00000000);

	_artifact_cursor_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, CURSOR_SIZE,CURSOR_SIZE, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_artifact_cursor_surface, SDL_SRCCOLORKEY, 0x00000000);


	_ship_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, CURSOR_SIZE,CURSOR_SIZE, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_ship_surface, SDL_SRCCOLORKEY, 0x00000000);

	_time_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, 44,14, SCREEN_DEPTH,0,0,0,0);
	_fps_surface = SDL_CreateRGBSurface(SDL_HWSURFACE, 44,14, SCREEN_DEPTH,0,0,0,0);

	_rect.x=SCREEN_CENTER*8/3;
	_rect.y=0;
	_rect.w=SCREEN_CENTER*2/3;
	_rect.h=SCREEN_CENTER*2/3;

	_cursor_rect.x=0;
	_cursor_rect.y=0;
	_cursor_rect.w=CURSOR_SIZE;
	_cursor_rect.h=CURSOR_SIZE;

	_artifact_cursor_rect.x=0;
	_artifact_cursor_rect.y=0;
	_artifact_cursor_rect.w=CURSOR_SIZE;
	_artifact_cursor_rect.h=CURSOR_SIZE;

	_time_rect.x=SCREEN_CENTER*2/3-45;
	_time_rect.y=SCREEN_CENTER*2/3-15;
	_time_rect.w=45;
	_time_rect.h=15;

	_fps_rect.x=3;
	_fps_rect.y=SCREEN_CENTER*2/3-15;
	_fps_rect.w=45;
	_fps_rect.h=15;

	_galaxy_halo = IMG_Load("./Sheets/Galaxy160.jpg");

 	blitMiniGalaxy();
}

/*
 * Destructors.  Frees surfaces.
 */
MinimapSurface::~MinimapSurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_galaxy_surface);
	SDL_FreeSurface(_mini_information_surface);
	SDL_FreeSurface(_mini_border_surface);
	SDL_FreeSurface(_cursor_surface);
	SDL_FreeSurface(_artifact_cursor_surface);
	SDL_FreeSurface(_ship_surface);
	SDL_FreeSurface(_time_surface);
	SDL_FreeSurface(_fps_surface);
	SDL_FreeSurface(_galaxy_halo);
}

/*
 * Blits halo, level 0 stars, ship cursor, opponent's ship cursors, selection cursor
 * Game time, FPS, and artifact cursor at game end.
 */
void MinimapSurface::blit() {
	if (View::isUpdateOverride()) {
		blitMiniGalaxy();
		InformationSurface::blit(true);
		SDL_BlitSurface(_mini_information_surface,NULL,_surface,NULL);
		SDL_BlitSurface(_mini_border_surface,NULL,_surface,NULL);
	} else {
		for (signed int i=0;i<_ship_rect.size();i++) {
			SDL_BlitSurface(_galaxy_surface,&(_ship_rect[i]),_surface,&(_ship_rect[i]));
			SDL_BlitSurface(_mini_information_surface,&(_ship_rect[i]),_surface,&(_ship_rect[i]));
			SDL_BlitSurface(_mini_border_surface,&(_ship_rect[i]),_surface,&(_ship_rect[i]));
		}

		SDL_BlitSurface(_galaxy_surface,&_cursor_rect,_surface,&_cursor_rect);
		SDL_BlitSurface(_mini_information_surface,&_cursor_rect,_surface,&_cursor_rect);
		SDL_BlitSurface(_mini_border_surface,&_cursor_rect,_surface,&_cursor_rect);

		SDL_BlitSurface(_galaxy_surface,&_artifact_cursor_rect,_surface,&_artifact_cursor_rect);
		SDL_BlitSurface(_mini_information_surface,&_artifact_cursor_rect,_surface,&_artifact_cursor_rect);
		SDL_BlitSurface(_mini_border_surface,&_artifact_cursor_rect,_surface,&_artifact_cursor_rect);

	}

	blitShips();
	blitCursor();
	if (GalaxySurface::getArtifactCursorCheat()||Artifact::isBetweenGames())
		blitArtifactCursor();
	blitTime();
	blitFPS();

	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
}

/*
 * Blits the galaxy halo and level 0 stars.
 */
void MinimapSurface::blitMiniGalaxy() {
	vector<Star> *star = Star::getStars();

	SDL_BlitSurface(_galaxy_halo,NULL,_galaxy_surface,NULL);
	rectangleColor(_galaxy_surface,0,0,SCREEN_CENTER*2/3-1,SCREEN_CENTER*2/3-1,0x3F3FFFFF);

	//This cruft here was to try to get zooming minimap.  Look for this in Release D.
	//int old_location[7] = {0,0,0,0,0,0,0};

	int location[7] = {0,0,0,0,0,0,0};
	int level =0;

	//Star::removeStars(1);
	//Star::addStars(old_location,location,level);

	//for (int i=(*star).size()-1;i>=0;i--) {
	for (int i=SEED_STARS-1;i>=0;i--) {
		int x_pos,y_pos;
		Coords offset = View::getOffset(location,level);
		Coords delta = (*star)[i].getPosition()-offset;
		delta.getIntValues(&x_pos,&y_pos,View::getScale(level));

		if ((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)) {
			x_pos/=3;
			y_pos/=3;
			Sint32 star_color = (*star)[i].getStarColor(level) & 0xFFFFFF3F;
			filledCircleColor(_galaxy_surface,x_pos,y_pos,(*star)[i].getSize(level)/3,star_color);
		}
	}

	stringColor(_galaxy_surface,50,5,"Minimap",0xFFFFFFFF);

	SDL_BlitSurface(_galaxy_surface,NULL,_surface,NULL);
}

/*
 * Blits green cursor to the screen surface.
 * ***NOTE!!!*** This is to be fixed.  This should show up as a highlighed white sector square
 *   in a future release.
 */
bool MinimapSurface::blitCursor() {
	Coords delta = View::getOffset();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,100000);

	if (!((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)))
		return false;

	x_pos/=3;
	y_pos/=3;

	int i=(-SDL_GetTicks()/5)%360;

	boxColor(_cursor_surface,0,0,CURSOR_SIZE,CURSOR_SIZE,0x000000FF);

	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,270+i,360+i,0x3FFF3FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,90+i,180+i,0x3FFF3FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,270+i,3600+i,0x3FFF3FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,90+i,180+i,0x3FFF3FFF);

	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,9,270+i,360+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,9,90+i,180+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,7,270+i,360+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,7,90+i,180+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,5,270+i,360+i,0x0F0F0FFF);
	arcColor(_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,5,90+i,180+i,0x0F0F0FFF);
	SDL_Rect location_rect;
	location_rect.x = x_pos-CURSOR_SIZE/2;
	location_rect.y = y_pos-CURSOR_SIZE/2;
	location_rect.w = location_rect.h = CURSOR_SIZE;

	SDL_Rect source_rect;
	source_rect.x=source_rect.y=0;
	source_rect.w = source_rect.h = CURSOR_SIZE;

	//Ensure cursor doesn't get pushed down or right if at edge of screen
	if (location_rect.x<0) {
		source_rect.x-=location_rect.x;
		source_rect.w+=location_rect.x;
		location_rect.x=0;
	}

	if (location_rect.y<0) {
		source_rect.y-=location_rect.y;
		source_rect.h+=location_rect.y;
		location_rect.y=0;
	}

	if (location_rect.x+location_rect.w>=SCREEN_CENTER*2/3) {
		source_rect.w=SCREEN_CENTER*2/3-location_rect.x;
	}

	if (location_rect.y+location_rect.h>=SCREEN_CENTER*2/3) {
		source_rect.h=SCREEN_CENTER*2/3-location_rect.y;
	}

	SDL_BlitSurface(_cursor_surface,&source_rect,_surface,&location_rect);

	_cursor_rect = location_rect;

	return true;
}

/*
 * Blits opponent ships to the minimap.
 */
void MinimapSurface::blitShips() {
	_ship_rect.erase(_ship_rect.begin(),_ship_rect.end());
	map<int,Ship>::iterator iter;
	map<const int, Ship, map_int_compare> *ships = Ship::getShips();
	for( iter = (*ships).begin(); iter!=(*ships).end(); ++iter ) {
		Ship other_ship = (*ships)[iter->first];
		blitShip(other_ship);
	}

	Ship my_ship= *Ship::getMyShip();
	blitShip(my_ship);

}

/*
 * Blits a single ship to the minimap.
 */
bool MinimapSurface::blitShip(Ship my_ship) {
	Coords delta = my_ship.getShipCoords();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,100000);

	Sint32 ship_color = 0xFF3F3FFF;
	ship_color = my_ship.getShipColor();

	if (!((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)))
		return false;

	x_pos/=3;
	y_pos/=3;

	int i=90+(-SDL_GetTicks()/5)%360;

	boxColor(_ship_surface,0,0,CURSOR_SIZE,CURSOR_SIZE,0x000000FF);

	filledPieColor(_ship_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,270+i,360+i,ship_color);
	filledPieColor(_ship_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,8,90+i,180+i,ship_color);
	filledCircleColor(_ship_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,5,0x000000FF);

	SDL_Rect location_rect;
	location_rect.x = x_pos-CURSOR_SIZE/2;
	location_rect.y = y_pos-CURSOR_SIZE/2;
	location_rect.w = location_rect.h = CURSOR_SIZE;

	SDL_Rect source_rect;
	source_rect.x=source_rect.y=0;
	source_rect.w = source_rect.h = CURSOR_SIZE;

	//Ensure cursor doesn't get pushed down or right if at edge of screen
	if (location_rect.x<0) {
		source_rect.x-=location_rect.x;
		source_rect.w+=location_rect.x;
		location_rect.x=0;
	}

	if (location_rect.y<0) {
		source_rect.y-=location_rect.y;
		source_rect.h+=location_rect.y;
		location_rect.y=0;
	}

	if (location_rect.x+location_rect.w>=SCREEN_CENTER*2/3) {
		source_rect.w=SCREEN_CENTER*2/3-location_rect.x;
	}

	if (location_rect.y+location_rect.h>=SCREEN_CENTER*2/3) {
		source_rect.h=SCREEN_CENTER*2/3-location_rect.y;
	}

	SDL_BlitSurface(_ship_surface,&source_rect,_surface,&location_rect);

	_ship_rect.push_back(location_rect);

	return true;
}

/*
 * Blits the game time to the minimap surface.
 */
void MinimapSurface::blitTime() {
	stringstream ss;
	string time;

	int time_elapsed = FrameTimer::getTimeElapsed();

	int secs = time_elapsed;
	int mins = secs/60;
	secs -= mins*60;


	if (mins>99) {
		int hours = mins/60;
		mins-=hours*60;

		ss << hours;
		string hours_str = "  "+ss.str();
		hours_str = hours_str.substr(hours_str.length()-2,2);
		ss << mins;
		string mins_str = "00"+ss.str();
		mins_str = mins_str.substr(mins_str.length()-2,2);
		ss.str(string());

		time = hours_str+"h"+mins_str;
	} else {
		ss << mins;
		string mins_str = "00"+ss.str();
		mins_str = mins_str.substr(mins_str.length()-2,2);
		ss.str(string());

		ss << secs;
		string secs_str = "00"+ss.str();
		secs_str = secs_str.substr(secs_str.length()-2,2);
		time = mins_str+":"+secs_str;
	}

	boxColor(_time_surface,0,0,45,15,0x000000FF);
	Font::setFont("9x15B.fnt");
	stringColor(_time_surface,0,0,time.c_str(),0xFFFFFFFF);
	Font::setFont("default");

	SDL_BlitSurface(_time_surface,NULL,_surface,&_time_rect);
}

/*
 * Blits the FPS (Frames per second) to the minimap surface
 */
void MinimapSurface::blitFPS() {
	stringstream ss;

	int fps = FrameTimer::getFPS();

	ss << fps << "fps";

	boxColor(_fps_surface,0,0,45,15,0x000000FF);
	Font::setFont("9x15B.fnt");
	stringColor(_fps_surface,0,0,ss.str().c_str(),0xFFFFFFFF);
	Font::setFont("default");

	SDL_BlitSurface(_fps_surface,NULL,_surface,&_fps_rect);
}

/*
 * Blits the artifact cursor to the minimap surface.
 * Dislayed at end of network game or if cheating during debugging.
 */
bool MinimapSurface::blitArtifactCursor() {
	Coords delta = Artifact::getPrizeStar().getPosition();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,100000);

	if (!((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)))
		return false;

	x_pos/=3;
	y_pos/=3;

	int i=(-SDL_GetTicks()/5)%360;

	boxColor(_artifact_cursor_surface,0,0,CURSOR_SIZE,CURSOR_SIZE,0x000000FF);

	filledPieColor(_artifact_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,270+i,360+i,0xFFFFFFFF);
	filledPieColor(_artifact_cursor_surface,CURSOR_SIZE/2,CURSOR_SIZE/2,6,90+i,180+i,0xFFFFFFFF);

	SDL_Rect location_rect;
	location_rect.x = x_pos-CURSOR_SIZE/2;
	location_rect.y = y_pos-CURSOR_SIZE/2;
	location_rect.w = location_rect.h = CURSOR_SIZE;

	SDL_Rect source_rect;
	source_rect.x=source_rect.y=0;
	source_rect.w = source_rect.h = CURSOR_SIZE;

	//Ensure cursor doesn't get pushed down or right if at edge of screen
	if (location_rect.x<0) {
		source_rect.x-=location_rect.x;
		source_rect.w+=location_rect.x;
		location_rect.x=0;
	}

	if (location_rect.y<0) {
		source_rect.y-=location_rect.y;
		source_rect.h+=location_rect.y;
		location_rect.y=0;
	}

	if (location_rect.x+location_rect.w>=SCREEN_CENTER*2/3) {
		source_rect.w=SCREEN_CENTER*2/3-location_rect.x;
	}

	if (location_rect.y+location_rect.h>=SCREEN_CENTER*2/3) {
		source_rect.h=SCREEN_CENTER*2/3-location_rect.y;
	}

	SDL_BlitSurface(_artifact_cursor_surface,&source_rect,_surface,&location_rect);

	_artifact_cursor_rect = location_rect;

	return true;
}

/*
 * Returns the Minimap Information surface (blitted to directly by InformationSurface.
 */
SDL_Surface *MinimapSurface::getMiniInformationSurface() {
	return _mini_information_surface;
}

/*
 * Returns the Minimap Information border surface. (Blitted to directly by Information Surface).
 */
SDL_Surface *MinimapSurface::getMiniBorderSurface() {
	return _mini_border_surface;
}
