/*
 * NetworkConnection
 *
 * Wrapper class for network communication, used by both client and server.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <string>
#include <vector>
#include <pthread.h>

#include "SDL/SDL.h"
#include "SDL/SDL_net.h"

#include "Definitions.h"
#include "Crypto.h"

#include "NetworkEnvelope.h"
#include "NetworkConnection.h"

using namespace std;

list<int> NetworkConnection::_used_ids;
NetworkConnection NetworkConnection::_client_network_connection = NetworkConnection(NULL);
bool NetworkConnection::_no_client_network_connection=true;

/*
 * Constructor.  Create a new network connection based on a TCPsocket.
 */
NetworkConnection::NetworkConnection(TCPsocket socket) {
	_socket=socket;
	_not_temp=false;
	_active=0;
	if (_socket==NULL) {
		_id=-1;
		return;
	}

	_message_queue="";

	_id=0;

	for (list<int>::iterator it = _used_ids.begin(); it != _used_ids.end(); it++) {
		if (_id<*it)
			break;
		_id=(*it)+1;

	}

	_used_ids.push_back(_id);
	_used_ids.sort();
	_last_recv_time=SDL_GetTicks();
	_last_send_time=SDL_GetTicks();
	_last_ping_send_time=SDL_GetTicks();
}

/*
 * Copy Constructor.
 */
NetworkConnection::NetworkConnection(const NetworkConnection &nc) {
	_socket=nc._socket;
	_message_queue=nc._message_queue;
	_ident=nc._ident;
	_id=nc._id;
	_not_temp=nc._not_temp;
	_active=nc._active;
	_last_recv_time=nc._last_recv_time;
	_last_send_time=nc._last_send_time;
	_last_ping_send_time=nc._last_ping_send_time;
}

/*
 * Destructor.  Closes socket if this is a _not_temp NetworkConnection.
 */
NetworkConnection::~NetworkConnection() {
	//NetworkConnection Destructors are called in code if the copy constructor is called.
	//The temporary copy calls its destructor and could clobber the socket.
	//So we have to be very careful here only to close the socket if this is the real thing.
	if (_not_temp) {
		SDLNet_TCP_Close(_socket);
		destroy();
	}
}

/*
 * Send a message via the network connection.
 */
bool NetworkConnection::sendMessage(string message) {
	message = Crypto::encryptMessage(message);

	if (_socket==NULL)
		return false;

	NetworkEnvelope (*ne) = new NetworkEnvelope(&_socket, message);
	(*ne).send();

	return true;
}

/*
 * Receive a message via the network connection.
 */
string NetworkConnection::receiveMessage() {
	int result;
	char msg[1024];

	string message="";

	SDLNet_SocketSet set=SDLNet_AllocSocketSet(1);
	SDLNet_TCP_AddSocket(set,_socket);
	SDLNet_CheckSockets(set, 0);
	bool socket_ready = SDLNet_SocketReady(_socket);

	if (socket_ready) {
		result=SDLNet_TCP_Recv(_socket,msg,1024);

		if(result>0) {
			message+= Crypto::decryptMessage(string(msg,result));
		}
	}

	SDLNet_FreeSocketSet(set);

	return message;

}

/*
 * Add a command to be sent over the network connection to the message queue.
 */
void NetworkConnection::queueMessage(string command) {
	_message_queue+=command+"\n";
}

/*
 * Add a message to be sent over the network connection to the message queue.
 */
void NetworkConnection::queueMessage(string command, string message) {
	_message_queue+=command+" "+message+"\n";
}

/*
 * Returns the message queue.
 */
string NetworkConnection::getMessageQueue() {
	return _message_queue;
}

/*
 * Send the message queue in a NetworkEvelope.
 */
bool NetworkConnection::send() {
	if (_message_queue!="") {
		_last_send_time = SDL_GetTicks();

		bool result=false;
		if (_message_queue.length()>512) {
			size_t found=_message_queue.substr(0,512).rfind("\n");
			if (found!=string::npos) {
				string remainder = _message_queue.substr(found+1,_message_queue.length());
				_message_queue = _message_queue.substr(0,found+1);
				result = sendMessage(_message_queue);
				_message_queue=remainder;
			} else {
				//512 with no \n?  has to be an error...
				//Do nothing.
			}

		} else {
			result = sendMessage(_message_queue);
			_message_queue="";
		}

		return result;
	} else {
		return true;
	}
}

/*
 * Receive messages from the network connection and return a vector of message strings.
 */
vector<string> NetworkConnection::receive() {
	vector<string> result;
	string message = receiveMessage();
	while (true) {
		size_t enter=message.find("\n");
		if( enter == string::npos )
			break;
		result.push_back(message.substr(0,enter));
		message=message.substr(enter+1,message.length());
	}
	return result;
}

/*
 * Manually disconnect the network socket.
 */
void NetworkConnection::destroy() {
	if (_socket!=NULL)
		SDLNet_TCP_Close(_socket);
	_socket=NULL;
	_used_ids.remove(_id);
}

/*
 * Game server sets the ident (Captain Name) of the client.
 */
void NetworkConnection::setIdent(string s) {
	_ident = s;
}

/*
 * Returns the ident (Captain Name) of the client for game server.
 */
string NetworkConnection::getIdent() {
	return _ident;
}

/*
 * Returns a reference to the game client's single static NetworkConnection object.
 * Used by multiple classes to pass messages to the server.
 */
NetworkConnection *NetworkConnection::getClientNetworkConnection() {
	if (_no_client_network_connection)
		return NULL;
	return &_client_network_connection;
}

/*
 * Voids the existing game client's NetworkConnection, and closes the socket if open.
 */
void NetworkConnection::nullClientNetworkConnection() {
	if ((_client_network_connection._not_temp)&&(_client_network_connection._socket)) {
		_client_network_connection.closeConnection();
		_client_network_connection=NetworkConnection(NULL);
	}
	_no_client_network_connection=true;

}

/*
 * Returns the unique ID (Client Player number) of the connection for the game server.
 */
int NetworkConnection::getID() {
	return _id;
}

/*
 * Close the connection without worrying about Idents or IDs
 * Used to kick players who've connected but haven't joined the game yet.
 */
void NetworkConnection::closeConnection() {
	SDLNet_TCP_Close(_socket);
}

/*
 * Initializes the game client connection.
 */
void NetworkConnection::initClient() {
	SDLNet_Init();
   	string hostname = string(SERVER_NAME);
   	int port = SERVER_PORT;
   	IPaddress ip_address;

	SDLNet_ResolveHost(&ip_address, hostname.c_str(), (Uint16) port);
	TCPsocket client_socket=SDLNet_TCP_Open(&ip_address);
	if (client_socket!=NULL) {
		_client_network_connection = NetworkConnection(client_socket);
		_client_network_connection._not_temp=true;
		_no_client_network_connection = false;
	} else {
		_client_network_connection=NULL;
		_no_client_network_connection = true;
	}
}

/*
 * Returns the IP address of the machine the NetworkConnection is connected to as a
 *  fixed length string.
 */
string NetworkConnection::getIPAddress() {
	IPaddress *remote_ip;
	if (_socket==NULL)
		return "Disconnected";
	remote_ip=SDLNet_TCP_GetPeerAddress(_socket);
	if (remote_ip==NULL)
		//Error
		return "Unknown";
	Uint32 host = (*remote_ip).host;
	Uint32 a,b,c,d;

	a=host>>24;
	b=host>>16;
	c=host>>8;
	d=host;
	a &=0x000000FF;
	b &=0x000000FF;
	c &=0x000000FF;
	d &=0x000000FF;

	stringstream ss;
	ss << d<<"."<<c<<"."<<b<<"."<<a;
	return ss.str();
}

/*
 * Returns true if the player has moved their ship this game. (Inactive players get kicked)
 */
bool NetworkConnection::isActive() {
	//Initial SHIP at login means that at least one SHIP gets sent, even
	//by inactive users...
	return (_active>1);
}

/*
 * Increments the active counter, measuring if a client has moved their ship this game.
 */
void NetworkConnection::setActive(bool active) {
	if (active)
		++_active;
	else
		_active=0;
}

/*
 * Sets the last time a PING command was sent.
 */
void NetworkConnection::setLastPingSendTime() {
	_last_ping_send_time = SDL_GetTicks();
}

/*
 * Set the last time data was received over the conection.
 */
void NetworkConnection::setLastRecvTime() {
	_last_recv_time = SDL_GetTicks();
}

/*
 * Returns the last time a PING command was sent.
 */
int NetworkConnection::getTimeSinceLastPingSend() {
	//FIXME - every 49 days, this will return a false value as SDL_GetTicks wraps around.
	//This means every 49 days, players will get kicked from the server.
	return SDL_GetTicks()-_last_ping_send_time;;
}

/*
 * Returns the last time data was received over the connection.
 */
int NetworkConnection::getTimeSinceLastRecv() {
	//FIXME - every 49 days, this will return a false value as SDL_GetTicks wraps around.
	//This means every 49 days, players will get kicked from the server.
	return SDL_GetTicks()-_last_recv_time;
}

/*
 * Send PING command after 5 seconds of inactivity, to keep connection alive.
 */
void NetworkConnection::sendPing() {
	if ((getTimeSinceLastRecv()>5000)&&(getTimeSinceLastPingSend()>1000)) {
		queueMessage("PING");
		setLastPingSendTime();
	}
}
