/*
 * NetworkEnvelope
 *
 * Threaded wrapper class for SDLNet_TCP_Send, which can block in certain circumstances.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <string>
#include <pthread.h>

#include "SDL/SDL.h"
#include "SDL/SDL_net.h"

#include "Crypto.h"
#include "NetworkEnvelope.h"

using namespace std;

list<NetworkEnvelope> NetworkEnvelope::_unsent;
int NetworkEnvelope::_last_id=0;

/*
 * Constructor.  Prepares a NetworkEnvelop with a socket and message.
 */
NetworkEnvelope::NetworkEnvelope(TCPsocket *socket, string message) {
	_socket=socket;
	_message=message;
	_id = _last_id++;
}

/*
 * Copy Constructor
 */
NetworkEnvelope::NetworkEnvelope(const NetworkEnvelope &ne) {
	_socket=ne._socket;
	_message=ne._message;
	_id=ne._id;
}

/*
 * Destructor.
 */
NetworkEnvelope::~NetworkEnvelope() {
}

/*
 * Equality Operator.  Was used in a map that was causing things to crash.  No longer used.
 */
bool NetworkEnvelope::operator==(const NetworkEnvelope &ne) {
	return (_id==ne._id);
}

/*
 * Thread to send message.
 * SDLNet_TCP_Send can hang in strange circumstances.  This ensures that no single user
 * can cause the server to hang for minutes or even hours.
 * Destroys the envelope after sending.
 */
void* NetworkEnvelope::sendEnvelopeThread(void* networkEnvelope) {
	NetworkEnvelope *this_envelope = (NetworkEnvelope*)networkEnvelope;

	TCPsocket* socket = (*this_envelope)._socket;

	string message = (*this_envelope)._message;

	int len = message.length();
	int ticks = SDL_GetTicks();
	if ((*socket)!=NULL) {
		int result=SDLNet_TCP_Send((*socket),message.c_str(),len);

		if (result<len)
			cout << "An error occured sending a message result = "<<result<<" len = "<<len<<endl;

		int ticks_diff = SDL_GetTicks()-ticks;
		if (ticks_diff>1000) {
			//Excessive time sending message
			cout << "WARNING!! Excessive time sending message time = "<<ticks_diff<<"ms"<<endl;
		}
	} else {
		cout << "WARNING!! Socket became null before message could be sent"<<endl;
	}

	delete this_envelope;
}

/*
 * Send the envelope via the socket.
 */
void NetworkEnvelope::send() {
	if (_socket==NULL)
		return;

	pthread_t my_thread;

	int result = pthread_create( &my_thread, NULL, &NetworkEnvelope::sendEnvelopeThread, this);
	if (result==12) {
		cout << "Network Envelope Error 12!  No new threads are being created!"<<endl;
	}

	//THIS IS CRITICAL
	//Without this the server will give error 12 on new threads after thread 380 or so.
	pthread_detach(my_thread);
}
