/*
 * NumberGenerator
 *
 * Generates evenly distributed numbers for the distribution of stars.  rand0() also replaces
 * system rand() call.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <time.h>
#include "NumberGenerator.h"

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define MASK 132459876

//Seed that _idum good!
long NumberGenerator::_idum = time(NULL)*MASK*MASK;

/*
 * Minimal Random Number Generator (Park, Miller - Numerical Recipies in C 2ndED p279)
 * The function returns a value between 0.0 and 1.0 in a consistent fashion, based on the seed value.
 * Used to generate fixed positions for stars.  Also used for game randomization.
 */
float NumberGenerator::ran0(long *idum) {
	long k;
	float ans;

	*idum ^= MASK;
	k=(*idum)/IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if (*idum < 0)
		*idum += IM;
	ans=AM*(*idum);
	*idum ^= MASK;
	return ans;
}

/*
 * Given a star number and an offset, generate a fixed number between 0.0 and 1.0 which has even
 * distribution probability, but is not random!
 * Used extensively in the program to generate a random look and feel without being random.
 */
float NumberGenerator::galaxy_rand(int star_number, int offset) {
	long seed=star_number+offset*6247;
	int i;
	float result;

	//This loop needs to be here in order to improve the random look.
	//Otherwise a distinct logarithmic spiral pattern appears in the galaxy arms.
	//It may be possible to optimize this further.
	for(i=0;i<3;i++) {
		result = ran0(&seed);
		seed= (long)(result*2147483647);
	}

	return (float)result;
}

/*
 * Integer power function.  Inefficient, but y is never larger than 6, so why not?
 */
int NumberGenerator::int_pow(int x,int y) {
	int z=1;
	for (int i=0;i<y;i++) {
		z*=x;
	}
	return z;
}

/*
 * Replacement for rand().  Uses the Park-Miller algorithm above.  Gives a nice distribution.
 */
float NumberGenerator::rand0() {
	return ran0(&_idum);
}
