/*
 * OverviewSurface
 *
 * Blits the Overview surface (sector information, star information, aliens, artifacts, etc)
 * to the screen surface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <sstream>
#include <iostream>
#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "NumberGenerator.h"
#include "Coords.h"
#include "View.h"
#include "Font.h"
#include "Artifact.h"
#include "Ship.h"
#include "Star.h"
#include "ScreenSurface.h"
#include "AlienInterfaceSurface.h"
#include "OverviewSurface.h"

using namespace std;

SDL_Surface *OverviewSurface::_surface;
SDL_Surface *OverviewSurface::_industrialist_surface;
SDL_Surface *OverviewSurface::_industrialist_face_surface;
SDL_Surface *OverviewSurface::_futurist_surface;
SDL_Surface *OverviewSurface::_futurist_jaw_surface;
SDL_Rect OverviewSurface::_rect;
SDL_Rect OverviewSurface::_industrialist_face_rect;
SDL_Rect OverviewSurface::_futurist_jaw_rect;
bool OverviewSurface::_update;
bool OverviewSurface::_alien_rendered;
int OverviewSurface::_frame_hold;
int OverviewSurface::_frame_hold_count;

/*
 * Constructor.  Prepares surfaces for rendering.  Must be instantiated before blit() is called.
 */
OverviewSurface::OverviewSurface() {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2/3, SCREEN_CENTER*2/3, SCREEN_DEPTH,0,0,0,0);

	_rect.x=SCREEN_CENTER*2;
	_rect.y=0;
	_rect.w=SCREEN_CENTER*2/3;
	_rect.h=SCREEN_CENTER*2/3;

	_futurist_jaw_rect.x=SCREEN_CENTER*2+62;
	_futurist_jaw_rect.y=71;
	_futurist_jaw_rect.w=33;
	_futurist_jaw_rect.h=25;

	_industrialist_face_rect.x=SCREEN_CENTER*2+69;
	_industrialist_face_rect.y=38;
	_industrialist_face_rect.w=39;
	_industrialist_face_rect.h=53;

	_update=true;
	_alien_rendered=false;

	_industrialist_surface = SDL_LoadBMP("Sheets/Industrialist.bmp");
	_industrialist_face_surface = SDL_LoadBMP("Sheets/Industrialist_Face.bmp");

	_futurist_surface = SDL_LoadBMP("Sheets/Futurist.bmp");
	_futurist_jaw_surface = SDL_LoadBMP("Sheets/Futurist_Jaw.bmp");

	_frame_hold_count=0;
	_frame_hold=0;
}

/*
 * Destructor.  Frees Surface.
 */
OverviewSurface::~OverviewSurface() {
	SDL_FreeSurface(_surface);
}

/*
 * Blit Overview Surface (Star View, Sector View, Aliens) to screen surface.
 */
void OverviewSurface::blit() {
	bool has_stars = Star::getVisibleStars().size()>0;

	Ship *my_ship = Ship::getMyShip();

	if (_update) {
		if ((*my_ship).isVisitingAliens()) {
			int alien_type = (*my_ship).getShipStar().getAlienType();
			if (alien_type==0) {
				blitAncient();
			} else if (alien_type==1) {
				blitIndustrialist();
			} else {
				blitFuturist();
			}
		} else if (View::isZoomedIn()&&has_stars) {
			_frame_hold=0;
			_frame_hold_count=0;
			blitStarOverview();
		} else {
			_frame_hold=0;
			_frame_hold_count=0;
			blitSectorOverview();
		}
	}

	if ((*my_ship).isVisitingAliens()) {
		_update=true;
	} else if (View::isZoomedIn()&&has_stars) {
		Ship *my_ship = Ship::getMyShip();
		Coords ship_coords = (*my_ship).getShipCoords();
		Coords cursor_coords = View::getCursorCoords();
		if (cursor_coords==ship_coords) {
			_update=false;
		} else {
			_update=true;
		}
	} else {
		if (View::isMoving()) {
			_update=true;
		} else {
			_update=false;
		}
	}
}

/*
 * Blit Sector Overview to Screen
 */
void OverviewSurface::blitSectorOverview() {
	_alien_rendered=false;
	stringstream message;
	message.precision(0);

	Ship *my_ship = Ship::getMyShip();

	int power = 6-View::getLevel();
	Coords delta = View::getOffset()-(*my_ship).getShipCoords();
	//float dist=sqrt(pow(offset.x-ship.x,2)+pow(offset.y-ship.y,2));
	float dist = delta.getLength();

	if (power<0)
		power=0;

	Font::setFont("default");

	boxColor(_surface,0,0,SCREEN_CENTER*2/3,SCREEN_CENTER*2/3,0x000000FF);
	rectangleColor(_surface,0,0,SCREEN_CENTER*2/3-1,SCREEN_CENTER*2/3-1,0x3F3FFFFF);

	message << "Sector ";
	for (int i=0;i<7;i++) {
		message << View::getLocation()[i];
		if (i==2)
			message << "-";
	}

	stringColor(_surface,80-strlen(message.str().c_str())*4,10,message.str().c_str(),0xFFFFFFFF);

	message.str(string());
	message << fixed << dist << " ly away";
	stringColor(_surface,80-strlen(message.str().c_str())*4,110,message.str().c_str(),0xFFFFFFFF);

	message.str(string());
	message << fixed << View::getScale() << " ly across";
	stringColor(_surface,80-strlen(message.str().c_str())*4,120,message.str().c_str(),0xFFFFFFFF);

	message.str(string());
	message << Star::getVisibleStars().size() << " visstars";
	stringColor(_surface,80-strlen(message.str().c_str())*4,130,message.str().c_str(),0xFFFFFFFF);

	if (View::getLevel()<6) {
		message.str(string());
		message << "(approx)";
		stringColor(_surface,80-strlen(message.str().c_str())*4,140,message.str().c_str(),0xFFFFFFFF);
	}

	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);

}

/*
 * Blit Star Overview to Screen.
 */
void OverviewSurface::blitStarOverview() {
	_alien_rendered=false;
	Ship *my_ship = Ship::getMyShip();
	Coords delta = (*my_ship).getShipCoords()-View::getCursorStar().getPosition();
	float dist = delta.getLength();

	float fuel_to_jump=dist/(*my_ship).getFuelEfficiency();
	float eta=dist/(*my_ship).getShipSpeed()*30.0/1000.0;

	stringstream message;
	message.precision(0);

	Sint32 fuel_color=0xFFFFFFFF;

	boxColor(_surface,0,0,SCREEN_CENTER*2/3,SCREEN_CENTER*2/3,0x000000FF);
	rectangleColor(_surface,0,0,SCREEN_CENTER*2/3-1,SCREEN_CENTER*2/3-1,0x3F3FFFFF);

	if (fuel_to_jump>(*my_ship).getFuel())
		fuel_color=0xFF0000FF;

	Star cursor_star = View::getCursorStar();

	int size = cursor_star.getSize();
	Sint32 color = cursor_star.getStarColor();

	Font::setFont("default");

	if (!cursor_star.isNullStar()) {
		filledCircleColor(_surface,SCREEN_CENTER/3,SCREEN_CENTER/3,(size+1)*20,color&0xFFFFFF1F);
		filledCircleColor(_surface,SCREEN_CENTER*1/3,SCREEN_CENTER*1/3,(size+1)*10,color&0xFFFFFF7F);
		filledCircleColor(_surface,SCREEN_CENTER*1/3,SCREEN_CENTER*1/3,(size+1)*5,color);

		string star_name=cursor_star.getStarName();

		int *location = View::getLocation();

		message << "Sector " << location[0] << location[1] << location[2] << "-" << location[3] << location[4] << location[5] <<location[6];

		stringColor(_surface,80-strlen(message.str().c_str())*4,10,message.str().c_str(),0xFFFFFFFF);

		stringColor(_surface,50,20,star_name.c_str(),0xFFFFFFFF);
		rectangleColor(_surface,0,0,SCREEN_CENTER*2/3,SCREEN_CENTER*2/3,0x3F3FFFFF);

		message.str(string());
		message << "Class " << cursor_star.getStarClassName();
		stringColor(_surface,80-strlen(message.str().c_str())*4,30,message.str().c_str(),0xFFFFFFFF);

		if (dist<0.0001) {
			if (cursor_star.hasAliens()) {
				message.str(string());
				message << "ALIENS";
				stringColor(_surface,80-strlen(message.str().c_str())*4,40,message.str().c_str(),0xFF0000FF);
			}
			if (cursor_star.hasArtifact()) {
				if (!cursor_star.isPlundered()) {
					message.str(string());
					message << "ARTIFACT";
					stringColor(_surface,80-strlen(message.str().c_str())*4,50,message.str().c_str(),0x00FF00FF);
				}
			}

		} else {
			message.str(string());
			message << "Distance:";
			stringColor(_surface,5,110,message.str().c_str(),0xFFFFFFFF);

			message.str(string());
			message << fixed << ((int)dist) << " ly";
			stringColor(_surface,155-strlen(message.str().c_str())*8,110,message.str().c_str(),0xFFFFFFFF);

			message.str(string());
			message << "Fuel Req:";
			stringColor(_surface,5,120,message.str().c_str(),fuel_color);

			message.str(string());
			message << fixed << ((int)fuel_to_jump) << " L";
			stringColor(_surface,155-strlen(message.str().c_str())*8,120,message.str().c_str(),fuel_color);

			message.str(string());
			message << "ETA:";
			stringColor(_surface,5,130,message.str().c_str(),fuel_color);

			message.str(string());
			message << fixed << ((int)eta) << " s";
			stringColor(_surface,155-strlen(message.str().c_str())*8,130,message.str().c_str(),fuel_color);

		}
		SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	}
}

/*
 * Reset in preparation for game restart.
 */
void OverviewSurface::reset() {
	_update=true;
}
/*
 * Blit animated Ancient Alien Image.
 */
void OverviewSurface::blitAncient() {
	//FIXME - Temporary while I wait for Ancient graphics.
	blitIndustrialist();
}

/*
 * Blit animated Industrialist Alien Image.
 */
void OverviewSurface::blitIndustrialist() {
	if (!_alien_rendered) {
		SDL_BlitSurface(_industrialist_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
		_alien_rendered=true;
	}
	int frame = 0;

	if ((AlienInterfaceSurface::isSpeaking())&&(_frame_hold==3)) {
		//Stop staring and start talking.
		_frame_hold_count=0;
	}

	if (_frame_hold_count>0) {
		frame = _frame_hold;
		--_frame_hold_count;
		if ((_frame_hold_count==0) && (_frame_hold==3)) {
			//For the following 16 frames (after this one), keep eyes open.
			_frame_hold_count=16;
			_frame_hold=0;
		}
	} else if (AlienInterfaceSurface::isSpeaking()) {
		frame = _frame_hold = ((int)(3*NumberGenerator::rand0()))%3;
		if (frame==0) {
			_frame_hold_count=0;
		} else {
			_frame_hold_count=((int)(4*NumberGenerator::rand0()));
		}
	} else if (NumberGenerator::rand0()<0.025) {
		//possibly blink on frame 3
		frame =_frame_hold = 3;
		_frame_hold_count=2;
	}

	SDL_Rect frame_rect;
	frame_rect.x=frame*39;
	frame_rect.y=0;
	frame_rect.w=39;
	frame_rect.h=53;
	SDL_BlitSurface(_industrialist_face_surface,&frame_rect,ScreenSurface::getScreenSurface(),&_industrialist_face_rect);
}


/*
 * Blit animated Futurist Alien Image.
 */
void OverviewSurface::blitFuturist() {
	if (!_alien_rendered) {
		SDL_BlitSurface(_futurist_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
		_alien_rendered=true;
	}

	int frame = 0;
	if (_frame_hold_count>0) {
		frame = _frame_hold;
		--_frame_hold_count;
	} else if (AlienInterfaceSurface::isSpeaking()) {
		frame = _frame_hold = ((int)(3*NumberGenerator::rand0()))%3;
		if (frame==0) {
			_frame_hold_count=0;
		} else {
			_frame_hold_count=((int)(4*NumberGenerator::rand0()));
		}
	}

	SDL_Rect frame_rect;
	frame_rect.x=frame*33;
	frame_rect.y=0;
	frame_rect.w=33;
	frame_rect.h=25;
	SDL_BlitSurface(_futurist_jaw_surface,&frame_rect,ScreenSurface::getScreenSurface(),&_futurist_jaw_rect);
}

