/*
 * RadarSurface
 *
 * Renders the ship radar and the radar echo information to the screen surface, as a layer
 *   displayed on top of the GalaxySurface.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "SoundSystem.h"
#include "Ship.h"
#include "View.h"
#include "Coords.h"
#include "Artifact.h"
#include "Star.h"
#include "FrameTimer.h"
#include "ScreenSurface.h"
#include "RadarSurface.h"

using namespace std;

/*
 * Constructor.  Prepares Radar Surface.  Must be instatiated before blit() is called.
 */
RadarSurface::RadarSurface() {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);
	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY, 0x00000000);
	SDL_SetAlpha(_surface,SDL_SRCALPHA,127);

	_rect.x=0;
	_rect.y=0;
	_rect.w=SCREEN_CENTER*2;
	_rect.h=SCREEN_HEIGHT;

}

/*
 * Destructor.  Frees radar surface.
 */
RadarSurface::~RadarSurface() {
	SDL_FreeSurface(_surface);
}



/*
 * Blit Radar surface, depending on Radar improvement level:
 *
 * Radar 1=No Improvements
 * Radar 2=Cursor aims radar
 * Radar 3=Beam narrowed
 * Radar 4=Alien radar blip
 * Radar 5=Artifact radar blip
 * Radar 6=Identify Alien Type
 * Radar 7=Identify Artifact Type
 * Radar 8=Identify Aritfact Price
 * Radar 9=Omnidirectional radar
 * Radar 10=Extended Range
 */
void RadarSurface::blit() {
	bool debug = false;
	if (debug) cout << endl;
	if (debug) cout << " Before Radar " << FrameTimer::getInterimFrameLength();

	Ship *my_ship = Ship::getMyShip();
	int radar = (*my_ship).getRadar();

	if ((!View::isZoomedIn()) && (!(View::isAlmostZoomedIn()&&radar>=10)))
		return;

	bool alien_match=false;
	bool artifact_match=false;

	Sint32 color;

	int beam_width_degrees=10;
	if (radar<3)
		beam_width_degrees=20;
	if (View::isAlmostZoomedIn())
		beam_width_degrees=10;

	float beam_width_radians=(((float)beam_width_degrees)/360.0)*2.0*PI;


	Coords ship = (*my_ship).getShipCoords();
	Coords offset = View::getOffset();
	Coords delta = ship-offset;
	float scale = View::getScale();
	int x_pos,y_pos;
	delta.getIntValues(&x_pos,&y_pos,scale);

	if ((x_pos<0)||(x_pos>SCREEN_CENTER*2)||(y_pos<0)||(y_pos>SCREEN_CENTER*2))
		return;

	delta = View::getCursorCoords()-ship;
	float radar_angle_theta=delta.getAngle();

	if (((radar<9)&&((delta.getLength()<0.1))||(radar==1)||View::isAlmostZoomedIn())) {
		radar_angle_theta=2.0*PI*(10000.0-(SDL_GetTicks()%10000))/10000.0;
	}

	int degrees=(int)(radar_angle_theta*180/PI)-90+1;

	boxColor(_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x000000FF);

	Font::setFont("10x20.fnt");

	if (debug) cout << " Early Radar Math " << FrameTimer::getInterimFrameLength();

	vector<Star> visible_artifact_stars = Star::getVisibleArtifactStars();

	for (int i=0;i<(int)visible_artifact_stars.size();i++) {
		delta = visible_artifact_stars[i].getPosition()-ship;
		float target_theta = delta.getAngle();

		if ((View::isZoomedIn()&&radar>=9)||((fabs(radar_angle_theta-target_theta)<=(beam_width_radians))&&(delta.getLength()>0.1))) {
			SoundSystem::playSound(ARTIFACT_SOUND);
			artifact_match=true;
			break;
		}
	}

	if (debug) cout << " After Artifact Math " << FrameTimer::getInterimFrameLength();

	vector<Star> visible_alien_stars = Star::getVisibleAlienStars();

	for (int i=0;i<(int)visible_alien_stars.size();i++) {
		delta = visible_alien_stars[i].getPosition()-ship;

		float target_theta = delta.getAngle();
		if ((View::isZoomedIn()&&radar>=9)||((fabs(radar_angle_theta-target_theta)<=(beam_width_radians))&&(delta.getLength()>0.1))) {
			SoundSystem::playSound(ALIEN_SOUND);
			alien_match=true;
			break;
		}
	}

	if (debug) cout << " After Alien Math " << FrameTimer::getInterimFrameLength();

	if (alien_match) {
		if (artifact_match) {
			color = 0xFFFFFF7F;
		} else {
			color = 0xFF3FFF7F;
		}
	} else {
		if (artifact_match) {
			color = 0x3FFFFF7F;
		} else {
			color = 0x3F3FFF7F;
		}
	}

	if ((radar<9)||(View::isAlmostZoomedIn())) {
		filledPieColor(_surface,x_pos,y_pos,(int)(SCREEN_CENTER*2.0*ROOT_2),degrees+90-beam_width_degrees,degrees+90+beam_width_degrees,color);
	} else {
		int radius = (SDL_GetTicks()*4/5)%(SCREEN_CENTER*3);
		filledCircleColor(_surface,x_pos,y_pos,radius,0x3F3FFF7F);
		if (radius>40) {
				filledCircleColor(_surface,x_pos,y_pos,radius-40,0x000000FF);
		}
	}

	if (debug) cout << " After Pie Wedge " << FrameTimer::getInterimFrameLength();

	if (artifact_match) {
		for (int i=0;i<(int)visible_artifact_stars.size();i++) {
			delta = visible_artifact_stars[i].getPosition()-ship;
			float target_theta = delta.getAngle();

			if ((View::isZoomedIn()&&radar>=9)||((fabs(radar_angle_theta-target_theta)<=(beam_width_radians))&&(delta.getLength()>0.1))) {

				delta = visible_artifact_stars[i].getPosition()-offset;
				int target_x_pos,target_y_pos;
				delta.getIntValues(&target_x_pos,&target_y_pos,scale);

				if (radar>=8) {
					switch (visible_artifact_stars[i].getArtifact().getArtifactValue()) {
						case 0:
							stringColor(_surface,target_x_pos+10,target_y_pos-20,"$",0xFFFF00FF);
							break;
						case 1:
							stringColor(_surface,target_x_pos+10,target_y_pos-20,"$$",0xFFFF00FF);
							break;
						case 2:
							stringColor(_surface,target_x_pos+10,target_y_pos-20,"$$$",0xFFFF00FF);
							break;
					}
				}

				if (radar>=7) {
					switch (visible_artifact_stars[i].getArtifact().getArtifactType()) {
						case 0:
							stringColor(_surface,target_x_pos-20,target_y_pos-20,"A",0xFFFF00FF);
							break;
						case 1:
							stringColor(_surface,target_x_pos-20,target_y_pos-20,"I",0xFFFF00FF);
							break;
						case 2:
							stringColor(_surface,target_x_pos-20,target_y_pos-20,"F",0xFFFF00FF);
							break;
					}
				}

				if (radar>=5) {
					switch (SDL_GetTicks()%800/200) {
						case 0:
							arcColor(_surface,target_x_pos,target_y_pos,3,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,315,45,0x7FFFFFFF);

							break;
						case 1:
							arcColor(_surface,target_x_pos,target_y_pos,6,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,315,45,0x7FFFFFFF);
							break;
						case 2:
							arcColor(_surface,target_x_pos,target_y_pos,9,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,315,45,0x7FFFFFFF);
							break;
						case 3:
							arcColor(_surface,target_x_pos,target_y_pos,3,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,315,45,0x7FFFFFFF);
							break;
					}
				}
			}
		}
	}

	if (debug) cout << " After Artifact Match " << FrameTimer::getInterimFrameLength();

	if (alien_match) {
		for (int i=0;i<(int)visible_alien_stars.size();i++) {
			delta = visible_alien_stars[i].getPosition()-ship;

			float target_theta = delta.getAngle();
			if ((View::isZoomedIn()&&(radar>=9))||((fabs(radar_angle_theta-target_theta)<(beam_width_radians))&&(delta.getLength()>0.1))) {

				delta = visible_alien_stars[i].getPosition()-offset;

				int target_x_pos,target_y_pos;
				delta.getIntValues(&target_x_pos,&target_y_pos,scale);

				if (radar>=6) {
					switch (visible_alien_stars[i].getAlienType()) {
						case 0:
							stringColor(_surface,target_x_pos+10,target_y_pos,"A",0xFFFFFFFF);
							break;
						case 1:
							stringColor(_surface,target_x_pos+10,target_y_pos,"I",0xFFFFFFFF);
							break;
						case 2:
							stringColor(_surface,target_x_pos+10,target_y_pos,"F",0xFFFFFFFF);
							break;
					}
				}

				if (radar>=4) {
					switch (SDL_GetTicks()%800/200) {
						case 0:
							arcColor(_surface,target_x_pos,target_y_pos,3,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,315,45,0x7FFFFFFF);

							break;
						case 1:
							arcColor(_surface,target_x_pos,target_y_pos,6,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,315,45,0x7FFFFFFF);
							break;
						case 2:
							arcColor(_surface,target_x_pos,target_y_pos,9,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,9,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,315,45,0x7FFFFFFF);
							break;
						case 3:
							arcColor(_surface,target_x_pos,target_y_pos,3,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,3,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,6,315,45,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,135,225,0x7FFFFFFF);
							arcColor(_surface,target_x_pos,target_y_pos,12,315,45,0x7FFFFFFF);
							break;
					}
				}
			}
		}
	}

	if (debug) cout << " After Alien Match " << FrameTimer::getInterimFrameLength();

	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	if (debug) cout << " After Blit " << FrameTimer::getInterimFrameLength() << endl;
}
