/*
 * Ship
 *
 * Class describing a ship's present location and upgrades.  Handles events when stars are visited.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>

#include "Coords.h"
#include "View.h"
#include "Artifact.h"
#include "Star.h"
#include "Messages.h"
#include "FrameTimer.h"
#include "NetworkConnection.h"
#include "Map.h"
#include "StringParser.h"
#include "Ship.h"

using namespace std;

Ship *Ship::_my_ship;
map<const int, Ship, map_int_compare> Ship::_ships;
bool Ship::_victory=false;
bool Ship::_failure=false;

string Ship::_upgrade_name[6] = {"Radar","Comp","Speed","Tank","Drive","Cargo"};

/*
 * Constructor.  Initializes a ship.
 */
Ship::Ship() {
	for (int i=0;i<6;i++) {
		_upgrades[i]=1;
	}
	_cash_dist = _cash = 100;
	_fuel_dist=_fuel=getMaxFuel();

	_updated=true;

	_visiting_aliens=false;
	_recheck_planet=true;
	_ship_coords = Coords(0,0);
	_arrived = true;
	_ship_id = 0;
	_captain_name = "";
	_ship_star=Star();
}

/*
 * Copy Constructor.
 */
Ship::Ship(const Ship &s) {
	for (int i=0;i<6;i++) {
		_upgrades[i]=s._upgrades[i];
		_ship_location[i]=s._ship_location[i];
	}
	_cash = s._cash;
	_cash_dist=s._cash_dist;
	_fuel=s._fuel;
	_fuel_dist=s._fuel_dist;
	_ship_coords=s._ship_coords;
	_ship_star=s._ship_star;
	_visiting_aliens=s._visiting_aliens;
	_recheck_planet=s._recheck_planet;
	_arrived = s._arrived;

	_updated=true;
	_ship_id=s._ship_id;
	_captain_name = s._captain_name;

}

/*
 * Returns actual ship cash.
 */
float Ship::getCash() {
	return _cash;
}

/*
 * Returns an interim ship cash value (used for updating cash on ShipSurface)
 */
float Ship::getCashDist() {
	return _cash_dist;
}

/*
 * Returns actual ship fuel
 */
float Ship::getFuel() {
	return _fuel;
}

/*
 * Returns an interim ship fuel value (used for updating fuel on ShipSurface)
 */
float Ship::getFuelDist() {
	return _fuel_dist;
}

/*
 * Get Radar upgrade level.
 */
int Ship::getRadar() {
	return _upgrades[UPGRADE_RADAR];
}

/*
 * Calculate fuel efficiency from Drive upgrade.
 */
float Ship::getFuelEfficiency() {
	return 10.0*_upgrades[UPGRADE_DRIVE];
}

/*
 * Returns max fuel tank size, based on Drive upgrade.
 */
float Ship::getMaxFuel() {
	return 40*_upgrades[UPGRADE_TANK];;
}

/*
 * Returns ship speed based on Speed upgrade
 */
float Ship::getShipSpeed() {
	return 0.04*pow(2.0,_upgrades[UPGRADE_SPEED]);
}

/*
 * Returns max cargo hold size based on Cargo upgrade.
 */
int Ship::getCargoHoldSize() {
	return _upgrades[UPGRADE_CARGO];;
}

/*
 * Set ship cash.
 */
void Ship::setCash(const float d) {
	_cash = d;
	_updated=true;
}

/*
 * Add to ship cash, for sale of artifacts.
 */
void Ship::addCash(const float d) {
	_cash += d;
	_updated=true;
}


/*
 * Set interim cash value.
 */
void Ship::setCashDist(const float d) {
	_cash_dist = d;
	_updated=true;
}

/*
 * Set ship fuel.
 */
void Ship::setFuel(const float d) {
	_fuel = d;
	_updated=true;
}

/*
 * Add to ship fuel.
 */
void Ship::addFuel(const float d) {
	_fuel += d;
	_updated=true;
}

/*
 * Set interim fuel value.
 */
void Ship::setFuelDist(const float d) {
	_fuel_dist = d;
	_updated=true;
}

/*
 * Updates cash interim value.  Returns true if it's are changed.
 */
bool Ship::updateCashDist() {
	if (_cash_dist==_cash)
		return false;
	_cash_dist = (5.0*_cash_dist+_cash)/6.0;
	if (fabs(_cash_dist-_cash)<=0.01)
		_cash_dist = _cash;
	_updated=true;
	return true;
}

/*
 * Updated fuel interim value.  Returns true if it's changed.
 */
bool Ship::updateFuelDist() {
	if (_fuel_dist==_fuel)
		return false;

	_fuel_dist = (5.0*_fuel_dist+_fuel)/6.0;
	if (fabs(_fuel_dist-_fuel)<=0.01)
		_fuel_dist = _fuel;
	_updated=true;
	return true;
}

/*
 * Returns the value of a specific ship upgrade.
 */
int Ship::getUpgrades(const int i) {
	return _upgrades[i];
}

/*
 * Sets the value of a specific ship upgrade.
 */
void Ship::setUpgrades(const int i,const int j) {
	_upgrades[i]=j;
	_updated=true;
}

/*
 * Increment the value of a specific ship upgrade.
 */
void Ship::incrementUpgrades(const int i) {
	_upgrades[i]++;
	_updated=true;
}

/*
 * Check planet to see if the player has won, or if they can plunder an artifact.
 */
void Ship::checkPlanet() {
	_recheck_planet=false;
	if (_ship_star.getStarNumber()<0)
		return;
	if (_ship_star.hasArtifact()) {
		if (_ship_star.getArtifact().isPrizeArtifact()) {
			//PLAYER WINS!!!!
			if (!_ship_star.isPlundered()) {
				_victory=true;
				//Ensure you can't win over and over again.
				_ship_star.plunder();
				if (_captain_name!="")
					Messages::addMessageToQueue(_captain_name+" has won the game!",getShipColor());
				else
					Messages::addMessageToQueue("You have won the game!");
			}
		}
		if (!_ship_star.isPlundered()) {
			string star_name = _ship_star.getStarName();
			Artifact a = _ship_star.getArtifact();
			string artifact_name = a.getArtifactName();
			Messages::addMessageToQueue("At "+star_name+" you find '"+artifact_name+"'!");
			if ((int)Artifact::getFoundArtifacts().size()>=getCargoHoldSize()) {
				Messages::addMessageToQueue("Your cargo hold is full!");
			} else {
				_ship_star.plunder();
				Artifact::addFoundArtifact(a);
			}
		}
	}
}

/*
 * Checks to see if there are any changes to ship location, fuel usage, cash expenditures
 * Returns true if these are updated.
 */
bool Ship::isUpdated() {
	if (updateCashDist() | updateFuelDist() )
		_updated=true;

	if ((_arrived)&&(_ship_star.getStarNumber()>=0)) {
		_arrived=false;
		string star_name = _ship_star.getStarName();
		Messages::addMessageToQueue("You have arrived at "+star_name+".");

		checkPlanet();

		if (_ship_star.hasAliens()) {
			Messages::addMessageToQueue("You have found aliens at "+star_name+".");
			_visiting_aliens=true;
		}
	}

	if ((!isMoving())&&(_recheck_planet)) {
		checkPlanet();
	}

	if (!_updated)
		return false;
	_updated=false;
	return true;
}

/*
 * Moves ship a distance based on the frame length.
 */
void Ship::moveShip() {
	if (_ship_coords!=_ship_star.getPosition()) {
		Coords delta = _ship_star.getPosition()-_ship_coords;
		float dist = delta.getLength();
		float frame_length = FrameTimer::getFrameLength();

		if (dist<(getShipSpeed()*(frame_length/30.0))) {
			_ship_coords=_ship_star.getPosition();
			_fuel-=dist/getFuelEfficiency();
			_arrived = true;
		} else {

			Coords normal = delta/delta.getLength();
			_ship_coords +=normal*getShipSpeed()*(frame_length/30.0);
			_fuel-=getShipSpeed()*(frame_length/30.0)/getFuelEfficiency();
		}
		_updated=true;
	}
}

/*
 * Move other opponent ships a distance based on the frame length.
 */
void Ship::moveShips() {
	map<int,Ship>::iterator iter;
	for( iter = _ships.begin(); iter != _ships.end(); ++iter ) {
		Ship *my_ship = Ship::getShip(iter->first);
		(*my_ship).moveShip();
	}
	(*_my_ship).moveShip();
}

/*
 * Sets the value of the static _my_ship, which is the player's ship.
 */
void Ship::setMyShip(Ship &s) {
	_my_ship=&s;
	(*_my_ship)._updated=true;
}

/*
 * Returns a reference to the player's ship.
 */
Ship *Ship::getMyShip() {
	return _my_ship;
}

/*
 * Initializes the ship location.
 * Forces a start location on one of the two minor arms, at least 40% out from the core.
 */
void Ship::initShipLocation() {
	(*_my_ship)._ship_star = Star::getRandomStar2((*_my_ship)._ship_location);
	View::setCursorStar((*_my_ship)._ship_star);
	(*_my_ship)._ship_coords = (*_my_ship)._ship_star.getPosition();
	View::setCursorCoords((*_my_ship)._ship_coords);
}

/*
 * Resets ship location.  Used to void out existing ship location on network game start.
 * Ship location is reset on receipt of ARTI command, so it's best not to set the ship
 * position twice.
 */
void Ship::nullShipLocation() {
	(*_my_ship)._ship_star = Star::nullStar();
	View::setCursorStar((*_my_ship)._ship_star);
	(*_my_ship)._ship_coords = (*_my_ship)._ship_star.getPosition();
	View::setCursorCoords((*_my_ship)._ship_coords);
}

/*
 * Returns an int[7] array of the ship's sector location.
 */
int *Ship::getShipLocation() {
	return _ship_location;
}

/*
 * Set's the int[7] array of the ship's sector location.
 */
void Ship::setShipLocation(int *ship_location) {
	for (int i=0;i<7;i++) {
		_ship_location[i] = ship_location[i];
	}
}

/*
 * Returns the ships (x,y) coords.
 */
Coords Ship::getShipCoords() {
	return _ship_coords;
}

/*
 * Sets the ship's (x,y) coords.
 */
void Ship::setShipCoords(Coords ship_coords) {
	_ship_coords = ship_coords;
}

/*
 * Returns the Star object the ship is presently at or heading to.
 */
Star Ship::getShipStar() {
	return _ship_star;
}

/*
 * Sets the Star object the ship is now heading to.
 */
void Ship::setShipStar(Star ship_star) {
	_ship_star = ship_star;
}

/*
 * A debug cheat to allow user to boost fuel, cash and upgrades.
 */
void Ship::beckyCheat() {
	Messages::addMessageToQueue("BECKY CHEAT ACTIVATED!");

	for (int i=0;i<6;i++) {
		if (i==UPGRADE_COMP)
			continue;
		_upgrades[i]+=1;
	}

	_cash=10000.0;
	_fuel=getMaxFuel();

	_updated=true;
}

/*
 * Returns true if the player has won the game.
 */
bool Ship::isVictory() {
	return _victory;
}

/*
 * Returns true if the player has lost the game.
 */
bool Ship::isFailure() {
	return _failure;
}

/*
 * Resets the victory/failure status of the player on new game start.
 */
void Ship::resetVictoryFailure() {
	_victory=false;
	_failure=false;
}

/*
 * Returns true if the player is visiting aliens.
 */
bool Ship::isVisitingAliens() {
	return _visiting_aliens;
}

/*
 * Sets if the player is visiting aliens.
 */
void Ship::setVisitingAliens(bool b) {
	_visiting_aliens = b;
	if (!b) {
		_recheck_planet=true;
	}
}

/*
 * Returns true if the ship is presently moving.
 */
bool Ship::isMoving() {
	Coords delta = _ship_star.getPosition()-_ship_coords;
	return (delta.getLength()>0);
}

/*
 * Queue a SHIP message to be sent to the game server indicating the ship has moved to other players.
 */
void Ship::queueShipDataMessage() {
	NetworkConnection *networkConnection = NetworkConnection::getClientNetworkConnection();
	stringstream ss;
	ss.precision(3);
	ss << fixed;
	ss << _ship_coords.getX() << " "<<_ship_coords.getY() << " ";
	ss << _ship_star.getPosition().getX() << " "<<_ship_star.getPosition().getY();
	for (int i=0;i<6;i++)
		ss << " " << _upgrades[i];
	if (networkConnection!=NULL) {
		(*networkConnection).queueMessage("SHIP",ss.str());
	}
}

/*
 * Handle an IDEN message, identifying the players ID number and name.
 */
void Ship::handleIdenMessage(string message) {
	int ship_id = StringParser::skimInt(message);
	string captain_name = message;

	(*_my_ship)._ship_id = ship_id;
	(*_my_ship)._captain_name = captain_name;

	Messages::addMessageToQueue("You are Captain "+captain_name+".",(*_my_ship).getShipColor());
}

/*
 * Handle a PART message, sent when another player leaves the game.
 */
void Ship::handlePartMessage(string message) {
	int ship_id = StringParser::skimInt(message);

	Ship *this_ship = getShip(ship_id);
	string captain_name = (*this_ship)._captain_name;
	Messages::addMessageToQueue(captain_name+" has left the game.",(*this_ship).getShipColor());

	removeShip(ship_id);
}

/*
 * Handle a TALK message, when another player sends a chat message.
 */
void Ship::handleTalkMessage(string message) {
	int ship_id = StringParser::skimInt(message);

	Ship *this_ship;
	if (ship_id == (*_my_ship)._ship_id)
		this_ship=_my_ship;
	else
		this_ship = getShip(ship_id);
	string captain_name = (*this_ship)._captain_name;
	Messages::addMessageToQueue(captain_name+"> "+message,(*this_ship).getShipColor());
}

/*
 * Handle a VICT message, when another player wins the game.
 */
void Ship::handleVictMessage(string message) {
	_failure=true;
	int ship_id = StringParser::skimInt(message);

	Ship *this_ship = getShip(ship_id);
	string captain_name = (*this_ship)._captain_name;
	Messages::addMessageToQueue(captain_name+" has won the game!",(*this_ship).getShipColor());
	Artifact::setBetweenGames(true);
}

/*
 * Handle a SHIP message, when another player moves their ship.
 */
void Ship::handleShipMessage(string message) {
	//message format ID SHIP_X SHIP_Y DEST_X DEST_Y UPG_0 UPG_1 UPG_2 UPG_3 UPG_4 UPG_5
	int id = StringParser::skimInt(message);
	Ship *this_ship = getShip(id);
	float ship_x = StringParser::skimFloat(message);
	float ship_y = StringParser::skimFloat(message);

	(*this_ship)._ship_coords = Coords(ship_x,ship_y);

	float star_x = StringParser::skimFloat(message);
	float star_y = StringParser::skimFloat(message);

	(*this_ship)._ship_star.setPosition(Coords(star_x,star_y));

	for (int i=0;i<6;i++)
		(*this_ship)._upgrades[i] = StringParser::skimInt(message);
}

/*
 * Handle a JOIN message, when another player joins the game.
 */
void Ship::handleJoinMessage(string message) {
	int id = StringParser::skimInt(message);
	Ship *this_ship = getShip(id);
	string captain_name = message;

	(*this_ship)._captain_name = captain_name;
	Messages::addMessageToQueue(captain_name+" has joined the game.",(*this_ship).getShipColor());
}

/*
 * Handle a PLAY message, sent when we are just joining a game in progress to identify
 *   existing players.
 */
void Ship::handlePlayMessage(string message) {
	int id = StringParser::skimInt(message);
	Ship *this_ship = getShip(id);
	string captain_name = message;

	(*this_ship)._captain_name = captain_name;
	Messages::addMessageToQueue(captain_name+" is already playing.",(*this_ship).getShipColor());
}

/*
 * Gets an opponent ship, based on ID.
 */
Ship *Ship::getShip(int id) {
	_ships[id].setShipID(id);
	return &(_ships[id]);
}

/*
 * Sets a Ship's ship ID.
 */
void Ship::setShipID(int id) {
	_ship_id = id;
}

/*
 * Remove ship from a client's list of opponent ships.
 */
void Ship::removeShip(int id) {
	map<int,Ship>::iterator iter;
	map<const int, Ship, map_int_compare> *ships = Ship::getShips();
	for( iter = (*ships).begin(); iter != (*ships).end(); ++iter) {
		if ((iter->first)==id) {
			_ships.erase(iter);
			break;
		}
	}
}

/*
 * Remove all other players ships from a client's list of opponent ships.
 */
void Ship::removeAllShips() {
	_ships.erase(_ships.begin(),_ships.end());
}

/*
 * Retuns a map object (like a hashtable) of ships.
 */
map<const int, Ship, map_int_compare> *Ship::getShips() {
	return &_ships;
}

/*
 * Returns this ship's colour.
 */
Sint32 Ship::getShipColor() {
	Sint32 colors[16] = {
		0xFC0000FF,
		0x3FEF1FFF,
		0xFFCF00FF,
		0x1FBFDFFF,
		0xFF009FFF,
		0x00EFDFFF,
		0xFF9F00FF,
		0x9F00FFFF,
		0x00EF7FFF,
		0xFF6F4FFF,
		0x005F9FFF,
		0xFF00AFFF,
		0x4F00FFFF,
		0xFF9F6FFF,
		0x9FFF00FF,
		0xFF00FFFF,
	};
	int colorval = (_ship_id)%16;

	return colors[colorval];
}

/*
 * Return this ship's captain's name.
 */
string Ship::getCaptainName() {
	return _captain_name;
}

/*
 * Resets the Ship object for a new game.
 */
void Ship::reset() {
	for (int i=0;i<6;i++) {
		_upgrades[i]=1;
	}

	//FIXME - take this out once COMP is implemented
	_upgrades[UPGRADE_COMP]=10;

	_cash_dist = _cash = 100;
	_fuel_dist=_fuel=getMaxFuel();

	_updated=true;
	_victory=false;
	_failure=false;
	_visiting_aliens=false;
	_recheck_planet=true;
	_ship_coords = Coords(0,0);
	_arrived = true;
}

/*
 * Resets this ship's captain's name.
 */
void Ship::resetCaptainName() {
   	_captain_name="";
}

/*
 * Returns the string name of an upgrade. (e.g. Radar, Drive, etc.)
 */
string *Ship::getUpgradeName() {
	return _upgrade_name;
}
