/*
 * SoundSystem
 *
 * Handles game requests to play sounds and music.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <iostream>
#include <vector>
#include "NumberGenerator.h"
#include "SoundSystem.h"

using namespace std;

bool SoundSystem::_muted;
vector<Mix_Chunk*> SoundSystem::_effects;
vector<Mix_Music*> SoundSystem::_music;
int SoundSystem::_current_track;

/*
 * Constructor.  Initializes the sound system.  Must be instantiated before sound can be played.
 * Loads all sounds into memory and prepares music for playback.
 */
SoundSystem::SoundSystem() {
	Mix_OpenAudio( 44100, MIX_DEFAULT_FORMAT, 2, 1024);
	Mix_VolumeMusic(MIX_MAX_VOLUME);

	//Ancient, Industrialist, Futurist MUST be first 3!
	_effects.push_back(Mix_LoadWAV("Sounds/TribalGlitchLoop1.ogg"));
	_effects.push_back(Mix_LoadWAV("Sounds/IndustrialMekaLoop.ogg"));
	_effects.push_back(Mix_LoadWAV("Sounds/GlitchHipHopBeat2.ogg"));
	_effects.push_back(Mix_LoadWAV("Sounds/DuffieldSpaceNoise4.ogg"));
	_effects.push_back(Mix_LoadWAV("Sounds/DuffieldSpaceNoise2.ogg"));

	_music.push_back(Mix_LoadMUS( "Music/harmonicsoffrequencymodulation+zerog.ogg" ));
	_music.push_back(Mix_LoadMUS( "Music/HeavenGate.ogg" ));
	_music.push_back(Mix_LoadMUS( "Music/harmonicsoffrequencymodulation+antigravity.ogg" ));
	_music.push_back(Mix_LoadMUS( "Music/harmonicsoffrequencymodulation+etacarinae.ogg" ));
	_music.push_back(Mix_LoadMUS( "Music/harmonicsoffrequencymodulation+spaceradio.ogg" ));

	Mix_VolumeMusic(MIX_MAX_VOLUME);
	//_current_track = (rand()%5)-1;
	_current_track = ((int)(NumberGenerator::rand0()*5))%5;


	_muted = false;
}

/*
 * Destructor.  Stops music, and unloads sounds from memory.
 */
SoundSystem::~SoundSystem() {
	Mix_HaltMusic();

	for (signed int i=0;i<_music.size();i++) {
		Mix_HaltChannel(i);
		Mix_FreeMusic(_music[i]);
		_music[i]=NULL;
	}

	for (signed int i=0;i<_effects.size();i++) {
		Mix_FreeChunk(_effects[i]);
		_effects[i]=NULL;
	}

	Mix_CloseAudio();
}

/*
 * Play a specific sound once.
 */
void SoundSystem::playSound(int i) {
	if(!Mix_Playing(i)) {
		Mix_PlayChannel(i,_effects[i],0);
	}
}

/*
 * Stop a specfic sound from playing.
 */
void SoundSystem::stopSound(int i) {
	if(Mix_Playing(i)) {
		Mix_HaltChannel(i);
	}
}

/*
 * Mute all sound.
 */
bool SoundSystem::mute() {
	_muted = !_muted;
	if (_muted) {
		Mix_VolumeMusic(0);
		Mix_Volume(-1,0);
	} else {
		Mix_VolumeMusic(MIX_MAX_VOLUME);
		Mix_Volume(-1,MIX_MAX_VOLUME);
	}
	return _muted;
}

/*
 * Play music, starting at a random track.
 */
void SoundSystem::playMusic() {
	if (!Mix_PlayingMusic()) {
		_current_track++;
		if (_current_track>=(int)_music.size())
			_current_track=0;
		Mix_PlayMusic(_music[_current_track],0);
	}
}

/*
 * Resume Music after a pause.
 */
void SoundSystem::resumeMusic() {
	if (Mix_PausedMusic())
		Mix_ResumeMusic();
}

/*
 * Pause music. Used when visiting Aliens.
 */
void SoundSystem::pauseMusic() {
	if (!Mix_PausedMusic())
		Mix_PauseMusic();
}
