/*
 * Star
 *
 * Object representing a single star in the galaxy.  Class also contains the static variable
 * _star, representing all stars in memory.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>
#include <math.h>

#include "SDL/SDL.h"

#include "Coords.h"
#include "StringParser.h"
#include "Artifact.h"
#include "View.h"
#include "NumberGenerator.h"
#include "NetworkConnection.h"
#include "Definitions.h"
#include "Messages.h"
#include "Star.h"


vector<Star> Star::_star;
int Star::_level_break[7];
vector<int> Star::_plundered_stars;
vector<Star> Star::_visible_stars;

/*
 * Constructor.
 */
Star::Star() {
}

/*
 * Copy Constructor.
 */
Star::Star(const Star &s) {
	_index_number=s._index_number;
	_star_number=s._star_number;
	_star_class=s._star_class;
	_level=s._level;
	_position=s._position;
	_visible=s._visible;
	_has_artifact=s._has_artifact;
	_artifact=s._artifact;
	for (int i=0;i<10;i++)
		_neighbour[i]=s._neighbour[i];
}

/*
 * Constructor.  Creates a specific star with initialization data.
 */
Star::Star(const int index_number, const int star_number, const int level, const int star_class, const Coords position, const bool has_artifact, const Artifact artifact) {
	_index_number=index_number;
	_star_number=star_number;
	_star_class=star_class;
	_level=level;
	_position=position;
	_visible=false;
	_has_artifact=has_artifact;
	_artifact=artifact;
}

/*
 * Null Star object.  Used for equality comparison.
 */
Star Star::nullStar() {
	Star s;
	s._index_number=0;
	s._star_number=-1;
	//cout << _star[0].getPosition().toString()<<endl;
	s._position=Coords(-110000,-110000);
	return s;
}

/*
 * Checks to see if this Star is the null Star.
 */
bool Star::isNullStar() {
	return (_star_number==-1);
}

/*
 * Equality Operator.
 */
bool Star::operator== (const Star &b) {
	return (_star_number==b._star_number);
}

/*
 * Inequality operator.
 */
bool Star::operator!= (const Star &b) {
	return (_star_number!=b._star_number);
}

/*
 * Returns true if the star is visible on the scren.
 */
bool Star::isVisible() {
	return _visible;
}

/*
 * Gets the star index number (the current position in the _star vector.
 */
int Star::getStarIndex() {
	return _index_number;
}

/*
 * Gets the star number, a unique number (<78,000,000) that identifies this specific star.
 */
int Star::getStarNumber() {
	return _star_number;
}

/*
 * Sets the star number.
 */
void Star::setStarNumber(int star_number) {
	_star_number=star_number;
}

/*
 * Returns the star's class.
 */
int Star::getStarClass() {
	return _star_class;
}

/*
 * Determine's a star's spectral class based on star number and level of zoom.
 *
 * These values have been calculated based on numbers published in the February
 * 2001 issue of the Journal of the Royal Astronomical Society of Canada in an
 * article entitled "The Real Starry Sky" by Glenn LeDrew.  The numbers in his
 * Table I ("General stellar luminosity function of the solar neighbourhood
 * (per 10,000 pc^3") provided a rough start and allowed me to extrapolate
 * galactic distribution of Population I stars from the distribution of stars
 * in our stellar neighbourhood.
 *
 * Yes, I researched real astronomical papers to make this game. :)
 *
 * There may be mistakes here, and several approximations were made.
 * I need to do a seperate calculation for Population II stars (Galactic Core, Halo stars)
 * But at the end of the day the end result looks pretty good.
 */
int Star::getStarClass(int star_number, int level) {
	float rand = NumberGenerator::galaxy_rand(star_number,42);
	switch (level) {
		case 0:
			if (rand<0.00153) return STARCLASS_MI;
			else if (rand<0.0260) return STARCLASS_KI;
			else if (rand<0.0291) return STARCLASS_FI;
			else if (rand<0.0389) return STARCLASS_GI;
			else if (rand<0.486) return STARCLASS_KII;
			else if (rand<0.512) return STARCLASS_MII;
			else if (rand<0.570) return STARCLASS_FII;
			else if (rand<0.748) return STARCLASS_GII;
			else if (rand<0.751) return STARCLASS_OV;
			else if (rand<0.755) return STARCLASS_MIII;
			else if (rand<0.902) return STARCLASS_BV;
			else if (rand<0.910) return STARCLASS_FIII;
			else if (rand<0.936) return STARCLASS_GIII;
			else return STARCLASS_KIII;
			break;
		case 1:
		case 2:
			if (rand<0.0162) return STARCLASS_MIII;
			else if (rand<0.605) return STARCLASS_BV;
			else if (rand<0.638) return STARCLASS_FIII;
			else if (rand<0.741) return STARCLASS_GIII;
			else return STARCLASS_KIII;
			break;
		case 3:
		case 4:
			if (rand<0.0123) return STARCLASS_FIV;
			else if (rand<0.153) return STARCLASS_AV;
			else if (rand<0.192) return STARCLASS_GIV;
			else if (rand<0.199) return STARCLASS_MIV;
			else if (rand<0.297) return STARCLASS_KIV;
			else return STARCLASS_FV;
			break;
		case 5:
			if (rand<0.387) return STARCLASS_GV;
			else return STARCLASS_KV;
			break;
		case 6:
			return STARCLASS_MV;
		default:
			return STARCLASS_UNKNOWN;
	}
}

/*
 * Returns a string Star spectrum class based on integer type.
 */
string Star::getStarClassName() {
	switch (_star_class) {
		case STARCLASS_FI: return "F I";
		case STARCLASS_GI: return "G I";
		case STARCLASS_KI: return "K I";
		case STARCLASS_MI: return "M I";
		case STARCLASS_FII: return "F II";
		case STARCLASS_GII: return "G II";
		case STARCLASS_KII: return "K II";
		case STARCLASS_MII: return "M II";
		case STARCLASS_FIII: return "F III";
		case STARCLASS_GIII: return "G III";
		case STARCLASS_KIII: return "K III";
		case STARCLASS_MIII: return "M III";
		case STARCLASS_FIV: return "F IV";
		case STARCLASS_GIV: return "G IV";
		case STARCLASS_KIV: return "K IV";
		case STARCLASS_MIV: return "M IV";
		case STARCLASS_OV: return "O V";
		case STARCLASS_BV: return "B V";
		case STARCLASS_AV: return "A V";
		case STARCLASS_FV: return "F V";
		case STARCLASS_GV: return "G V";
		case STARCLASS_KV: return "K V";
		case STARCLASS_MV: return "M V";
		default: return "X";
	}
}

/*
 * Returns this star's 8 character name.
 * All names take the form: consonant vowel consonant vowel consonant vowel consonant vowel
 * with the letter Q excluded.
 * It was meant as an interim measure, but since has grown on me.
 */
string Star::getStarName() {
	string star_name;

	int digits[4];
	int i,j;
	j=0;
	char first[20]={'b','c','d','f','g','h','j','k','l','m','n','p','r','s','t','v','w','x','y','z'};
	char second[5]={'a','e','i','o','u'};

	long long rand;

	//49245653 is relatively prime, no repeats...
	rand = (((long long)_star_number)*49245653)%78125000;

	digits[0]=(rand/1000000)%100;
	digits[1]=(rand/10000)%100;
	digits[2]=(rand/100)%100;
	digits[3]=rand%100;

	for (i=0;i<4;i++) {
		if (i==0)
			star_name+=toupper(first[digits[i]%20]);
		else
			star_name+=first[digits[i]%20];
		star_name+=second[digits[i]/20];
	}

	return star_name;
}

/*
 * Seeds the galaxy's initial 5000 stars.
 * Stars 0 to 2999 are the major spiral arms.
 * Stars 3000 to 3999 are the minor sprial arms
 * Stars 4000 to 4499 is the galactic core.
 * Stars 4500 to 4999 is the galactic halo (random stars thrown about the galaxy)
 * The sprial arms are placed in a logarithmic sprial shape.  I've modified this slightly so that
 *   the arms don't overlap.
 * The core is a circlular polar distribution squashed at 45 degrees.
 * And the Halo is just random polar with a bit of a vertical squash.
 */
void Star::seedStars() {
	float galaxy_arm_sign, galaxy_arm_curve, galaxy_arm_theta, galaxy_arm_curve_delta;
	float galaxy_arm_rand_factor;
	for (int i=0;i<10;i++) {
		if (i%2==0)
			galaxy_arm_sign=1;
		else
			galaxy_arm_sign=-1;
		if (i<6) {
			//Major Arms
			galaxy_arm_curve=0.26;
			galaxy_arm_theta=1.75;
			galaxy_arm_curve_delta=0;
			if (i<2)
				galaxy_arm_rand_factor=1000;
			if (i<4)
				galaxy_arm_rand_factor=2000;
			else
				galaxy_arm_rand_factor=3000;
		} else if (i<8) {
			//Minor Arms
			galaxy_arm_curve=0.14;
			galaxy_arm_theta=2.625;
			galaxy_arm_curve_delta=.00010;
			galaxy_arm_rand_factor=1500;
		} else {
			//Core -- no curves!
		}

		if (i<8) { //Arm
			for (int j=0;j<(SEED_STARS/10);j++) {
				int k = i*(SEED_STARS/10)+j;
				int star_number = k*15625;
				int level = 0;
				int star_class = getStarClass(star_number,level);

				Coords rand( (float)(NumberGenerator::galaxy_rand(star_number,0)*(galaxy_arm_rand_factor*2))-(galaxy_arm_rand_factor), (float)(NumberGenerator::galaxy_rand(star_number,1)*(galaxy_arm_rand_factor*2))-(galaxy_arm_rand_factor));

				int index_number = k;

				float theta = (10.0/SEED_STARS)*galaxy_arm_theta*PI*j;
				float r = 10000.0*exp(galaxy_arm_curve*theta);

				Coords position(-galaxy_arm_sign*r*cos(theta+PI/4.0)+rand.getX(),
								galaxy_arm_sign*r*sin(theta+PI/4.0)+rand.getY());

				Artifact artifact;
				int has_artifact=false;

				if (star_number%9==0) {
					has_artifact=true;
					artifact=Artifact(star_number);
				}

				Star my_star(index_number, star_number, level, star_class, position, has_artifact, artifact);

				_star.push_back(my_star);

				galaxy_arm_curve+=galaxy_arm_curve_delta*(5000.0/SEED_STARS);
			}
		} else if (i<9) { //Core
			for (int j=0;j<(SEED_STARS/10);j++) {
				int k = (4*SEED_STARS/5)+j;

				int star_number = k*15625;
				int level = 0;
				int star_class = getStarClass(star_number,level);
				float rand_theta = ((float)(NumberGenerator::galaxy_rand(star_number,0)))*2.0*PI;
				float rand_r = (float)(NumberGenerator::galaxy_rand(star_number,1)*10000);
				int index_number = k;

				Coords temp(rand_r*cos(rand_theta),rand_r*sin(rand_theta)/2.0);
				Coords position(-temp.getX()/pow(2,0.5)+temp.getY()/pow(2,0.5),
								temp.getX()/pow(2,0.5)+temp.getY()/pow(2,0.5));

				Artifact artifact;
				bool has_artifact=false;
				if (star_number%9==0) {
					has_artifact=true;
					artifact=Artifact(star_number);
				}

				Star my_star(index_number, star_number, level, star_class, position, has_artifact, artifact);

				_star.push_back(my_star);
			}
		} else { //Galactic Halo
			for (int j=0;j<(SEED_STARS/10);j++) {
				int k = (9*SEED_STARS/10)+j;

				int star_number = k*15625;
				int level = 0;
				int star_class = getStarClass(star_number,level);

				float rand_theta = (NumberGenerator::galaxy_rand(star_number,0))*2.0*PI;
				float rand_r = (float)(NumberGenerator::galaxy_rand(star_number,1)*50000.0);
				int index_number = k;

				Coords position(rand_r*cos(rand_theta),0.9*rand_r*sin(rand_theta));
				Artifact artifact;
				bool has_artifact=false;
				if (star_number%9==0) {
					has_artifact=true;
					artifact=Artifact(star_number);
				}

				Star my_star(index_number, star_number, level, star_class, position, has_artifact, artifact);

				_star.push_back(my_star);
			}
		}
	}

	_level_break[0]=SEED_STARS;
}

/*
 * A debug routine to scan through all the stars and gather data.  Takes at least an hour to run.
 */
void Star::surveyStars() {
	seedStars();

	int location[7];
	int old_location[7];
	int level_count[7];

	int star_count=0;
	int alien_count=0;
	int artifact_count=0;
	int max_stars=0;

	int artifacts[3][3][143];
	for (int i=0;i<3;i++)
		for (int j=0;j<3;j++)
			for (int k=0;k<143;k++)
				artifacts[i][j][k]=0;

	for (int i=0;i<7;i++) {
		old_location[i]=0;
		level_count[i]=0;
	}

	for (int i1=1;i1<10;i1++) {
		location[0]=i1;
		cout << "Loc "<<i1<<endl;
		for (int i2=1;i2<10;i2++) {
			location[1]=i2;
			cout << "     "<<i2<<endl;
			for (int i3=1;i3<10;i3++) {
				location[2]=i3;
				for (int i4=1;i4<10;i4++) {
					location[3]=i4;
					for (int i5=1;i5<10;i5++) {
						location[4]=i5;
						for (int i6=1;i6<10;i6++) {
							location[5]=i6;
							for (int i7=1;i7<10;i7++) {
								location[6]=i7;

								//cout << "Location=";
								//for (int i=0;i<7;i++)
									//cout << location[i];

								addStars (old_location,location,7);
								//cout << " istars=" << _star.size();
								for (int i=0;i<7;i++) {
									old_location[i]=location[i];
								}

								View::setOffset(View::getOffset(location,7));
								View::setScale(100000/pow(3.0,7));


								findVisible();

								vector<Star> visible_stars = getVisibleStars();
								int num_stars = (int)visible_stars.size();

								for (int i=0;i<num_stars;i++) {
									int star_level = visible_stars[i]._level;
									level_count[star_level]++;
								}

								int num_alien_stars = getVisibleAlienStars().size();
								int num_artifact_stars = getVisibleArtifactStars().size();

								//cout << " nstars=" << num_stars;
								//cout << " nalstars=" << num_alien_stars;
								//cout << " narstars=" << num_artifact_stars;

								star_count+= num_stars;
								alien_count+=num_alien_stars;
								artifact_count+=num_artifact_stars;
								//cout << " scount=" << star_count;
								//cout << " alcount=" << alien_count;
								//cout << " arcount=" << artifact_count;

								if (max_stars<num_stars)
									max_stars=num_stars;

								//cout << " mstars=" << max_stars;
								//cout << endl;

								vector<Star> artifact_stars = getVisibleArtifactStars();
								for (int i=0;i<(int)artifact_stars.size();i++) {
									Star s = artifact_stars[i];
									Artifact a = s.getArtifact();
									int artifact_number = a.getArtifactNumber();
									int artifact_type = a.getArtifactType();

									int word_part[3];

									//remember, offset by 1...
									word_part[0]=(artifact_number/20449)%143;
									word_part[1]=(artifact_number/143)%143;
									word_part[2]=artifact_number%143;

									artifacts[artifact_type][0][word_part[0]]+=1;
									artifacts[artifact_type][1][word_part[1]]+=1;
									artifacts[artifact_type][2][word_part[2]]+=1;

								}
							}
						}
					}
				}
			}
		}
	}

	cout << endl;

	cout << "STARS\n";

	cout << "Star Count=" << star_count<< endl;
	cout << "Aliens Count=" << alien_count<< endl;
	cout << "Artifact Count=" << artifact_count<< endl;
	cout << "Max Stars=" << max_stars<< endl;

	cout << endl;

	cout << "LEVEL COUNT\n";

	for (int i=0;i<7;i++) {
		cout << "Level "<<i<<" = "<<level_count[i]<<endl;
	}

	cout << endl;

	cout << "ARTIFACTS\n";

	for (int i=0;i<3;i++)
		for (int j=0;j<3;j++)
			for (int k=0;k<143;k++)
				cout << i<< " "<<j<<" "<<k<<" = "<<artifacts[i][j][k]<<endl;


	_star.erase(_star.begin(),_star.end());
	exit(0);
}

/*
 * Determines if star i is closer than star j in a specific keypad direction.
 */
void Star::isCloser(Star *i, Star *j, int direction) {
	int k; //Closest So Far

	k = (*i)._neighbour[direction];

	if (k==-1) {
		(*i)._neighbour[direction]=(*j)._index_number;
	} else {
		Coords jc =  (*j)._position-(*i)._position;
		Coords kc =  _star[k]._position-(*i)._position;
		float dist_j = jc.getLength();
		float dist_k = kc.getLength();

		if (dist_j<dist_k)
			(*i)._neighbour[direction]=(*j)._index_number;
	}

}

/*
 * Sorts stars based on keypad direction neighbours.
 * Used to navigate between stars with the cursor once the view is loaded.
 */
Star Star::sortStars(Coords offset) {
	int selected_star=-1;
	//This needs to be big to avoid Babababa bugs.
	float max_dist=200000;
	int direction_i,direction_j;
	direction_i=-1;
	direction_j=-1;

	findVisible();
	vector<Star> visible_stars = getVisibleStars();

	for (int i=0;i<(int)visible_stars.size();i++) {
		Coords delta = visible_stars[i]._position-offset;
		float dist = delta.getLength();
		if (dist<max_dist) {
			max_dist=dist;
			selected_star=i; //closest to offset.x,offset.y
		}
	}

	if (visible_stars.size()==0) {
		return nullStar();
	}

	for (int i=0;i<(int)visible_stars.size();i++) {
		for (int j=0;j<10;j++) {
			visible_stars[i]._neighbour[j]=-1;
		}
	}

	visible_stars[visible_stars.size()-1]._neighbour[9]=visible_stars[0]._index_number;
	visible_stars[0]._neighbour[8]=visible_stars[visible_stars.size()-1]._index_number;

	for (int i=0;i<((int)visible_stars.size()-1);i++) {
		visible_stars[i]._neighbour[9]=visible_stars[i+1]._index_number;
		visible_stars[i+1]._neighbour[8]=visible_stars[i]._index_number;
		for (int j=(i+1);j<(int)visible_stars.size();j++) {
			//Compare i,j
			Coords delta = visible_stars[j].getPosition()-visible_stars[i].getPosition();
			float angle = delta.getAngle();

			//567
			//4 0
			//321
			if ((angle>15.0*PI/8.0)||(angle<=1.0*PI/8.0)) {
				direction_i=0;
				direction_j=4;
			} else if ((angle>1.0*PI/8.0)&&(angle<=3.0*PI/8.0)) {
				direction_i=1;
				direction_j=5;
			} else if ((angle>3.0*PI/8.0)&&(angle<=5.0*PI/8.0)) {
				direction_i=2;
				direction_j=6;
			} else if ((angle>5.0*PI/8.0)&&(angle<=7.0*PI/8.0)) {
				direction_i=3;
				direction_j=7;
			} else if ((angle>7.0*PI/8.0)&&(angle<=9.0*PI/8.0)) {
				direction_i=4;
				direction_j=0;
			} else if ((angle>9.0*PI/8.0)&&(angle<=11.0*PI/8.0)) {
				direction_i=5;
				direction_j=1;
			} else if ((angle>11.0*PI/8.0)&&(angle<=13.0*PI/8.0)) {
				direction_i=6;
				direction_j=2;
			} else if ((angle>13.0*PI/8.0)&&(angle<=15.0*PI/8.0)) {
				direction_i=7;
				direction_j=3;
			}

			isCloser(&visible_stars[i],&visible_stars[j],direction_i);
			isCloser(&visible_stars[j],&visible_stars[i],direction_j);
		}
	}

	//Put this location data back into the star database...
	for (int i=0;i<(int)visible_stars.size();i++) {
		_star[visible_stars[i]._index_number] = visible_stars[i];
	}

	Star cursor_star =  visible_stars[selected_star];

	return cursor_star;
}


/*
 * Moves the cursor star in a specific keypad direction.
 * Returns new cursor star, if any,
 */
Star Star::moveCursorStar(int direction) {
	int new_star=-1;

	new_star = _neighbour[direction];

	if (new_star==-1)
		return *this;

	//FIXME
	if ((new_star<-1) || (new_star>(int)_star.size())) {
		cout << "DEBUG moveCursorStar.  This star has no location data!\n";
		cout << "DEBUG returning null star\n";
		Messages::addMessageToQueue("You found the Babababa Bug",0xFF0000FF);
		Messages::addMessageToQueue("Try to reproduce what you just did.",0xFF0000FF);
		return _star[0];
	}

	return _star[new_star];
}

/*
 * Returns true if star has aliens (9% chance)
 */
bool Star::hasAliens() {
	float rand = NumberGenerator::galaxy_rand(_star_number,13);
	if (rand<0.09)
		return true;
	return false;
}

/*
 * Returns true if star has an artifact (11% chance)
 */
bool Star::hasArtifact() {
	return _has_artifact;
}

/*
 * Returns a vector of visible alien stars in the View.
 */
vector<Star> Star::getVisibleAlienStars() {
	int i=0;
	vector<Star> visible_alien_stars;
	vector<Star> visible_stars = getVisibleStars();

	for (i=0;i<(int)visible_stars.size();i++) {
		if (visible_stars[i].hasAliens()) {
			visible_alien_stars.push_back(visible_stars[i]);
		}
	}

	return visible_alien_stars;
}

/*
 * Returns a vector of visible artifact stars in the View.
 */
vector<Star> Star::getVisibleArtifactStars() {
	vector<Star> visible_artifact_stars;
	vector<Star> visible_stars = getVisibleStars();

	for (int i=0;i<(int)visible_stars.size();i++) {
		if ((visible_stars[i]._has_artifact)&&(!visible_stars[i].isPlundered())) {
			visible_artifact_stars.push_back(visible_stars[i]);
		}
	}

	return visible_artifact_stars;
}

/*
 * Adds stars at a specific location, level, and offset.
 * Finds all visible stars, and spawns four children for each.
 * Discards these new stars if they're not in the view (which is why we have 40 million instead
 *  of 78 million stars).
 */
void Star::addStars (int location[], int level, Coords offset) {
	float scale = 100000.0/pow(3,level);
	int x_pos,y_pos;

	if(level>=7) {
		return;
	}

	int num_seed_stars = _star.size();
	for (int i=0;i<num_seed_stars;i++) {
		Coords delta = _star[i]._position-offset;
		delta.getIntValues(&x_pos,&y_pos,scale);

		if ((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)) {
			for (int j=0;j<STAR_POWER;j++) {
				int star_number = _star[i]._star_number+(j+1)*NumberGenerator::int_pow(STAR_POWER+1,6-level);
				//int star_level = _star[i]._level+1;
				int star_level = level;
				int star_class = getStarClass(star_number,level);

				float rand_r = level*(scale/100000.0)*(2750.0+((float)(NumberGenerator::galaxy_rand(star_number,0)*1000.0)));
				float rand_theta = 2.0*PI*((float)(NumberGenerator::galaxy_rand(star_number,1)));
				Coords rand(rand_r,rand_theta,true);

				Coords position = _star[i]._position+rand;

				int index_number = _star.size();

				bool has_artifact=false;
				Artifact artifact;
				if (star_number%9==0) {
					has_artifact=true;
					artifact=Artifact(star_number);
				}

				Coords delta = position-offset;
				delta.getIntValues(&x_pos,&y_pos,scale);

				if ((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)) {
					Star my_star(index_number, star_number, star_level, star_class, position, has_artifact, artifact);
					_star.push_back(my_star);
				}
			}
		}
	}

	_level_break[level]=_star.size();
}

/*
 * Given an existing location and a new location, discard old stars in the _star vector
 *  and calculate new stars in the new location.  For panning between sectors or zooming to
 *  the home star.
 */
void Star::addStars (int old_location[],int new_location[], int level) {
	bool reset=false;

	int temp_location[7];

	Coords offset(0.0,0.0);
	float scale=100000.0;

	for (int i=0;i<7;i++)
		temp_location[i]=0;

	for (int i=1;i<=level;i++) {
		temp_location[i-1]=new_location[i-1];

		scale=100000/pow(3.0,i);
		if ((new_location[i-1]==1)||(new_location[i-1]==2)||(new_location[i-1]==3))
			offset+=Coords(0,scale);
		if ((new_location[i-1]==7)||(new_location[i-1]==8)||(new_location[i-1]==9))
			offset+=Coords(0,-scale);
		if ((new_location[i-1]==1)||(new_location[i-1]==4)||(new_location[i-1]==7))
			offset+=Coords(-scale,0);
		if ((new_location[i-1]==3)||(new_location[i-1]==6)||(new_location[i-1]==9))
			offset+=Coords(scale,0);

		if (old_location[i-1]!=new_location[i-1])
			reset=true;
		if (reset) {
			removeStars(i);
			addStars(temp_location,i,offset);
		}
	}
}

/*
 * Remove stars from the _star vector up to a certain zoom level.
 */
void Star::removeStars (int level) {
	vector<Star>::iterator pos;
	pos = _star.begin()+_level_break[level-1];
	_star.erase(pos,_star.end());
}

/*
 * Returns true if star has already had its artifact plundered.
 */
bool Star::isPlundered() {
	for(int i=0;i<(int)_plundered_stars.size();i++) {
		if (_plundered_stars[i]==_star_number)
			return true;
	}

	return false;
}

/*
 * Plunder this star.  Queue network message stating so to server.
 */
void Star::plunder() {
	_plundered_stars.push_back(_star_number);
	_has_artifact=false;
	queueStarPlunderMessage(_star_number);
}

/*
 * Queues a PLUN message to the game server, stating a certian star has been plundered
 * of its artifacts, so that no other player can take it.
 */
void Star::queueStarPlunderMessage(int star_number) {
	NetworkConnection *networkConnection = NetworkConnection::getClientNetworkConnection();
	stringstream ss;
	ss << star_number;
	if (networkConnection!=NULL) {
		(*networkConnection).queueMessage("PLUN",ss.str());
	}
}

/*
 * Handles a PLUN message from the server, indicating another player has taken an artifact.
 */
void Star::handlePlunMessage(string message) {
	while (message!="") {
		int star_number = StringParser::skimInt(message);
		_plundered_stars.push_back(star_number);
	}
}

/*
 * Returns the alien type at an alien star. (33.3% chance of each)
 */
int Star::getAlienType() {
	float rand = NumberGenerator::galaxy_rand(_star_number,14);
	if (rand<0.333333)
		return 0;
	if (rand<0.666666)
		return 1;
	return 2;
}

/*
 * Returns the _star vector of current stars in memory.
 */
vector<Star> *Star::getStars() {
	return &_star;
}

/*
 * Get a specific star from the stars in memory.
 */
Star Star::getStar(const int i) {
	return _star[i];
}

/*
 * Get the (x,y) Coords position of this star.
 */
Coords Star::getPosition() {
	return _position;
}

/*
 * Set the (x,y) Coords position of this star.
 */
void Star::setPosition(Coords pos) {
	_position = pos;
}

/*
 * Returns this size of this star.
 */
int Star::getSize() {
	return getSize(View::getLevel());
}

/*
 * Returns the size of a star at a certain zoom level.
 */
int Star::getSize(int level) {
	return (6-_level)/(8-level);
}

/*
 * Returns the average of two RGBA colours.  Used by GalaxySurface to "White out" stars
 * at a distance.  This is actually fair because zoomed out stars are supposed to be
 * representative of all their 5^6 children.
 */
Sint32 Star::averageColor(Sint32 c1, Sint32 c2) {
	Sint32 r1,g1,b1,r2,g2,b2,a1,a2,color;
	r1=g1=b1=a1=c1;
	r2=g2=b2=a2=c2;
	r1 &= 0xFF000000;
	g1 &= 0x00FF0000;
	b1 &= 0x0000FF00;
	a1 &= 0x000000FF;

	r1 = r1 >>24;
	r1 &= 0x000000FF;
	g1 = g1 >>16;
	b1 = b1 >>8;

	r2 &= 0xFF000000;
	g2 &= 0x00FF0000;
	b2 &= 0x0000FF00;
	a2 &= 0x000000FF;

	r2 = r2 >>24;
	r2 &= 0x000000FF;
	g2 = g2 >>16;
	b2 = b2 >>8;


	r1 = (r1+r2)/2;
	g1 = (g1+g2)/2;
	b1 = (b1+b2)/2;
	a1 = (a1+a2)/2;

	r1=r1<<24;
	g1=b1<<16;
	b1=b1<<8;
	color = r1|g1|b1|a1;

	return color;
}

/*
 * Returns the star color of this star.
 */
Sint32 Star::getStarColor() {
	return getStarColor(View::getLevel());
}

/*
 * returns the star colour of this star at a certain View zoom level.
 */
Sint32 Star::getStarColor(int level) {

	Sint32 color=0xEDFCFFAF;
	switch(_star_class%10) {
		case 1:
			//O
			color = 0x7F7FFFFF;
			break;
		case 2:
			//B
			color = 0xFFFFFFFF;
			break;
		case 3:
			//A
			color = 0xFFFFD0FF;
			break;
		case 4:
			//F
			color = 0xFFFFB0FF;
			break;
		case 5:
			//G
			color = 0xFFFF7FFF;
			break;
		case 6:
			//H
			color = 0xFF8020FF;
			break;
		case 7:
			//M
			color = 0xFF2020FF;
			break;
	}

	Sint32 a1,a2;
	a2=averageColor(0xEDFCFFAF,color);
	a1=averageColor(0xEDFCFFAF,a2);
	//a3=averageColor(color,a2);
	if (level==0) {
		color = 0xEDFCFFAF;
	} else if (level==1) {
		color = a1;
	} else if (level==2) {
		color = a2;
	//} else if (level==3) {
	//	color = a3;
	}

	return color;
}

/*
 * Returns the artifact object associated with this star.
 */
Artifact Star::getArtifact() {
	return _artifact;
}

/*
 * Finds and tags all visible stars in the current View.
 */
void Star::findVisible() {
	_visible_stars.erase(_visible_stars.begin(),_visible_stars.end());
	for (int i=0;i<(int)_star.size();i++) {
		Coords delta = _star[i]._position-View::getOffset();
		int x_pos,y_pos;
		delta.getIntValues(&x_pos,&y_pos,View::getScale());
		if ((x_pos>0)&&(x_pos<SCREEN_CENTER*2)&&(y_pos>0)&&(y_pos<SCREEN_CENTER*2)) {
			_star[i]._visible=true;
			_visible_stars.push_back(_star[i]);
		} else {
			_star[i]._visible=false;
		}
	}
}

/*
 * Returns a vector of all visible stars in the current view.
 */
vector<Star> Star::getVisibleStars() {
	return _visible_stars;
}

/*
 * Returns a random star in the galaxy.  This method pics a star with no bias to stellar density.
 * Sectors with any star are picked at random.  This method gives poor start placement, but is
 * fine for artifacts.
 *
 * WARNING - Destroys existing star data for non-original seed stars!  Also destroys View.
 */
Star Star::getRandomStar(int location[],bool with_artifact) {
	int level=0;
	int old_location[7];

	for (int i=0;i<7;i++) {
		location[i]=0;
		old_location[i]=0;
	}

	bool star_found=false;
	while (!star_found) {
		for (int i=0;i<7;i++)
			old_location[i]=location[i];

		if (location[level]==0) {
			//location[level]=rand()/((RAND_MAX)/9)+1;
			location[level]=((int)(NumberGenerator::rand0()*9))%9+1;
		}

		level++;

		addStars(old_location,location, level);
		View::setOffset(View::getOffset(location, level));
		View::setScale(100000/pow(3.0,level));

		findVisible();

		bool visible_star=false;

		for (int i=0;i<(int)_star.size();i++) {
			if ((_star[i]._visible)&&((!with_artifact)||(_star[i]._has_artifact))) {
				visible_star=true;
				break;
			}
		}

		if (!visible_star) {
			location[level-1]+=1;
			if (location[level-1]>=10)
				location[level-1]=1;
			level--;
		} else {
			if (level>6) {
				star_found=true;
			}
		}


	}

	vector<int> visible_stars;
	for (int i=0;i<(int)_star.size();i++) {
		if ((_star[i]._visible)&&((!with_artifact)||(_star[i]._has_artifact))) {
			visible_stars.push_back(i);
		}
	}

	//int this_star = rand()/((RAND_MAX)/visible_stars.size());
	int this_star = ((int)(NumberGenerator::rand0()*visible_stars.size()))%visible_stars.size();

	sortStars(View::getOffset(location, level));

	Star s = _star[visible_stars[this_star]];

	removeStars(1);
	View::reset();

	return s;
}

/*
 * Returns a random star 40%-100% out a minor arm.  Better for ship starts.
 *
 * WARNING - Destroys existing star data for non-original seed stars!  Also destroys View.
 */
Star Star::getRandomStar2(int location[]) {
	int old_location[7];

	//Pick a seed star in a minor arm...
	int i = 6+((int)(NumberGenerator::rand0()*2))%2;
	int j = (2*SEED_STARS/50)+((int)((3*SEED_STARS/50)*NumberGenerator::rand0()))%(3*SEED_STARS/50);
	int k = i*(SEED_STARS/10)+j;

	Star my_star = _star[k];

	for (int i=0;i<7;i++) {
		old_location[i]=0;
		location[i]=0;
	}

	for (int i=0;i<7;i++) {
		Coords star_coords = my_star.getPosition();
		View::setOffset(View::getOffset(location, i));
		View::setScale(100000/pow(3.0,i));

		Coords delta = star_coords-View::getOffset();
		delta = delta*(2/View::getScale());

		location[i]=5;
		if (delta.getX()<=(-1.0f/3.0f))
			location[i]-=1;
		else if (delta.getX()>(1.0f/3.0f))
			location[i]+=1;
		if (delta.getY()<=(-1.0f/3.0f))
			location[i]+=3;
		else if (delta.getY()>(1.0f/3.0f))
			location[i]-=3;

		if (i>=6)
			break;

		addStars(old_location,location, i+1);

		old_location[i]=location[i];

		findVisible();

		vector<Star> child_stars;

		for (signed int j=0;j<_star.size();j++) {
			if ((_star[j]._star_number/NumberGenerator::int_pow(STAR_POWER+1,6-i))==
				(my_star._star_number/NumberGenerator::int_pow(STAR_POWER+1,6-i))) {
				//Child Star
				child_stars.push_back(_star[j]);
			}
		}

		int choice = ((int)(child_stars.size()*NumberGenerator::rand0()))%child_stars.size();
		my_star = child_stars[choice];
	}

	Star s = my_star;

	removeStars(1);
	View::reset();

	return s;
}

/*
 * Resets the star data for game reset.
 */
void Star::reset() {
	_plundered_stars.erase(_plundered_stars.begin(),_plundered_stars.end());
	_visible_stars.erase(_visible_stars.begin(),_visible_stars.end());
	removeStars(1);
}
