/*
 * VictorySurface
 *
 * Displayed to screen if the user wins or loses the game.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <string>
#include <vector>
#include <iostream>

#include "SDL/SDL.h"
#include "SDL/SDL_gfxPrimitives.h"

#include "Definitions.h"
#include "Font.h"
#include "Ship.h"
#include "View.h"
#include "Coords.h"
#include "Artifact.h"
#include "Star.h"
#include "FrameTimer.h"
#include "Coords.h"

#include "ScreenSurface.h"
#include "VictorySurface.h"

using namespace std;

/*
 * Constructor.  Prepares Victory Surface for blitting.  Must be instatiated before blit() is called.
 */
VictorySurface::VictorySurface(bool victory) {
	_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);

	SDL_SetColorKey(_surface, SDL_SRCCOLORKEY, 0x00000000);

	_translucency_surface =  SDL_CreateRGBSurface(SDL_HWSURFACE, SCREEN_CENTER*2, SCREEN_HEIGHT, SCREEN_DEPTH,0,0,0,0);
	SDL_SetAlpha(_translucency_surface,SDL_SRCALPHA,127);
	boxColor(_translucency_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x00007FFF);

	_rect.x=0;
	_rect.y=0;
	_rect.w=SCREEN_CENTER*2;
	_rect.h=SCREEN_HEIGHT;

	_frame_count=0;

	if (victory)
		_message.push_back("You have found the artifact!");
	else
		_message.push_back("Another player has found the artifact.");
	_message.push_back("");
	string artifact_name = Artifact::getPrizeArtifact().getArtifactName();
	string artifact_name_2 = "";
	size_t found = artifact_name.substr(artifact_name.length()/2,artifact_name.length()).find(" ");
	if (found!=string::npos) {
		artifact_name_2 = artifact_name.substr(artifact_name.length()/2+found+1,artifact_name.length());
		artifact_name = artifact_name.substr(0,artifact_name.length()/2+found);
		_message.push_back(artifact_name);
		_message.push_back(artifact_name_2);
	} else {
		_message.push_back(artifact_name);
	}

	_message.push_back("");
	stringstream ss;
	int *prize_location = Artifact::getPrizeLocation();
	ss << prize_location[0] << prize_location[1] << prize_location[2] << "-" << prize_location[3] << prize_location[4] << prize_location[5] << prize_location[6];
	_message.push_back("The artifact was in sector "+ss.str());
	_message.push_back("on a planet orbiting "+Artifact::getPrizeStar().getStarName());
	_message.push_back("");

	int secs = FrameTimer::getTimeElapsed();
	int mins = secs/60;
	secs %=60;
	ss.str(string());
	ss << mins << " minutes and " << secs << " seconds ";
	if (victory)
		_message.push_back("You found the artifact in");
	else
		_message.push_back("You played for");
	_message.push_back(ss.str());
	_message.push_back("");
	if (victory)
		_message.push_back("WELL DONE!");
	else
		_message.push_back("GOOD TRY!");
}

/*
 * Destructor.  Frees surface.
 */
VictorySurface::~VictorySurface() {
	SDL_FreeSurface(_surface);
	SDL_FreeSurface(_translucency_surface);
}

/*
 * Blits victory surface to screen surface.
 */
void VictorySurface::blit() {
	SDL_BlitSurface(_translucency_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	boxColor(_surface,0,0,SCREEN_CENTER*2,SCREEN_CENTER*2,0x000000FF);
	blitMessage();
	SDL_BlitSurface(_surface,NULL,ScreenSurface::getScreenSurface(),&_rect);
	_frame_count++;
}

/*
 * Blit victory message to Victory surface.
 */
void VictorySurface::blitMessage() {
	int height = -1;
	int count=_frame_count*2;
	int w,h;

	int alien_type = Artifact::getPrizeArtifact().getArtifactType();

	TTF_Font *font=NULL;
	SDL_Color textColor = {255,255,255};

	switch (alien_type) {
		case 0:
			font = Font::getTTFFont("Burtinomatic.ttf");
			break;
		case 1:
			font = Font::getTTFFont("Incognitype.ttf");
			break;
		case 2:
			font = Font::getTTFFont("HookedUp.ttf");
			break;
	}

	if (font==NULL) {
		//Alien Type not determined yet.
		return;
	}

	if (height==-1) {
		//height=SCREEN_CENTER-10*(_message.size());
		TTF_SizeText( font, "X", &w,&h );
		height=SCREEN_CENTER-(h+2)*(_message.size())/2;
	}
	int char_count=0;

	SDL_Surface *message;

	for (int i=0;i<(int)_message.size();i++) {
		string this_line = _message[i];
		int mes_len = this_line.length();
		TTF_SizeText( font, this_line.c_str(), &w,&h );

		SDL_Rect message_rect;
		message_rect.x = (SCREEN_CENTER*2-w) / 2;
		message_rect.y = height+i*(h+2);

		if (count<char_count+mes_len) {
			this_line = this_line.substr(0,count-char_count);
			message = TTF_RenderText_Blended( font, this_line.c_str(), textColor );
			SDL_BlitSurface(message,NULL,_surface,&message_rect);
			SDL_FreeSurface(message);
			break;
		} else {
			message = TTF_RenderText_Blended( font, this_line.c_str(), textColor );
			SDL_BlitSurface(message,NULL,_surface,&message_rect);
			SDL_FreeSurface(message);
			char_count+=mes_len;
		}
	}

}
