/*
 * View
 *
 * Class representing what the user is looking at, and where the user will be looking at soon.
 *
 * Copyright (c) 2009 Trevor Bradley <trevor.bradley@shaw.ca>
 *
 * This code is being shared for educational purposes only.
 * You are free to download this code from authorized sites.
 * You may examine, modify and compile the code for personal purposes.
 * You may not redistribute any portion of this code without the author's permission.
 * You may not redistribute binaries containing this code without the author's permission.
 *
 * Please read the LICENSE file for licensing details.
 *
 */

#include <math.h>
#include <iostream>

#include "Definitions.h"
#include "Messages.h"
#include "Coords.h"
#include "KeyPressed.h"
#include "Artifact.h"
#include "Ship.h"
#include "FrameTimer.h"
#include "Information.h"
#include "Star.h"
#include "View.h"

Coords View::_offset;
Coords View::_offset_dest;
float View::_scale;
float View::_scale_dest;
int View::_location[7];
int View::_level;
bool View::_updated=true;
bool View::_update_override=true;

Star View::_cursor_star;
Coords View::_cursor_coords;
bool View::_cursor_star_set=false;

/*
 * Constructor.  Initializes full zoomed out level 0 zoom, viewing whole galaxy.
 */
View::View() {
	_updated=true;
	_level=0;
	for (int i=0;i<7;i++)
		_location[i]=0;
	_scale=_scale_dest=100000.0;
	_offset = _offset_dest = Coords(0.0,0.0);
	_cursor_star_set=false;
}

/*
 * Copy Constructor.
 */
View::View (View &v) {
	_offset = v._offset;
	_scale = v._scale;
	for (int i=0;i<7;i++)
		_location[i]=v._location[i];
	_level = v._level;
}

/*
 * Resets view for new game.
 */
void View::reset() {
	View();
}

/*
 * Gets the current view offset: The (x,y) coords at the center of the screen.
 */
Coords View::getOffset() {
	return _offset;
}

/*
 * Gets the destination of the current view offset:  The (x,y) Coords where the view is going.
 */
Coords View::getOffsetDest() {
	return _offset_dest;
}

/*
 * Sets the View offset.
 */
void View::setOffset(const Coords c) {
	_offset=c;
	_updated=true;
}

/*
 * returns the view offset from an integer location (e.g. 1234567)
 */
Coords View::getOffset(int loc) {
	int location[7];

	for (int i=6;i>=0;i--) {
		location[i]=loc%10;
		loc/=10;
	}

	return getOffset(location);
}

/*
 * Gets the zoom level based on the View location array. (e.g. {1,2,3,4,5,6,7})
 */
int View::getLevel(int location[]) {
	int level=0;
	for (level=6;level>=0;level--) {
		if (location[level]!=0)
			break;
	}

	return level+1;
}

/*
 * Gets the zoom level based on the view location (e.g 1234567)
 */
int View::getLevel(int location) {
	int level=0;
	int this_digit;
	for (level=6;level>=0;level--) {
		this_digit=location%10;
		if (this_digit!=0)
			break;
		location/=10;
	}

	return level+1;
}

/*
 * Returns the scale of the level (light years across)
 */
float View::getScale(int level) {
	float this_scale=100000/pow(3.0,level);
	return this_scale;
}

/*
 * Returns the offset of a sector location array.
 */
Coords View::getOffset(int location[]) {
	return getOffset(location,getLevel(location));
}

/*
 * Returns the offset of a sector location array.
 */
Coords View::getOffset(int location[], int level) {
	Coords offset(0.0,0.0);

	for (int i=1;i<=level;i++) {
		float this_scale=100000/pow(3.0,i);
		if ((location[i-1]==1)||(location[i-1]==2)||(location[i-1]==3))
			offset+=Coords(0,this_scale);
		if ((location[i-1]==7)||(location[i-1]==8)||(location[i-1]==9))
			offset+=Coords(0,-this_scale);
		if ((location[i-1]==1)||(location[i-1]==4)||(location[i-1]==7))
			offset+=Coords(-this_scale,0);
		if ((location[i-1]==3)||(location[i-1]==6)||(location[i-1]==9))
			offset+=Coords(this_scale,0);
	}

	return offset;
}

/*
 * Returns the current scale of the view.
 */
float View::getScale() {
	return _scale;
}

/*
 * Returns the destination scale of the view (where we are zooming to).
 */
float View::getScaleDest() {
	return _scale_dest;
}

/*
 * Set the scale we are zooming to.
 */
void View::setScale(const float d) {
	_scale=d;
	_updated=true;
}

/*
 * Get the int[7] array of the current view location.
 */
int *View::getLocation() {
	return _location;
}

/*
 * Set the int[7] array of the current view location.
 */
void View::setLocation(const int location[7]) {
	for (int i=0;i<7;i++)
		_location[i]=location[i];
	_updated=true;
}

/*
 * Returns the zoom level.  Level 0 is zoomed out to see the whole galaxy.  Level 7 is zoomed
 * in all the way.
 */
int View::getLevel() {
	return _level;
}

/*
 * Sets the view zoom level.
 */
void View::setLevel(int i) {
	_level=i;
	_updated=true;
}

/*
 * Returns true if the view is visually updated.
 */
bool View::isUpdated() {
	return _updated;
}

/*
 * Forces a recalculation of the view.  Used by Information Surface to trick GalaxySurface,
 *   MinimapSurface into thinking the galaxy has changed, thus redrawing it.
 */
void View::setUpdateOverride() {
	_update_override=true;
}

/*
 * Returns true if view update override is set.
 */
bool View::isUpdateOverride() {
	return _update_override;
}

/*
 * resets View update override to false.
 */
void View::resetOverride() {
	_update_override=false;
}

/*
 * Increment time by frame_length.  Gives a standard zoom rate for all fps.
 */
void View::incrementTime() {
	float frame_length = FrameTimer::getFrameLength();
	if ((_level==7)&&(_cursor_star_set)&&(_scale==_scale_dest)&&(_cursor_coords != _cursor_star.getPosition())) {
		Coords delta = _cursor_star.getPosition() - _cursor_coords;
		float dist = delta.getLength();

		if (dist>_scale*4) {
			_cursor_coords = _cursor_star.getPosition();
		} else {
			_cursor_coords  = (delta) * -(pow(10.0,-frame_length /200.0)) + _cursor_star.getPosition();
		}
	}

	if (_updated) {
		Coords delta = _offset_dest-_offset;

		cout.precision(10);

		//Scale occasionally getting microscopically larger than scale_dest on slide
		//Must be a power vs scale addition thing.
		if ((_scale_dest!=_scale) && (fabs(1-(_scale_dest/_scale))<0.0001)) {
			_scale=_scale_dest;
		}

		if ((_scale_dest/_scale<1.001)&&(_scale_dest/_scale>0.999)&&(delta.getLength()<(_scale/1000.0))) {
			//Stopped
			_updated=false;
			_scale = _scale_dest;
			_offset= _offset_dest;
			if (_level==7) {
				if (_cursor_star_set) {
					//Use existing star as offset.

					Star cs = Star::sortStars(_cursor_star.getPosition());
					if (!cs.isNullStar()) {
						_cursor_star = cs;
					} else
						_cursor_star_set=false;
				} else {
					//Use sector offset as offset
					Star cs = Star::sortStars(_offset_dest);
					if (!cs.isNullStar()) {
						_cursor_star = cs;
						_cursor_star_set=true;
					} else
						_cursor_star_set=false;
				}
			}
		} else {
			if (_scale<_scale_dest) {
				//Expanding
				_scale *=pow(_scale_dest/_scale,frame_length/200.0);
				if (_scale>_scale_dest)
					_scale=_scale_dest;
			} else if (_scale>_scale_dest) {
				//Shrinking
				_scale /=pow(_scale/_scale_dest,frame_length/200.0);
				if (_scale<_scale_dest)
					_scale=_scale_dest;
			}

			_offset = (_offset-_offset_dest) * (pow(3.65,-frame_length /200.0)) + _offset_dest;
		}
	}
}

/*
 * Calculate a new int[7] location based on existing level and keypad pan direction.
 */
bool View::moveLocation(int location[], const int level, const int direction) {
	if (level==0)
		return false;
	if (direction==1) {
		if (!moveLocation(location, level, 2)) return false;
		if (!moveLocation(location, level, 4)) return false;
	} else if (direction==7) {
		if (!moveLocation(location, level, 8)) return false;
		if (!moveLocation(location, level, 4)) return false;
	} else if (direction==9) {
		if (!moveLocation(location, level, 8)) return false;
		if (!moveLocation(location, level, 6)) return false;
	} else if (direction==3) {
		if (!moveLocation(location, level, 6)) return false;
		if (!moveLocation(location, level, 2)) return false;
	} else if (direction==4) {
		if ((location[level-1]==1)||(location[level-1]==4)||(location[level-1]==7)) {
			location[level-1]+=2;
			if (!moveLocation(location, level-1, 4)) return false;
		} else {
			location[level-1]--;
		}
	} else if (direction==8) {
		if ((location[level-1]==7)||(location[level-1]==8)||(location[level-1]==9)) {
			location[level-1]-=6;
			if (!moveLocation(location, level-1, 8)) return false;
		} else {
			location[level-1]+=3;
		}
	} else if (direction==6) {
		if ((location[level-1]==3)||(location[level-1]==6)||(location[level-1]==9)) {
			location[level-1]-=2;
			if (!moveLocation(location,level-1, 6)) return false;
		} else {
			location[level-1]++;
		}
	} else if (direction==2) {
		if ((location[level-1]==1)||(location[level-1]==2)||(location[level-1]==3)) {
			location[level-1]+=6;
			if (!moveLocation(location, level-1, 2)) return false;
		} else {
			location[level-1]-=3;
		}
	}
	return true;
}

/*
 * Take action based on a mouse movements sent in a KeyPressed object to alter the View.
 */
void View::mousePressed(KeyPressed kp) {
	if (kp.isMousePressed()==0)
		return;

	int old_location[7];
	for (int i=0;i<7;i++)
		old_location[i]=_location[i];

	if (((kp.isMousePressed()==2)||(kp.isMousePressed()==5))&&(_level>0)) {
		//0 Zoom Out
		_cursor_star_set=false;
		_location[_level-1]=0;
		_offset_dest = getOffset(_location, _level-1);
		_scale_dest=100000/pow(3.0,_level-1);

		_level--;
		if (_level<7) {
			Star::removeStars(_level+1);
		}
		_updated=true;
	} else if ((kp.isMousePressed()==1)||(kp.isMousePressed()==4)) {
		int keypad=5;
		int mouse_x = kp.getMouseX();
		int mouse_y = kp.getMouseY();
		if (mouse_x<SCREEN_CENTER*2/3)
			keypad--;
		else if (mouse_x>SCREEN_CENTER*4/3)
			keypad++;
		if (mouse_y<SCREEN_CENTER*2/3)
			keypad+=3;
		else if (mouse_y>SCREEN_CENTER*4/3)
			keypad-=3;

		int dir = kp.getMouseDir();

		if (dir!=5) {
			if (!isZooming()) {
				keypad=dir;
				if (!moveLocation(_location, _level, keypad)) {
					//Failed Move, Aborting...
					for (int i=0;i<7;i++) {
						_location[i]=old_location[i];
					}
					return;
				}
			} else {
				//Do Nothing
				return;
			}
		} else if (_level<7) {
			_location[_level]=keypad;
			_scale_dest=100000/pow(3.0,_level+1);
			_level++;
		} else if (!isMoving()) {
			//Select Cursor Star
			_scale_dest=100000/pow(3.0,_level);
			Coords click_delta = Coords(((mouse_x-SCREEN_CENTER)*_scale_dest/SCREEN_CENTER*2),((mouse_y-SCREEN_CENTER)*_scale_dest/SCREEN_CENTER*2));
			Coords click_pos = View::getOffset()+click_delta;

			Star cs = Star::sortStars(click_pos);
			if (!cs.isNullStar()) {
				if (cs.getPosition()==_cursor_star.getPosition()) {
					//Double Click, attempt to move (ENTER)
					Ship (*my_ship) = Ship::getMyShip();

					Coords delta = _cursor_star.getPosition()-(*my_ship).getShipCoords();
					if (((*my_ship).getShipStar()!=_cursor_star)&&(_cursor_star_set)) {
						Coords delta = _cursor_star.getPosition()-(*my_ship).getShipCoords();
						float fuel_to_jump=delta.getLength()/(*my_ship).getFuelEfficiency();

						if ((*my_ship).getFuel()>fuel_to_jump) {
							(*my_ship).setShipStar(_cursor_star);
							(*my_ship).setShipLocation(_location);
							(*my_ship).queueShipDataMessage();
						}
					} else {
						if (((*my_ship).getShipStar().hasAliens())&&(!(*my_ship).isMoving())) {
							(*my_ship).setVisitingAliens(true);
						}
					}

				} else {
					//New Star, move Cursor
					_cursor_star = cs;
					_cursor_coords = _cursor_star.getPosition();
				}
			} else {
				//Do Nothing...
			}
			return;
		} else {
			//We are moving at level 7.
			return;
		}

		Coords old_offset_dest = _offset_dest;

		if ((keypad==1)||(keypad==2)||(keypad==3)) {
			_offset_dest+=Coords(0,_scale_dest);
		}
		if ((keypad==7)||(keypad==8)||(keypad==9)) {
			_offset_dest+=Coords(0,-_scale_dest);
		}
		if ((keypad==1)||(keypad==4)||(keypad==7)) {
			_offset_dest+=Coords(-_scale_dest,0);
		}
		if ((keypad==3)||(keypad==6)||(keypad==9)) {
			_offset_dest+=Coords(_scale_dest,0);
		}

		if ((fabs(_offset_dest.getX())>50000.0)||(fabs(_offset_dest.getY())>50000.0)) {
			//Undo the changes!
			_offset_dest=old_offset_dest;
			for (int i=0;i<7;i++) {
				if (i<_level)
					_location[i]=old_location[i];
				else
					_location[i]=0;
			}
			return;
		}

		if (dir!=5) {
			Star::removeStars(_level);
		}

		if ((_level<7)||(dir!=5)) {
			Star::addStars(old_location,_location, _level);
		} else {

		}

		_updated=true;

	}

}

/*
 * Take action based on a KeyPressed object to alter the View.
 */
void View::keyPressed(KeyPressed kp) {
	int keypad = kp.getKeyPressed();

	Ship *my_ship = Ship::getMyShip();

	int old_location[7];

	for (int i=0;i<7;i++)
		old_location[i]=_location[i];

	if ((keypad==KEY_0)&&(_level>0)) {
		//0 Zoom Out
		_cursor_star_set=false;
		_location[_level-1]=0;

		_offset_dest = getOffset(_location, _level-1);

		_scale_dest=100000/pow(3.0,_level-1);

		_level--;
		if (_level<7) {
			Star::removeStars(_level+1);
		}
		_updated=true;
	} else if ((keypad>=KEY_1)&&(keypad<=KEY_9)) {
		//1-9 Zoom In
		int direction_index[] = {-1,3,2,1,4,9,0,5,6,7,8,9};
		int alt_direction_index_a[] = {-1,4,3,2,5,9,1,6,7,0,8,9};
		int alt_direction_index_b[] = {-1,2,1,0,3,9,7,4,5,6,8,9};

		if (kp.isCtrlPressed()) {
			if (keypad==KEY_5) {
				return;
			}
		} else {
			if (_level==7) {
				//Ctrl+Left Left before movement causes Babababa bug.  Making sure
				//Cursor keys are ignored until we snap in place should kill the babababa bug.
				//Becky found another Babababa bug here, if the sector has no stars
				//we should not run this.
				if ((!isMoving())&&(Star::getVisibleStars().size()>0)) {
					Star old_cursor_star = _cursor_star;
					_cursor_star = _cursor_star.moveCursorStar(direction_index[keypad]);
					if (_cursor_star == old_cursor_star)
						_cursor_star = _cursor_star.moveCursorStar(alt_direction_index_a[keypad]);
					if (_cursor_star == old_cursor_star)
						_cursor_star = _cursor_star.moveCursorStar(alt_direction_index_b[keypad]);

					return;
				} else {
					//Do nothing, to avoid panning while zooming in.
					return;
				}
			}
		}

		if (kp.isCtrlPressed()) {
			if (!moveLocation(_location, _level, keypad)) {
				//Failed Move, Aborting...
				for (int i=0;i<7;i++) {
					_location[i]=old_location[i];
				}
				return;
			}
		} else if (_level<7) {
			_location[_level]=kp.getKeyPressed();
			_scale_dest=100000/pow(3.0,_level+1);
			_level++;
		}

		Coords old_offset_dest = _offset_dest;

		if ((keypad==1)||(keypad==2)||(keypad==3)) {
			_offset_dest+=Coords(0,_scale_dest);
		}
		if ((keypad==7)||(keypad==8)||(keypad==9)) {
			_offset_dest+=Coords(0,-_scale_dest);
		}
		if ((keypad==1)||(keypad==4)||(keypad==7)) {
			_offset_dest+=Coords(-_scale_dest,0);
		}
		if ((keypad==3)||(keypad==6)||(keypad==9)) {
			_offset_dest+=Coords(_scale_dest,0);
		}

		if ((fabs(_offset_dest.getX())>50000.0)||(fabs(_offset_dest.getY())>50000.0)) {
			//Undo the changes!
			_offset_dest=old_offset_dest;
			for (int i=0;i<7;i++) {
				if (i<_level)
					_location[i]=old_location[i];
				else
					_location[i]=0;
			}
			return;
		}

		if (kp.isCtrlPressed()) {
			Star::removeStars(_level);
		}

		if ((_level<7)||(kp.isCtrlPressed())) {
			Star::addStars(old_location,_location, _level);
		} else {
			_cursor_coords = _cursor_star.getPosition();
		}

		_updated=true;
	} else if (keypad==KEY_HOME) {

		int old_location[7];
		for (int i=0;i<7;i++) {
			old_location[i]=_location[i];
			if (_location[i]!=((*my_ship).getShipLocation())[i]) {
				_location[i]=((*my_ship).getShipLocation())[i];
				_updated=true;
			}
		}

		_level=7;

		_scale_dest=100000/pow(3.0,_level);

		_offset_dest = getOffset((*my_ship).getShipLocation(), _level);

		Star::addStars(old_location,_location,_level);

		_cursor_star = (*my_ship).getShipStar();
		_cursor_star_set=true;
	} else if (keypad==KEY_ENTER) {
		//if (_level==7) {
		if ((_level==7)&&(!isMoving()&&(Star::getVisibleStars().size()>0))) {
			Coords delta = _cursor_star.getPosition()-(*my_ship).getShipCoords();
			//float dist = delta.getLength();
			if (((*my_ship).getShipStar()!=_cursor_star)&&(_cursor_star_set)) {
				Coords delta = _cursor_star.getPosition()-(*my_ship).getShipCoords();
				float fuel_to_jump=delta.getLength()/(*my_ship).getFuelEfficiency();
				if ((*my_ship).getFuel()>fuel_to_jump) {
					(*my_ship).setShipStar(_cursor_star);
					(*my_ship).setShipLocation(_location);
					(*my_ship).queueShipDataMessage();
				}
			} else {
				if (((*my_ship).getShipStar().hasAliens())&&(!(*my_ship).isMoving())) {
					(*my_ship).setVisitingAliens(true);
				}
			}
		}
	} else if (keypad==KEY_CHEAT4) {
		int j=1;
		if (kp.isCtrlPressed())
			j=20;
		for (int i=0;i<j;i++) {
			if (kp.isShiftPressed())
				Information::readdInformation();
			else
				Information::removeInformation();
		}
		_update_override=true;
	}
}

/*
 * Return (x,y) coords of the cursor location.
 */
Coords View::getCursorCoords() {
	return _cursor_coords;
}

/*
 * Set coords of the cursor location.
 */
void View::setCursorCoords(Coords cursor_coords) {
	_cursor_coords = cursor_coords;
}

/*
 * Return Star which cursor is over.
 */
Star View::getCursorStar() {
	return _cursor_star;
}

/*
 * Set star which cursor is over.
 */
void View::setCursorStar(Star cursor_star) {
	_cursor_star = cursor_star;
}

/*
 * Returns true if the view is compeltely zoomed in to level 7.
 */
bool View::isZoomedIn() {
	return ((_level==7)&&(_scale==_scale_dest)&&(_offset==_offset_dest));
}

/*
 * Returns true if the view has completed zooming to level 6.
 */
bool View::isAlmostZoomedIn() {
	return ((_level==6)&&(_scale==_scale_dest)&&(_offset==_offset_dest));
}

/*
 * Returns true if the view is panning (left,right,up,down) or zooming (in,out).
 */
bool View::isMoving() {
	return ((_scale!=_scale_dest)||(_offset!=_offset_dest));
}

/*
 * Returns true if the view is zooming (in or out)
 */
bool View::isZooming() {
	return (_scale!=_scale_dest);
}

